/*
 * Copyright (c) 2020 The BytePlusRTC project authors. All Rights Reserved.
 * @brief BytePlusRTC Audio Device Manager
*/

#pragma once

namespace bytertc {

/** 
 * @hidden
 * @brief  Set id maximum length
 */
const unsigned int MAX_DEVICE_ID_LENGTH = 512;

/** 
 * @type api
 * @region  engine management
 * @brief  Audio & video equipment related information
 */
class IDeviceCollection {
public:
    /** 
     * @hidden
     * @brief  Constructor
     */
    IDeviceCollection() {
    }
    /** 
     * @type api
     * @region engine management
     * @brief Get the number of audio & video devices in the current system
     * @return Number of audio & video devices
     */
    virtual int GetCount() = 0;
    /** 
     * @type api
     * @region Engine management
     * @brief According to the index number, get device information
     * @param  [in] index Device index number, starting from 0, note that it must be less than the return value of GetCount{@link #IDeviceCollection#GetCount}.
     * @param  [out] Device_name device name
     * @param  [out] Device_id device ID
     * @return   <br>
     *         + 0: Success. <br>
     *         +! 0: failure <br>
     */
    virtual int GetDevice(int index, char device_name[MAX_DEVICE_ID_LENGTH], char device_id[MAX_DEVICE_ID_LENGTH]) = 0;
    /** 
     * @type api
     * @region  engine management
     * @brief  Releases the resources occupied by the current IDeviceCollection{@link #IDeviceCollection} object.
     * @notes This method should be called to release related resources when you do not need to return a list of audio & video device related information.
     */
    virtual void Release() = 0;
/**
 * @hidden
 */
protected:
    /** 
     * @hidden
     * @brief  Destructor
     */
    virtual ~IDeviceCollection() {
    }
};

/** 
 * @type api
 * @brief Audio Facility Management
 */
class IAudioDeviceManager {
public:
    /** 
     * @hidden
     * @brief  Constructor
     */
    IAudioDeviceManager() {
    }
    /** 
     * @type api
     * @region  Audio device management
     * @brief  Gets a list of the audio playback device. When the audio playback device changes, you will receive the `OnMediaDeviceStateChanged` callback, and you need to call this API again to get the new device list.
     * @return  A list of all audio playback devices. See IDeviceCollection{@link #IDeviceCollection}.
     */
    virtual IDeviceCollection* EnumerateAudioPlaybackDevices() = 0;
    /** 
     * @type api
     * @region  Audio Facility Management
     * @brief  Get a list of audio acquisition devices in the current system. If there are subsequent device changes, you need to call this interface again to get a new device list.
     * @return An object that contains a list of all audio capture devices in the system. See IDeviceCollection{@link #IDeviceCollection}.
     * @notes When the returned object is no longer used, you need to call Release{@link #IDeviceCollection#Release} to release it.
     */
    virtual IDeviceCollection* EnumerateAudioCaptureDevices() = 0;

    /** 
     * @type api
     * @region Audio Facility Management
     * @brief Set the audio playback device to follow the OS default setting or not.
     * @param followed
     *        + true: follow the os default setting. You can not call SetAudioPlaybackDevice{@link #IAudioDeviceManager#SetAudioPlaybackDevice} at the same time.
     *        + false: do not follow the os default setting. You can call SetAudioPlaybackDevice{@link #IAudioDeviceManager#SetAudioPlaybackDevice} to set the audio playback device.
     */
    virtual void FollowSystemPlaybackDevice(bool followed) = 0;

    /** 
     * @type api
     * @region Audio Facility Management
     * @brief Set the audio capture device to follow the OS default setting or not.
     * @param followed
     *        + true: follow the os default setting. You can not call SetAudioCaptureDevice{@link #IAudioDeviceManager#SetAudioCaptureDevice} at the same time.
     *        + false: do not follow the os default setting. You can call SetAudioCaptureDevice{@link #IAudioDeviceManager#SetAudioCaptureDevice} to set the audio capture device.
     */
    virtual void FollowSystemCaptureDevice(bool followed) = 0;
    /** 
     * @type api
     * @region Audio device management
     * @brief Sets the audio playback device.
     * @param  [in] device_id Audio playback device's ID. You can get the ID by calling
     * EnumerateAudioPlaybackDevices{@link #EnumerateAudioPlaybackDevices}.
     * @return    <br>
     *         + 0: Success <br>
     *         + < 0: Failure <br>
     * @notes After you call FollowSystemPlaybackDevice{@link #IAudioDeviceManager#FollowSystemPlaybackDevice} to set the audio playback device to follow the system setting, you cannot call this API to set the audio playback device.
     */
    virtual int SetAudioPlaybackDevice(const char device_id[MAX_DEVICE_ID_LENGTH]) = 0;
    /** 
     * @type api
     * @region Audio Facility Management
     * @brief Set up audio capture devices.
     * @param  [in] device_id Audio capture device ID, available through EnumerateAudioCaptureDevices{@link #EnumerateAudioCaptureDevices}.
     * @return   Method call result   <br>
     *         + 0: Success. <br>
     *         + < 0: failure <br>
     * @notes After you call FollowSystemCaptureDevice{@link #IAudioDeviceManager#FollowSystemCaptureDevice} to set the audio playback device to follow the system setting, you cannot call this API to set the audio capture device.
     */
    virtual int SetAudioCaptureDevice(const char device_id[MAX_DEVICE_ID_LENGTH]) = 0;
    /** 
     * @type api
     * @region  Audio Facility Management
     * @brief Sets the current audio playback device volume
     * @param  [in] volume Audio playback device volume, the value range is [0,255], the setting beyond this range is invalid.
     *        + [0,25] Is nearly silent; <br>
     *        + [25,75] Is low volume; <br>
     *        + [76,204] Is medium volume; <br>
     *        + [205,255] Is high volume. <br>
     * @return   Method call result   <br>
     *         + 0: Success. <br>
     *         + < 0: failure <br>
     */
    virtual int SetAudioPlaybackDeviceVolume(unsigned int volume) = 0;
    /** 
     * @type api
     * @region Audio Facility Management
     * @brief Get the current audio playback device volume
     * @param  [out] Volume Audio playback device volume, the range should be within [0,255].
     *        + [0,25] Is nearly silent; <br>
     *        + [25,75] Is low volume; <br>
     *        + [76,204] Is medium volume; <br>
     *        + [205,255] Is high volume. <br>
     * @return   Method call result   <br>
     *         + 0: Success. <br>
     *         + < 0: failure <br>
     */
    virtual int GetAudioPlaybackDeviceVolume(unsigned int* volume) = 0;
    /** 
     * @type api
     * @region  Audio Facility Management
     * @brief Set the current audio acquisition device volume
     * @param  [in] volume Audio acquisition device volume, the value range is [0,255], the setting beyond this range is invalid.
     *        + [0,25] Is nearly silent; <br>
     *        + [25,75] Is low volume; <br>
     *        + [76,204] Is medium volume; <br>
     *        + [205,255] Is high volume. <br>
     * @return   Method call result   <br>
     *         + 0: Success. <br>
     *         + < 0: failure <br>
     */
    virtual int SetAudioCaptureDeviceVolume(unsigned int volume) = 0;
    /** 
     * @type api
     * @region Audio Facility Management
     * @brief Get the current audio capture device volume
     * @param  [out] Volume Audio capture device volume, the range should be within [0,255].
     *        + [0,25] Is nearly silent; <br>
     *        + [25,75] Is low volume; <br>
     *        + [76,204] Is medium volume; <br>
     *        + [205,255] Is high volume. <br>
     * @return   Method call result   <br>
     *         + 0: Success. <br>
     *         + < 0: failure <br>
     */
    virtual int GetAudioCaptureDeviceVolume(unsigned int* volume) = 0;
    /** 
     * @type api
     * @region Audio Facility Management
     * @brief Sets the current audio playback device to be mute, and the default is non-mute.
     * @param  [in] mute <br>
     *        + True: mute <br>
     *        + False: non-mute <br>
     * @return   Method call result   <br>
     *         + 0: Success. <br>
     *         + < 0: failure <br>
     */
    virtual int SetAudioPlaybackDeviceMute(bool mute) = 0;
    /** 
     * @type api
     * @region Audio Facility Management
     * @brief Gets information about whether the current audio playback device is muted.
     * @param  [out] Mute <br>
     *        + True: mute <br>
     *        + False: non-mute <br>
     * @return   Method call result   <br>
     *         + 0: Success. <br>
     *         + < 0: failure <br>
     */
    virtual int GetAudioPlaybackDeviceMute(bool* mute) = 0;
    /** 
     * @type api
     * @region Audio Facility Management
     * @brief Sets the current audio acquisition device to be mute, and the default is non-mute.
     * @param  [in] mute <br>
     *        + True: mute <br>
     *        + False: non-mute <br>
     * @return   Method call result   <br>
     *         + 0: Success. <br>
     *         + < 0: failure <br>
     */
    virtual int SetAudioCaptureDeviceMute(bool mute) = 0;
    /** 
     * @type api
     * @region Audio Facility Management
     * @brief Gets information about whether the current audio capture device is silent.
     * @param  [out] Mute <br>
     *        + True: mute <br>
     *        + False: non-mute <br>
     * @return   Method call result   <br>
     *         + 0: Success. <br>
     *         + < 0: failure <br>
     */
    virtual int GetAudioCaptureDeviceMute(bool* mute) = 0;
    /** 
     * @type api
     * @region Audio Facility Management
     * @brief Gets the current audio playback device ID.
     * @param  [out] Device_id audio playback device ID
     * @return   Method call result   <br>
     *         + 0: Success. <br>
     *         + < 0: failure <br>
     */
    virtual int GetAudioPlaybackDevice(char device_id[MAX_DEVICE_ID_LENGTH]) = 0;
    /** 
     * @type api
     * @region Audio Facility Management
     * @brief Gets the current audio capture device ID.
     * @param  [out] Device_id audio capture device ID, the user is responsible for allocating and freeing memory according to the MAX_DEVICE_ID_LENGTH size
     * @return   Method call result   <br>
     *         + 0: Success. <br>
     *         + < 0: failure <br>
     */
    virtual int GetAudioCaptureDevice(char device_id[MAX_DEVICE_ID_LENGTH]) = 0;
    /** 
     * @type api
     * @region Audio Facility Management
     * @brief Start the audio playback device test.   <br>
     *        This method tests whether the playback device works properly. The SDK plays the specified audio file. If the tester can hear the sound, the playback device can work normally.
     * @param  [in] test_audio_file_path The absolute path of the audio file. The path string uses UTF-8 encoding format and supports the following audio formats: mp3, aac, m4a, 3gp, wav.
     * @return  Method call result   <br>
     *         + 0: Success. <br>
     *         + < 0: failure <br>
     * @notes   <br>
     *        + This method must be called before joining the room and cannot be applied at the same time as other audio device testing functions. <br>
     *        + You need to call StopAudioPlaybackDeviceTest{@link #IAudioDeviceManager#StopAudioPlaybackDeviceTest} to stop the test. <br>
     */
    virtual int StartAudioPlaybackDeviceTest(const char* test_audio_file_path) = 0;
    /** 
     * @type api
     * @region Audio Facility Management
     * @brief Stop audio playback device testing.
     * @return  Method call result   <br>
     *         + 0: Success. <br>
     *         + < 0: failure <br>
     * @notes  After calling StartAudioPlaybackDeviceTest{@link #IAudioDeviceManager#StartAudioPlaybackDeviceTest}, you need to call this method to stop the test.
     */
    virtual int StopAudioPlaybackDeviceTest() = 0;
    /** 
     * @type api
     * @region Audio Facility Management
     * @brief Start the audio capture device test.   <br>
     *        This method tests whether the audio acquisition device can work properly. After starting the test, you will receive the volume information reported by the `OnLocalAudioPropertiesReport` callback.
     * @param  [in] indication_interval The interval(ms) in `OnLocalAudioPropertiesReport` callback. It is recommended to set to greater than 200 milliseconds. The minimum shall not be less than 10 milliseconds. Less than 10 milliseconds behavior is not defined.
     * @return  Method call result   <br>
     *         + 0: Success. <br>
     *         + < 0: failure <br>
     * @notes   <br>
     *        + This method must be called before joining the room and cannot be applied at the same time as other audio device testing functions. <br>
     *        + You need to call StopAudioCaptureDeviceTest{@link #IAudioDeviceManager#StopAudioCaptureDeviceTest} to stop the test. <br>
     */
    virtual int StartAudioCaptureDeviceTest(int indication_interval) = 0;
    /** 
     * @type api
     * @region Audio Facility Management
     * @brief Stop audio capture device testing.
     * @return  Method call result   <br>
     *         + 0: Success. <br>
     *         + < 0: failure <br>
     * @notes  After calling StartAudioCaptureDeviceTest{@link #IAudioDeviceManager#StartAudioCaptureDeviceTest}, you need to call this method to stop the test.
     */
    virtual int StopAudioCaptureDeviceTest() = 0;
    /** 
     * @type api
     * @region Audio Facility Management
     * @brief Begins audio device loop testing.   <br>
     *        This method tests whether the audio acquisition device and the audio playback device can work normally. Once the test starts, the audio capture device will capture the local sound
     *         And play it out through the audio playback device, and the user App will receive the volume information reported by the `OnLocalAudioPropertiesReport` callback.
     * @param  [in] indication_interval The time interval(ms) in `OnLocalAudioPropertiesReport`. It is recommended to set to greater than 200 milliseconds. The minimum shall not be less than 10 milliseconds. Less than 10 milliseconds behavior is not defined.
     * @return   Method call result   <br>
     *         + 0: Success. <br>
     *         + < 0: failure <br>
     * @notes   <br>
     *        + This method must be called before joining the room. And cannot be applied at the same time as other audio equipment test functions. <br>
     *        + You need to call StopAudioDeviceLoopbackTest{@link #IAudioDeviceManager#StopAudioDeviceLoopbackTest} to stop the test. <br>
     *        + This method only tests audio equipment locally and does not involve network connection. <br>
     */
    virtual int StartAudioDeviceLoopbackTest(int indication_interval) = 0;
    /** 
     * @type api
     * @region Audio Facility Management
     * @brief Stop audio device loop testing.
     * @return  Method call result   <br>
     *         + 0: Success. <br>
     *         + < 0: failure <br>
     * @notes After calling StartAudioDeviceLoopbackTest{@link #IAudioDeviceManager#StartAudioDeviceLoopbackTest}, you need to call this method to stop the test.
     */
    virtual int StopAudioDeviceLoopbackTest() = 0;
    /** 
     * @type api
     * @region Audio Facility Management
     * @brief Try to initialize the audio playback device, and you can detect abnormal problems such as the device does not exist, permissions are denied/disabled, etc.
     * @param  [in] index Device index number
     * @return Device status error code
     *         + 0: device detection result is normal
     *         + -1: The interface status is incorrect, for example, the interface is called for detection after the acquisition is started normally
     *         + -2: The acquisition device does not have microphone permission, and attempts to initialize the device fail
     *         + -3: The device does not exist, there is currently no device or return when the device is removed
     *         + -4: Device audio format not supported
     *         + -5: Error for other reasons
     * @notes 1. This interface needs to be called before entering the room; <br>
     *        2. Successful detection does not necessarily mean that the device can be started successfully. It may also fail to start due to the device being monopolized by other application processes, or CPU/memory shortage.
     */
    virtual int InitAudioPlaybackDeviceForTest(const char deviceId[MAX_DEVICE_ID_LENGTH]) = 0;
    /** 
     * @type api
     * @region Audio Facility Management
     * @brief Try to initialize the audio capture device, which can detect abnormal problems such as the device does not exist, permissions are denied/disabled, etc.
     * @param  [in] index Device index
     * @return Device status error code
     *         + 0: device detection result is normal
     *         + -1: The interface status is incorrect, for example, the interface is called for detection after the acquisition is started normally
     *         + -2: The acquisition device does not have microphone permission, and attempts to initialize the device fail
     *         + -3: The device does not exist, there is currently no device or return when the device is removed
     *         + -4: Device audio format not supported
     *         + -5: Error for other reasons
     * @notes 1. This interface needs to be called before entering the room; <br>
     *        2. Successful detection does not necessarily mean that the device can be started successfully. It may also fail to start due to the device being monopolized by other application processes, or CPU/memory shortage.
     */
    virtual int InitAudioCaptureDeviceForTest(const char deviceId[MAX_DEVICE_ID_LENGTH]) = 0;

    /** 
     * @hidden
     * @brief  Destructor
     */
    virtual ~IAudioDeviceManager() {
    }
};

}  // namespace bytertc
