/*
 * Copyright (c) 2020 The BytePlusRTC project authors. All Rights Reserved.
 * @brief BytePlusRTC Interface
 */

#pragma once

#ifndef BYTE_RTC_INTERFACE_H__
#define BYTE_RTC_INTERFACE_H__

#include <stddef.h>
#include <stdio.h>
#include <stdarg.h>
#include <memory>
#include "bytertc_defines.h"
#include "bytertc_engine_lite_interface.h"
#include "bytertc_engine_event_handler.h"
#include "bytertc_video_frame.h"
#include "bytertc_audio_frame.h"

namespace bytertc {

class IRTCRoomEventHandler;
/** 
 * @type api
 * @brief Engine API
 *      IRtcEngine uses IRtcEngineering EventHandler to register interface class methods to send callback notifications to applications
 */
class IRtcEngine : public IRtcRoom,
                   public IRtcEngineLite {
public:

    /** 
     * @hidden
     * @brief  Destructor
     */
    virtual ~IRtcEngine() {
    }
                       
                       

    /** 
     * @hidden
     * @deprecated since 326.1, use function joinRoom with RTCRoomConfig parameter instead
     * @type api
     * @region Room management
     * @brief Create/join a room: create a room when the room does not exist; join the room when the room exists.   <br>
     *        Users in the same room can talk to each other. <br>
     *         Repeated calls after entering the room are invalid. The user must call LeaveRoom{@link #LeaveRoom} to exit the current room before joining the next room. <br>
     *         Local users will receive OnRoomStateChanged{@link #IRTCRoomEventHandler#OnRoomStateChanged} callback notification after calling this method to join the room successfully, and remote users will receive OnUserJoined{@link #IRTCRoomEventHandler#OnUserJoined} callback notification. <br>
     * @param  [in] token Dynamic key for authenticating the logged-in user. <br>
     *         You need to bring Token to enter the room. When testing, you can use the console to generate temporary tokens. The official launch requires the use of the key SDK to generate and issue tokens at your server level. <br>
     * @param  [in] room_id Added room ID. <br>
     *        The room ID is a non-empty string with a length of less than 128 bytes, and the following character set ranges are supported: <br>
     *            1. 26 capital letters A~ Z. <br>
     *            2. 26 lowercase letters a~ z. <br>
     *            3. 10 numbers 0~ 9. <br>
     *            4. Underline "_", at character "@", minus sign "-". <br>
     * @param  [in] user_info User information. See UserInfo{@link #UserInfo}. <br>
     * @param  [in] profile_type Room mode. <br>
     *        You should choose the right room mode according to your business needs. <br>
     *        You must set the room mode before entering the room, and it cannot be changed after entering the room. <br>
     *         For room patterns. See RoomProfileType{@link #RoomProfileType}. <br>
     * @return   <br>
     *         + 0: Success <br>
     *         + -1: Room_id is empty, failure <br>
     *         + -2: User_info is empty, failure <br>
     * @notes   <br>
     *        + Apps with different App IDs cannot communicate with each other. <br>
     *        + Make sure that the App ID used to generate the Token is the same as the App ID used to create the engine, otherwise it will cause the join room to fail. <br>
     *        + After the user successfully joins the room, the SDK may lose connection to the server in case of poor local network conditions, and the SDK will automatically reconnect at this time. After a successful reconnection, you will receive an OnRoomStateChanged{@link #IRTCRoomEventHandler#OnRoomStateChanged} callback notification locally. <br>
     *        + In the same room with the same App ID, the user ID of each user must be unique. If two users have the same user ID, the user who entered the room later will kick the user who entered the room out of the room, and the user who entered the room will receive the OnError{@link #IRtcEngineLiteEventHandler#OnError} callback notification. See ErrorCode{@link #ErrorCode} for the error type. <br>
     */
    virtual int JoinRoom(
            const char* token, const char* room_id, const UserInfo& user_info, RoomProfileType profile_type) = 0;
    /** 
      * @type api
      * @region Room management
      * @brief Create or join a room: create a room when the room does not exist; join the room when the room exists.   <br>
      *        Users in the same room can talk to each other. <br>
      *         Repeated calls of the API after entering the room are invalid. The user must call LeaveRoom{@link #IRtcRoom#LeaveRoom} to exit the current room before joining the next room. <br>
      * @param [in] token Dynamic key for authentication of users entering the room. <br>
      *         You must use token to join the room. When debugging, you can use the temporary tokens generated on the console. For official launch, you must generate token on your application server and use the tokens to join the room.<br>
      *         Make sure that the AppID used to generate the token is the same as that used to create the engine, otherwise it will fail to join room. <br>
      * @param [in] room_id  The ID of the room to join. <br>
      *        The string matches the regular expression: `[a-zA-Z0-9_@\-]{1,128}`.
      * @param [in] user_info See UserInfo{@link #UserInfo}. <br>
      * @param [in] room_config  Sets the room mode and whether to automatically publish or subscribe to the stream. See RTCRoomConfig{@link #RTCRoomConfig}. <br>
      * @return   <br>
      *         + 0: Success. <br>
      *         + -1: Failure. There's invalid parameters in RoomID or userInfo.uid. <br>
      *         + -2: Already in the room. When calling the API call， receiving the value of 0 and not calling LeaveRoom:{@link #IRtcRoom#LeaveRoom},  value `-2` is returned for further API calls, regardless of whether the roomId and the userInfo are duplicated. <br>
      * @notes   <br>
      *        + In the same room with the same App ID, the user ID of each user must be unique. If two users use the same user ID, the former will be kicked out of the room by the latter and receive OnError{@link #IRtcEngineLiteEventHandler#OnError}.<br>
      *        + You will receive OnRoomStateChanged{@link #IRTCRoomEventHandler#OnRoomStateChanged} after joining the room successfully. <br>
      *        + When you call SetUserVisibility{@link #IRtcRoom#SetUserVisibility} to make yourself visible and join the room, the remote users will receive OnUserJoined{@link #IRTCRoomEventHandler#OnUserJoined}. <br>
      *        + After joining the room, you may lose connection to the server due to poor network conditions. In this case, the SDK will automatically reconnect. After reconnected, you will receive OnRoomStateChanged{@link #IRTCRoomEventHandler#OnRoomStateChanged}. If you are visible in the room, the remote users will receive OnUserJoined{@link #IRTCRoomEventHandler#OnUserJoined}. <br>
      */
    virtual int JoinRoom(const char* token, const char* room_id, const UserInfo& user_info, const RTCRoomConfig& room_config) = 0;
    /**
     * @hidden
     */
    virtual void PublishStream(MediaStreamType type) override {};
    /**
     * @hidden
     */                                          
    virtual void UnpublishStream(MediaStreamType type) override {};
/**
 * @hidden
 */
private:
    /**
     * @hidden
     */
    void JoinRoom(const char* token, const UserInfo& user_info, RoomProfileType profile_type) override{};
    /**
     * @hidden
     */
    int JoinRoom(const char* token, const UserInfo& user_info, const MultiRoomConfig& config) override{ return 0;};
    /**
     * @hidden
     */
    void SetRtcRoomEventHandler(IRTCRoomEventHandler* room_event_handler) override {};
                       

};

/** 
 * @type api
 * @region Engine Management
 * @brief Create an engine instance.   <br>
 *        This is the very first API that you must call if you want to use all the RTC capabilities.  <br>
 *        If there is no engine instance in current thread, calling this API will create one. If an engine instance has been created, calling this API again will create another instance.
 * @param [in] app_id A unique identifier for each App, randomly generated by the RTC console. Only instances created with the same app_id are able to communicate with each other.
 * @param [in] event_handler Handler sent from SDK to App. See IRtcEngineEventHandler{@link #IRtcEngineEventHandler}.
 * @param [in] parameters Reserved parameter, in JSON string format.
 * @return  <br>
 *        + IRtcEngine：A successfully created engine instance  <br>
 *        + Null：Creation failed
 * @notes   <br>
 *         + If you create multiple engine instances in the same thread, you must destroy each of these instances by calling DestroyRtcEngine{@link #DestroyRtcEngine}.
 *         + The number of engine instances created on a Linux platform depends on the hardware performance of the system.
 */
BYTERTC_API bytertc::IRtcEngine* CreateRtcEngine(const char* app_id,
        bytertc::IRtcEngineEventHandler* event_handler, const char* parameters);

/** 
 * @hidden
 * @region Engine Management
 * @brief Create a game RTCEngine instance.
 * @param  [in] app_id <br>
 *         A unique device identifier for each application. Instances generated by different AppIDs are completely independent for audio & video calls and cannot communicate with each other.
 * @param  [in] event_handler <br>
 *        SDK callback to the Callback object at the application layer. See IRtcEngineEventHandler{@link #IRtcEngineEventHandler}.
 * @param  [in] parameters Preserve the parameters
 * @return  Available IRtcEngine{@link #IRtcEngine} instances.
 * @notes   <br>
 *         + This method creates and initializes an IRtcEngine{@link #IRtcEngine} instance. With IRtcEngine, you must first call the interface for initialization. The <br>
 *         + IRtcEngine{@link #IRtcEngine} instance notifies the application engine runtime event via the specified IRtcEngineEventHandler{@link #IRtcEngineEventHandler}
 *          . All methods defined in IRtcEngineEventHandler{@link #IRtcEngineEventHandler} are optional implementations.
 */
BYTERTC_API bytertc::IRtcEngine* CreateGameRtcEngine(const char* app_id,
        bytertc::IRtcEngineEventHandler* event_handler, const char* parameters);

/** 
 * @hidden
 * @region Engine management
 * @brief Create RTCEngine instances.
 * @param  [in] app_id <br>
 *         A unique device identifier for each application. Instances generated by different AppIDs are completely independent for audio & video calls and cannot communicate with each other.
 * @param  [in] event_handler <br>
 *        SDK callback to the Callback object at the application layer. See IRtcEngineEventHandler{@link #IRtcEngineEventHandler}.
 * @param  [in] parameters Preserve the parameters
 * @return  Available IRtcEngine{@link #IRtcEngine} instances.
 * @notes   <br>
 *         + This method creates and initializes an IRtcEngine{@link #IRtcEngine} instance. With IRtcEngine, you must first call the interface for initialization. The <br>
 *         + IRtcEngine{@link #IRtcEngine} instance notifies the application engine runtime event via the specified IRtcEngineEventHandler{@link #IRtcEngineEventHandler}
 *          . All methods defined in IRtcEngineEventHandler{@link #IRtcEngineEventHandler} are optional implementations.
 */
BYTERTC_API bytertc::IRtcEngine* CreateRtcEngineWithPtr(
        const char* app_id, std::unique_ptr<bytertc::IRtcEngineEventHandler> event_handler,
        const char* parameters);

/** 
 * @hidden
 * @region Engine Management
 * @brief Create a game RTCEngine instance.
 * @param  [in] app_id <br>
 *         A unique device identifier for each application. Instances generated by different AppIDs are completely independent for audio & video calls and cannot communicate with each other.
 * @param  [in] event_handler <br>
 *        SDK callback to the Callback object at the application layer. See IRtcEngineEventHandler{@link #IRtcEngineEventHandler}.
 * @param  [in] parameters Preserve the parameters
 * @return  Available IRtcEngine{@link #IRtcEngine} instances.
 * @notes   <br>
 *         + This method creates and initializes an IRtcEngine{@link #IRtcEngine} instance. With IRtcEngine, you must first call the interface for initialization. The <br>
 *         + IRtcEngine{@link #IRtcEngine} instance notifies the application engine runtime event via the specified IRtcEngineEventHandler{@link #IRtcEngineEventHandler}
 *          . All methods defined in IRtcEngineEventHandler{@link #IRtcEngineEventHandler} are optional implementations.
 */
BYTERTC_API bytertc::IRtcEngine* CreateGameRtcEngineWithPtr(
        const char* app_id, std::unique_ptr<bytertc::IRtcEngineEventHandler> event_handler,
        const char* parameters);

/** 
 * @type api
 * @region Engine Management
 * @brief Destroy the engine instance created by CreateRtcEngine{@link #CreateRtcEngine}, and release all related resources.
 * @param [in] engine The engine instance created by CreateRtcEngine{@link #CreateRtcEngine}
 * @notes  <br>
 *         + Call this API after all business scenarios related to the engine instance are destroyed.  <br>
 *         + When the API is called, RTC SDK destroys all memory associated with the engine instance and stops any interaction with the media server.  <br>
 *         + Calling this API will start the SDK exit logic. The engine thread is held until the exit logic is complete. The engine thread is retained until the exit logic is complete.  <br>
 *           Therefore, do not call this API directly in the callback thread, or wait for the execution of the main thread in the callback and call this API in the main thread at the same time. Otherwise, it will cause a deadlock.
 */    
BYTERTC_API void DestroyRtcEngine(bytertc::IRtcEngine* engine);

/** 
 * @hidden
 * @region Engine Management
 * @brief Destroy the game RTCEngine instance created by CreateGameRtcEngine{@link #CreateGameRtcEngine} and release all related resources.
 * @param  [in] engine <br>
 *         The instance returned by CreateGameRtcEngine{@link #CreateGameRtcEngine}.
 * @notes   <br>
 *         + Make sure that this method is called in the last stage of all business scenarios. <br>
 *         + After this method is called, it will destroy all SDK-related memory and stop any interaction with the media server. <br>
 *         + Any settings made to the SDK after the last CreateGameRtcEngine{@link #CreateGameRtcEngine} call,
 *           Will take effect the next time CreateGameRtcEngine{@link #CreateGameRtcEngine} is effectively executed. <br>
 *         + This method is a blocking call, which will block the current thread until the SDK completes the exit logic completely. Therefore, care should be taken not to call this method directly in the callback thread, and care should also be taken not to wait for the main thread in the callback method. Execution while calling this method in the main thread, thus causing a deadlock.
 */
BYTERTC_API void DestroyGameRtcEngine(bytertc::IRtcEngine* engine);

/** 
 * @type api
 * @region  error code
 * @brief  Get the description of the error code
 * @param  [in] code Needs to get the description of the error code
 * @return  The description of the error code
 * @notes  This interface is a general function and does not need to rely on the engine object when calling.
 */
BYTERTC_API const char* GetErrorDescription(int code);

/** 
 * @type api
 * @region  engine management
 * @brief  Get the current SDK version information.
 * @return Current SDK version information.
 */
BYTERTC_API const char* GetSDKVersion();

}    // namespace bytertc

#endif  // BYTE_RTC_INTERFACE_H__
