/*
 * Copyright (c) 2020 The BytePlusRTC project authors. All Rights Reserved.
 * @brief BytePlusRTC Interface Lite
 */

#pragma once

#ifndef BYTE_RTC_LITE_INTERFACE_H__
#define BYTE_RTC_LITE_INTERFACE_H__

#include "bytertc_room_interface.h"
#include "bytertc_video_device_manager.h"
#include "bytertc_audio_frame.h"
#include "byte_rtc_asr_engine_event_handler.h"
#include "bytertc_audio_mixing_manager.h"
#include "bytertc_video_processor_interface.h"
#include "bytertc_camera_control_interface.h"

namespace bytertc {

/** 
 * @type callback
 * @brief  Custom coded frame callback class
 */
class IExternalVideoEncoderEventHandler {
public:
    /** 
     * @hidden
     * @brief  Destructor
     */
    virtual ~IExternalVideoEncoderEventHandler(){}
    /** 
     * @type callback
     * @region Video management
     * @brief Prompt custom encoded frames to start pushing callbacks.   <br>
     *         After receiving this callback, you can call PushExternalEncodedVideoFrame{@link #IRtcEngineLite#PushExternalEncodedVideoFrame} to push a custom encoded video frame to the SDK
     * @param  [in] index The properties of the encoded stream that can be pushed. See StreamIndex{@link #StreamIndex}
     */
    virtual void OnStart(StreamIndex index) = 0;
    /** 
     * @type callback
     * @region Video management
     * @brief When you receive the callback, you need to stop pushing custom encoded video frames to the SDK
     * @param  [in] index The properties of the encoded stream that you need to stop pushing. See StreamIndex{@link #StreamIndex}
     */
    virtual void OnStop(StreamIndex index) = 0;
    /** 
     * @type callback
     * @region Video management
     * @brief When the frame rate or bit rate of a custom encoded stream changes, the callback is triggered
     * @param  [in] index The Properties of the encoded stream with the changed index. See StreamIndex{@link #StreamIndex}
     * @param  [in] video_index The subscript of the corresponding encoded stream
     * @param  [in] info The encoded stream information after the change. See VideoRateInfo{@link #VideoRateInfo}
     */
    virtual void OnRateUpdate(StreamIndex index, int32_t video_index, VideoRateInfo info) = 0;
    /** 
     * @type callback
     * @region Video management
     * @brief This callback is used to inform the stream publisher that a keyframe needs to be regenerated.
     * @param [in] index Properties of the remote encoded stream. See StreamIndex{@link #StreamIndex}
     * @param [in] video_index The subscript of the corresponding encoded stream
     */
    virtual void OnRequestKeyFrame(StreamIndex index, int32_t video_index) = 0;
};

/** 
 * @type callback
 * @region Video Data Callback
 * @brief Local video frame observer
 */
class ILocalEncodedVideoFrameObserver {
public:
    /** 
     * @hidden
     * @brief Destructor
     */
    virtual ~ILocalEncodedVideoFrameObserver() {
    }
    /** 
     * @type callback
     * @region Video Data Callback
     * @brief After calling RegisterLocalEncodedVideoFrameObserver{@link #IRtcEngineLite#RegisterLocalEncodedVideoFrameObserver}, SDK will trigger this callback when it receives local video frames.
     * @param [in] type Type of the local video frame. See StreamIndex{@link #StreamIndex}.
     * @param [in] video_stream Information about the local video frame. See IEncodedVideoFrame{@link #IEncodedVideoFrame}.
     */
    virtual void OnLocalEncodedVideoFrame(StreamIndex type, const IEncodedVideoFrame& video_stream) = 0;
};

/** 
 * @type callback
 * @region  video management
 * @brief  Remote encoded video data monitor
 */
class IRemoteEncodedVideoFrameObserver {
public:
    /** 
     * @hidden
     * @brief  Destructor
     */
    virtual ~IRemoteEncodedVideoFrameObserver() {
    }
    /** 
     * @type callback
     * @region  video data callback
     * @brief  Call RegisterRemoteEncodedVideoFrameObserver{@link #IRtcEngineLite#RegisterRemoteEncodedVideoFrameObserver}, the callback is triggered when the SDK detects the remote encoded video data
     * @param  [in] stream_info The received remote stream information. See RemoteStreamKey{@link #RemoteStreamKey}
     * @param  [in] video_stream The received remote video frame information. See IEncodedVideoFrame{@link #IEncodedVideoFrame}
     */
    virtual void OnRemoteEncodedVideoFrame(const RemoteStreamKey& stream_info, const IEncodedVideoFrame& video_stream) = 0;
};

/** 
 * @type keytype
 * @brief The metadata information contained in the video frame
 */
struct VideoMetadataBuffer {
    /** 
     * @brief Received or sent metadata
     */
    char* data;
    /** 
     * @brief The size of metadata data received or sent cannot exceed 1024
     */
    int size;
    /** 
     * @brief A timestamp containing metadata video frames in microseconds
     */
    int64_t timestamp_us;
};

/** 
 * @hidden
 * @deprecated since 326.1, use SendSEIMessage instead
 * @type callback
 * @region video data callback
 * @brief Metadata observer, you can receive metadata in the media stream, or add metadata to the media stream
 */
class IMetadataObserver {
public:
    /**
     * @hidden
     */
    virtual ~IMetadataObserver() {
    }

    /** 
     * @type callback
     * @region Video data callback
     * @brief When the SDK is ready to send a video frame, the event is called back to determine whether metadata needs to be added to the video frame.
     * @param  [in/out] Metadata The data to be sent, copy the data into the data field, and set the size to the real size.
     * @return   <br>
     *         + True: You need to add metadata <br>
     *         + False: You don't need to add metadata <br>
     * @notes   <br>
     *         + The size of metadata cannot exceed 1024 bytes <br>
     *         + The timestamp_us in metadata is the input field, which represents the timestamp of the video frame. It is used for synchronization and does not need to be modified .. <br>
     *         + There must be no time-consuming operations in the callback to avoid affecting the video card <br>
     */
    virtual bool OnReadyToSendMetadata(VideoMetadataBuffer* metadata) = 0;

    /** 
     * @type callback
     * @region Video data callback
     * @brief When a video frame is received by the SDK and contains a medatada, the event is called back.
     * @param  [in] roomid The room ID to which the current frame belongs.
     * @param  [in] uid The user ID to which the current frame belongs.
     * @param  [in] metadata The metadata information contained in the video frame. See VideoMetadataBuffer{@link #VideoMetadataBuffer}.
     * @notes There must be no time-consuming operations in the callback so as not to affect the video card.
     */
    virtual void OnMetadataReceived(const char* roomid, const char* uid, const VideoMetadataBuffer& metadata) = 0;
};

/** 
 * @type callback
 * @region Video Management
 * @brief Video data callback observer
 */
class IVideoFrameObserver {
public:
    /** 
     * @hidden
     * @brief Destructor
     */
    virtual ~IVideoFrameObserver() = default;

    /** 
     * @type callback
     * @region Video Management
     * @brief Gets the successfully captured local screen video frames for custom processing or rendering.
     * @param [in] videoFrame Video data. See IVideoFrame{@link #IVideoFrame}.
     */
    virtual bool OnLocalScreenFrame(IVideoFrame* videoFrame) = 0;

    /** 
     * @type callback
     * @region Video Management
     * @brief Geta the successfully captured local camera stream for custom processing or rendering.
     * @param [in] videoFrame Video data. See IVideoFrame{@link #IVideoFrame}.
     */
    virtual bool OnLocalVideoFrame(IVideoFrame* videoFrame) = 0;

    /** 
     * @type callback
     * @region Video Management
     * @brief Gets the successfully captured remote screen video frames for custom processing or rendering.
     * @param [in] roomid The ID of the room from which the video is streamed.
     * @param [in] uid The ID of the remote user who published the stream.
     * @param [in] videoFrame Video data. See IVideoFrame{@link #IVideoFrame}.
     */
    virtual bool OnRemoteScreenFrame(const char* roomid, const char* uid, IVideoFrame* videoFrame) = 0;

    /** 
     * @type callback
     * @region Video Management
     * @brief Get the successfully captured remote camera stream for custom processing or rendering.
     * @param [in] roomid The ID of the room from which the video is streamed.
     * @param [in] uid The ID of the remote user who published the stream.
     * @param [in] videoFrame Video data. See IVideoFrame{@link #IVideoFrame}.
     */
    virtual bool OnRemoteVideoFrame(const char* roomid, const char* uid, IVideoFrame* videoFrame) = 0;

    /** 
     * @hidden
     * @type callback
     * @region video management
     * @brief Callback carrying splicing video data
     * @param [in] roomid The ID of the room from which the video is streamed.
     * @param [in] uid The ID of the remote user who published the stream.
     * @param [in] videoFrame Video data. See IVideoFrame{@link #IVideoFrame}.
     */
    virtual bool OnMergeFrame(const char* roomid, const char* uid, IVideoFrame* videoFrame) {
        return false;
    }
};

/** 
 * @hidden
 * @type callback
 * @region audio data callback
 * @brief Audio data callback observer
 */
class IRemoteAudioFrameObserver {
public:
    /** 
     * @hidden
     * @brief  Destructor
     */
    virtual ~IRemoteAudioFrameObserver() {
    }

    /** 
     * @hidden
     * @type callback
     * @region Audio data callback
     * @brief Get audio data for a single stream, this callback is triggered by calling RegisterRemoteAudioFrameObserver {@link
     *  #RegisterRemoteAudioFrameObserver}.
     * @param  [in] audio_frame Audio data, see: IAudioFrame{@link #IAudioFrame}
     * @param  [in] stream_info The audio stream business information, see: RemoteStreamKey{@link #RemoteStreamKey}
     */
    virtual void OnRemoteAudioFrame(const IAudioFrame& audio_frame, const RemoteStreamKey& stream_info) = 0;
};

/** 
 * @type keytype
 * @brief Type of the screen capture object
 */
enum ScreenCaptureSourceType {
    /** 
     * @brief Type unknown
     */
    kScreenCaptureSourceTypeUnknown,

    /** 
     * @brief Application window
     */
    kScreenCaptureSourceTypeWindow,

    /** 
     * @brief Desktop
     */
    kScreenCaptureSourceTypeScreen
};

/** 
 * @type keytype
 * @brief Screen capture object specific information
 */
struct ScreenCaptureSourceInfo {
    /** 
     * @brief When sharing a screen. See ScreenCaptureSourceType{@link #ScreenCaptureSourceType} for the type of shared object
     */
    ScreenCaptureSourceType type = kScreenCaptureSourceTypeUnknown;

    /** 
     * @brief ID of the screen-shared object
     */
    view_t source_id = nullptr;

    /** 
     * @brief Name of the screen-shared object<br>
     *        Save it as a `string` object before calling Release{@link #IScreenCaptureSourceList#Release} to release the dynamic memory.
     */
    const char* source_name = nullptr;
};

/** 
 * @hidden(Android, iOS)
 * @type api
 * @region  screen sharing
 * @brief  Screen source type information list
 */
class IScreenCaptureSourceList {
public:
  /**
   * @hidden
   */
    virtual ~IScreenCaptureSourceList() {
    }
    /** 
     * @region Screen sharing
     * @brief  Get list size
     * @return  Get list size
     */
    virtual int32_t GetCount() = 0;

    /** 
     * @region Screen share
     * @brief According to the index, get the elements in the screen share list
     * @param [in] index Index number
     * @return Screen source type information
     * @notes Return a string of type char* in the type, which is released after this object is released, please pay attention to memory management.
     */
    virtual ScreenCaptureSourceInfo GetSourceInfo(int32_t index) = 0;

    /** 
     * @region Screen sharing
     * @brief Destruct the current object and free up memory
     * @notes Call delete the structure is strictly prohibited, this function should be called to free resources when the structure is not needed
     */
    virtual void Release() = 0;
};

/** 
 * @type api
 * @brief Engine API
 */
class IRtcEngineLite {
public:
    /** 
     * @hidden
     * @deprecated since 327.1, use SetCaptureVolume instead
     * @type api
     * @region Audio management
     * @brief Adjust the audio acquisition volume
     * @param  [in] volume Audio acquisition volume, the value range: [0,400] <br>
     *        + 0: Mute <br>
     *        + 100: Original volume <br>
     *        + 400: Up to 4 times the original volume (with overflow protection) <br>
     * @notes  To ensure better call quality, it is recommended to set the volume value to [0,100], this interface is obsolete, please use SetCaptureVolume{@link #SetCaptureVolume}.
     */
    virtual void SetRecordingVolume(const int volume) = 0;

    /** 
     * @hidden (Linux)
     * @type api
     * @region Volume management
     * @brief Adjust the audio acquisition volume
     * @param  [in] index Index of the stream, whose volume needs to be adjusted. Refer to StreamIndex{@link #StreamIndex} for more details.
     * @param  [in] volume Ratio of capture volume to original volume. Ranging: [0,400]. Unit: %
     *        + 0: Mute <br>
     *        + 100: Original volume. To ensure the audio quality, we recommend [0, 100].
     *        + 400: Four times the original volume with signal-clipping protection.
     * @notes  Call this API to set the volume of the audio capture before or during the audio capture.
     */
    virtual void SetCaptureVolume(StreamIndex index, int volume) = 0;

    /** 
     * @hidden (Linux)
     * @type api
     * @region Audio management
     * @brief Adjust the playback volume of the mixed remote audios.
     * @param  [in] volume Ratio(%) of playback volume to original volume, in the range [0, 400], with overflow protection.  <br>
     *        To ensure the audio quality, we recommend setting the volume within `100`.  <br>
     *        + 0: mute <br>
     *        + 100: original volume <br>
     *        + 400: Four times the original volume with signal-clipping protection.
     * @notes To ensure better call quality, it is recommended to set the volume value to [0,100].
     */
    virtual void SetPlaybackVolume(const int volume) = 0;


    /** 
     * @hidden (Windows)
     * @type api
     * @region Audio management
     * @brief Turn on/off the earphone monitor function
     * @param  [in] mode Whether to turn on the earphone monitor function. See EarMonitor Mode {@link #EarMonitor Mode}
     */
    virtual void SetEarMonitorMode(EarMonitorMode mode) = 0;

    /** 
     * @hidden (Windows)
     * @type api
     * @region Audio Management
     * @brief Set the volume of the earphone monitor
     * @param  [in] volume The volume of the earphone monitor, the value range: [0,100], the unit:%
     * @notes  <br>
     *         + Before setting the volume of the earphone monitor, you must first call SetEarMonitor Mode {@link #SetEarMonitor Mode} to turn on the earphone monitor function. <br>
     *         + The earphone monitor function is only effective when the internal audio capture function provided by the RTC SDK is used and a wired earphone is connected. <br>
     */
    virtual void SetEarMonitorVolume(const int volume) = 0;

    /** 
     * @hidden
     * @deprecated since 327.1, use SetScreenAudioStreamIndex instead
     * @type api
     * @region Screen Sharing
     * @brief Turn on the sound card acquisition and merge the audio played by the sound card into the locally collected audio stream. <br>
     *         If you want to transfer the sound played by the sound card to a remote location (such as screen sharing and other scenes), you must turn on the sound card acquisition. In this way, the sound played by the sound card is merged into the locally collected audio stream and sent to the remote end together.
     * @notes   <br>
     *        + After turning on the sound card acquisition, you can set the corresponding volume. See SetScreenAudioVolumeBeforeMixing {@link
     *  #SetScreenAudioVolumeBeforeMixing} <br>
     *        + If you need to turn off the sound card acquisition. See StopMixScreenAudioToMainStream{@link #StopMixScreenAudioToMainStream}
     */
    virtual void StartMixScreenAudioToMainStream() = 0;

    /** 
     * @hidden
     * @deprecated since 327.1, use SetScreenAudioStreamIndex instead
     * @type api
     * @region Screen Sharing
     * @brief Turn off the sound card acquisition, the audio stream played by the sound card is no longer combined into the locally collected audio stream.
     * @notes If you need to turn on the sound card acquisition, please refer to StartMixScreenAudioToMainStream{@link #StartMixScreenAudioToMainStream}
     */
    virtual void StopMixScreenAudioToMainStream() = 0;

    /** 
     * @hidden
     * @deprecated since 327.1, use SetCaptureVolume instead
     * @type api
     * @region screen sharing
     * @brief To adjust the local sound card acquisition volume. <br>
     *        After the local sound card acquisition is turned on, you can use this interface to adjust the acquisition volume.
     *         For information on how to turn on local sound card acquisition, refer to StartMixScreenAudioToMainStream{@link #StartMixScreenAudioToMainStream}.
     * @param volume The sound card collects the volume, the value range: [0,400] <br>
     *        + 0: mute <br>
     *        + 100: the original volume <br>
     *        + 400: the maximum can be 4 times the original volume (with overflow protection) <br>
     */
    virtual void SetScreenAudioVolumeBeforeMixing(int volume) = 0;

    /** 
     * @hidden (Linux)
     * @type api
     * @region Audio management
     * @brief Start internal audio capture. The default is off.   <br>
     *        Internal audio capture refers to: capturing audio using the built-in module.
     *        The local client will be informed via OnAudioDeviceStateChanged{@link #IRtcEngineLiteEventHandler#OnAudioDeviceStateChanged} after starting audio capture by calling this API. <br>
     *        The remote clients in the room will be informed of the state change via OnUserStartAudioCapture{@link #IRTCRoomEventHandler#OnUserStartAudioCapture} after the visible user starts audio capture by calling this API..
     * @notes   <br>
     *        + To enable a microphone without the user's permission will trigger OnWarning{@link #IRtcEngineLiteEventHandler#OnWarning}. <br>
     *        + Call StopAudioCapture{@link #StopAudioCapture} to stop the internal audio capture. Otherwise, the internal audio capture will sustain until you destroy the engine instance. <br>
     *        + Once you create the engine instance, you can start internal audio capture regardless of the audio publishing state. The audio stream will start publishing only after the audio capture starts. <br>
     *        + To switch from custom to internal audio capture, stop publishing before disabling the custom audio capture module and then call this API to enable the internal audio capture.<br>
     */
    virtual void StartAudioCapture() = 0;

    /** 
     * @hidden (Linux)
     * @type api
     * @region Audio management
     * @brief Stop internal audio capture. The default is off.   <br>
     *        Internal audio capture refers to: capturing audio using the built-in module.
     *         The local client will be informed via  OnAudioDeviceStateChanged{@link #IRtcEngineLiteEventHandler#OnAudioDeviceStateChanged}  after stopping audio capture by calling this API.<br>. <br>
     *        The remote clients in the room will be informed of the state change via OnUserStopAudioCapture{@link #IRTCRoomEventHandler#OnUserStopAudioCapture} after the visible client stops audio capture by calling this API.<br>.
     * @notes   <br>
     *        + Call StartAudioCapture{@link #StartAudioCapture} to enable the internal audio capture. <br>
     *        + Without calling this API the internal audio capture will sustain until you destroy the engine instance.<br>
     */
    virtual void StopAudioCapture() = 0;

    /** 
     * @hidden (macOS, Windows, Linux)
     * @type api
     * @region  Audio management
     * @brief  Set the audio scene type.   <br>
     *        You can choose the appropriate audio scene type according to the scene where your application is located.
     *        After selecting an audio scenario, RTC automatically applies call volume/media volume based on client side audio routing and publishing subscription status. <br>
     *         Settings can take effect before and after entering the room.
     * @param  [in] scenario Audio scene type,
     *         See AudioScenarioType{@link #AudioScenarioType}
     * @notes   <br>
     *         +
     * Call volume is more suitable for calls, meetings and other scenes that require higher information accuracy. Call volume will activate the system hardware signal processing, making the call sound clearer. At this point, the volume cannot be reduced to
     * 0. <br>
     *         + Media volume is more suitable for entertainment scenes, because its sound will be more expressive. Under the media volume, the minimum volume can be reduced to 0.
     */
    virtual void SetAudioScenario(AudioScenarioType scenario) = 0;

    /** 
     * @type api
     * @region Bel Sound Effect Management
     * @brief Set the sound change effect type
     * @param  [in] voice_changer The sound change effect type. See VoiceChangerType{@link #VoiceChangerType}
     * @notes   <br>
     *         + It can be set before and after entering the room. <br>
     *         + Is effective for both internally captured and custom captured audio from the RTC SDK. <br>
     *         + Only valid for mono audio. <br>
     *         + Is only valid in SDKs that include the ability of American sound effects. <br>
     *         + Is mutually exclusive with SetVoiceReverbType{@link #SetVoiceReverbType}, and the effects set later will override the effects set first. <br>
     */
    virtual int SetVoiceChangerType(VoiceChangerType voice_changer) = 0;

    /** 
     * @type api
     * @region Bel Sound Effect Management
     * @brief Set the reverb effect type
     * @param  [in] voice_reverb The reverb effect type. See VoiceReverbType{@link #VoiceReverbType}
     * @notes   <br>
     *         + You can set it before and after entering the room. <br>
     *         + Is effective for both internally captured and custom captured audio from the RTC SDK. <br>
     *         + Only valid for mono audio. <br>
     *         + Is only valid in SDKs that include the ability of American sound effects. <br>
     *         + Is mutually exclusive with SetVoiceChangerType{@link #SetVoiceChangerType}, and the effects set later will override the effects set first. <br>
     */
    virtual int SetVoiceReverbType(VoiceReverbType voice_reverb) = 0;

    /** 
     * @type api
     * @region audio management
     * @brief Sets the sound quality gear. You should choose the appropriate sound quality gear according to the needs of the business scenario.   <br>
     * @param  [in] audio_profile Sound quality gear. See AudioProfileType{@link #AudioProfileType}
     * @notes   <br>
     *         + This method can be called before and after entering the room; <br>
     *         + Support dynamic switching of sound quality gear during a call.
     */
    virtual void SetAudioProfile(AudioProfileType audio_profile) = 0;

    /** 
     * @hidden
     * @deprecated since 336.1, use UnpublishStream and PublishStream instead.
     * @type api
     * @region Media stream management
     * @brief Enable/ Disable the transmission of the local audio stream.   <br>
     *        Remote users in the room will be informed via OnUserMuteAudio{@link #IRTCRoomEventHandler#OnUserMuteAudio} after the visible user changes the transmission state of the audio streams by calling this API.
     * @param  [in] mute_state An identifier denoting whether the local audio stream is sending. Refer to MuteState{@link #MuteState} for more details.
     * @notes <br>
     *        + Calling this API toggles the transmission state of the local audio streams instead of the capture state.
     *        + You can enable audio transmission regardless of the audio capture state. Enable the transmission before capture, thus the audio stream can be sent once the capture starts.
     */
    virtual void MuteLocalAudio(MuteState mute_state) = 0;

    /** 
     * @type api
     * @region Room Management
     * @brief Publishes media streams captured by camera/microphone in the current room.
     * @param [in] type Media stream type, used for specifying whether to publish audio stream or video stream. See MediaStreamType{@link #MediaStreamType}.
     * @notes  <br>
     *        + An invisible user cannot publish media streams. Call SetUserVisibility{@link #IRtcRoom#SetUserVisibility} to change your visibility in the room.  <br>
     *        + Call PublishScreen{@link #IRtcRoom#PublishScreen} to start screen sharing.  <br>
     *        + Call StartForwardStreamToRooms{@link #IRtcRoom#StartForwardStreamToRooms} to forward the published streams to the other rooms.  <br>
     *        + After you call this API, the other users in the room will receive OnUserPublishStream{@link #IRTCRoomEventHandler#OnUserPublishStream}. Those who successfully received your streams will receive OnFirstRemoteAudioFrame{@link #IRTCRoomEventHandler#OnFirstRemoteAudioFrame}/OnFirstRemoteVideoFrameDecoded{@link #IRTCRoomEventHandler#OnFirstRemoteVideoFrameDecoded} at the same time.  <br>
     *        + Call UnpublishStream{@link #IRtcEngineLite#UnpublishStream} to stop publishing streams.
     */
    virtual void PublishStream(MediaStreamType type) = 0;

    /** 
     * @type api
     * @region Room Management
     * @brief Stops publishing media streams captured by camera/microphone in the current room.
     * @param [in] type Media stream type, used for specifying whether to stop publishing audio stream or video stream. See MediaStreamType{@link #MediaStreamType}.
     * @notes  <br>
     *         + After calling PublishStream{@link #IRtcEngineLite#PublishStream}, call this API to stop publishing streams. <br>
     *         + After calling this API, the other users in the room will receive OnUserUnPublishStream{@link #IRTCRoomEventHandler#OnUserUnPublishStream}.
     */
    virtual void UnpublishStream(MediaStreamType type) = 0;
    /** 
      * @hidden
      * @deprecated since 340.1, use SetAudioSourceType and SetAudioRenderType instead.
      * @type api
      * @region Custom Audio Capture and Rendering
      * @brief Enables custom audio capture and rendering.
      * @param [in] recording_format See AudioFormat{@link #AudioFormat}
      * @param [in] playback_format See AudioFormat{@link #AudioFormat}
      * @notes   <br>
      *       + You must call this API before joining the room.<br>
      *       + The state of enabling custom audio capture and rendering is still valid after leaving the room, until DisableExternalAudioDevice{@link #IRtcEngineLite#DisableExternalAudioDevice} is called to disable custom audio capture and rendering. <br>
      *       + You must enable both custom audio capture and custom audio rendering, or enable both internal audio capture and internal audio rendering. By default, internal audio capture and rendering are used. <br>
      *       + After enabling custom audio capture and rendering, call PushExternalAudioFrame{@link #IRtcEngineLite#PushExternalAudioFrame} to push custom captured audio frames to the RTC SDK for transmission; and call PullExternalAudioFrame{@link #IRtcEngineLite#PullExternalAudioFrame} to get remote audio frames. <br>
      *       + If calling the API to switch from internal audio capture to custom capture, the internal audio capture is automatically disabled. <br>
      *       + To switch to internal capture when custom capture is enabled, you must call DisableExternalAudioDevice{@link #IRtcEngineLite#DisableExternalAudioDevice} to disable custom capture, and then call StartAudioCapture{@link #IRtcEngineLite#StartAudioCapture} to manually enable internal capture.
      */
    virtual void EnableExternalAudioDevice(const AudioFormat &recording_format, const AudioFormat &playback_format) = 0;
     /** 
      * @hidden
      * @deprecated since 340.1, use SetAudioSourceType and SetAudioRenderType instead.
      * @type api
      * @region Custom Audio Capture Rendering
      * @brief Disables custom audio capture and rendering.
      * @notes   <br>
      *       + To switch to internal capture when custom capture is enabled, you must call this API and then call StartAudioCapture{@link #IRtcEngineLite#StartAudioCapture}. <br>
      *       + To enable custom audio capture and rendering, call EnableExternalAudioDevice{@link #IRtcEngineLite#EnableExternalAudioDevice}.
      */
    virtual void DisableExternalAudioDevice() = 0;
    /** 
      * @hidden
      * @deprecated since 340.1, use PushExternalAudioFrame with IAudioFrame parameter instead.
      * @type api
      * @region Custom Audio Capture and Rendering
      * @brief Push custom captured audio data to the RTC SDK.
      * @param [in] data PCM data. The audio sampling format must be S16. The data format in the audio buffer must be PCM, and its capacity size should be `audioFrame.samples × audioFrame.channel × 2`.
      * @param [in] samples The samples (number of audio sampling points) of a single push should be `audioFrame.sample Rate/100`. For example, when the sampling rate is set to 48000, data of 480 sampling points should be pushed each time. <br>
      * @return <br>
      *         + 0: Success <br>
      *         + < 0: Failure <br>
      * @notes   <br>
      *        + Before pushing external audio data, you must call EnableExternalAudioDevice{@link #IRtcEngineLite#EnableExternalAudioDevice} to enable custom audio capture. <br>
      *        + You must push custom captured audio data every 10 milliseconds.
      *        + This API runs in a user-level thread. If other processes are running at the same time, this process will be interrupted. <br>
      */
    virtual bool PushExternalAudioFrame(int8_t* data, int samples) = 0;
    /** 
     * @hidden
     * @deprecated since 340.1, use PullExternalAudioFrame with IAudioFrame parameter instead.
     * @type api
     * @region  custom audio acquisition rendering
     * @brief  Pull remote audio data. After you enable custom audio capture rendering with EnableExternalAudioDevice{@link #IRtcEngineLite#EnableExternalAudioDevice}
     * , You can use this method to pull remote audio data.
     * @param [out] Data
     *        Pcm data, the memory size should be samples × playback_format.channel × 2.
     * @param [in] samples
     *         The number of sampling points should be
     * Playback_format.sample _rate/100 set in EnableExternalAudioDevice{@link #IRtcEngineLite#EnableExternalAudioDevice}. When the sample rate is set to 48000, 480 sample points should be pulled each time
     * @return   Method call result   <br>
     *         + True: Success. <br>
     *         + False: failure <br>
     * @notes   <br>
     *        + Must be pcm data, pull interval is 10ms, audio sampling format is s16.  <br>
     *        + This function runs in the user calling thread and is a synchronous function <br>
     */
    virtual bool PullExternalAudioFrame(int8_t* data, int samples) = 0;
    /** 
     * @type api
     * @region Custom Audio Capture and Rendering
     * @brief  Switch the audio capture type.
     * @param type Audio input source type. See AudioSourceType{@link #AudioSourceType} <br>
     *            Use internal audio capture by default. The audio capture type and the audio render type may be different from each other.
     * @return   Method call result:   <br>
     *         + >0: Success. <br>
     *         + -1: Failure.
     * @notes   <br>
     *       + You can call this API before or after joining the room.<br>
     *       + If you call this API to switch from internal audio capture to custom capture, the internal audio capture is automatically disabled. You must call PushExternalAudioFrame{@link #IRtcEngineLite#PushExternalAudioFrame} to push custom captured audio data to RTC SDK for transmission. <br>
     *       + If you call this API to switch from custom capture to internal capture, you must then call StartAudioCapture{@link #IRtcEngineLite#StartAudioCapture} to enable internal capture. <br>
     */
    virtual int SetAudioSourceType (AudioSourceType type) = 0;
    /** 
     * @type api
     * @region Custom Audio Capture and Rendering
     * @brief  Switch the audio render type.
     * @param type Audio output source type. See AudioRenderType{@link #AudioRenderType}. <br>
     *             Use internal audio render by default. The audio capture type and the audio render type may be different from each other.
     * @return   Method call result:   <br>
     *         + >0: Success. <br>
     *         + -1: Failure.
     * @notes   <br>
     *       + You can call this API before or after joining the room. <br>
     *       + After calling this API to enable custom audio rendering, call PullExternalAudioFrame{@link #IRtcEngineLite#PullExternalAudioFrame} for audio data.
     */
    virtual int SetAudioRenderType (AudioRenderType type) = 0;
    /** 
     * @type api
     * @region Custom Audio Capture and Rendering
     * @brief Push custom captured audio data to the RTC SDK.
     * @param [in] audioFrame. The audio data of 10 ms. See IAudioFrame{@link #IAudioFrame}。
     * @return <br>
     *         + 0: Success <br>
     *         + < 0: Failure <br>
     * @notes   <br>
     *        + Before calling this API, you must call EnableExternalAudioDevice{@link #IRtcEngineLite#EnableExternalAudioDevice} to enable custom audio capture. <br>
     *        + You must push custom captured audio data every 10 ms. <br>
     *        + This API runs in the user-level thread. If other processes are running at the same time, this process will be interrupted.
     */
    virtual bool PushExternalAudioFrame(IAudioFrame* audioFrame) = 0;
    /** 
     * @region  custom audio acquisition rendering
     * @brief  Pull remote audio data. You can use the data for audio processing or custom rendering.
     * @param [out] audioFrame The obtained audio data of 10 ms. See IAudioFrame{@link #IAudioFrame}。
     * @return   Method call result:   <br>
     *         + True: Success. <br>
     *         + False: failure <br>
     * @notes   <br>
     *        + Before pulling the audio data, call SetAudioRenderType{@link #IRtcEngineLite#SetAudioRenderType} to enable custom rendering. <br>
     *        + Pull the audio data every 10 ms. <br>
     *        + This function runs in the user calling thread and is a synchronous function. <br>
     */
    virtual bool PullExternalAudioFrame(IAudioFrame* audioFrame) = 0;

    /** 
     * @hidden
     * @deprecated since 325.1, use IAudioMixingManager instead
     * @type api
     * @region  mixing
     * @brief  To start playing music files and mixing.
     * @param  [in] file_path <br>
     *         Specifies the absolute path of the local file that needs to be mixed. Supported audio file formats are: mp3, aac, m4a, 3gp, wav.
     * @param  [in] loopback <br>
     *        + True: Only the mixed or replaced audio stream can be heard locally. <br>
     *        + False: Both local and opposite parties can hear the mixed or replaced audio stream
     * @param  [in] replace <br>
     *        + True: Only push the set local audio file, not transfer the audio collected by the microphone. <br>
     *        + False: The audio file content will be mixed with the audio stream collected by the microphone
     * @param  [in] cycle Specify the number of times the audio file is cycled. <br>
     *        + > 0: The number of cycles. <br>
     *        + -1: Infinite loop
     * @return   <br>
     *        + 0: Success <br>
     *        + < 0: Failure
     * @notes   <br>
     *       + This method specifies that the audio stream captured by the local and microphone is mixed or replaced. Replacement refers to replacing the audio stream captured by the microphone with an audio file. <br>
     *       + This method can choose whether to let the other party hear the locally played audio and specify the number of times to loop it.
     * When this method is called to play the music file and the mix ends, the app receives an OnAudioMixingStateChanged{@link #IRtcEngineLiteEventHandler#OnAudioMixingStateChanged} callback. <br>
     *       + After you start playing the music file and mixing, you can call the StopAudioMixing{@link #IAudioMixingManager#StopAudioMixing} method to stop playing the music file. <br>
     *       + This method mixes external files of data sources, and the supported file formats are: mp3, aac, m4a, 3gp, wav; and SetAudioPlayoutMixStream {@link
     *  #SetAudioPlayoutMixStream} The data source of the mix is external cache and the audio format is PCM; these two mixing methods can coexist. <br>
     *       + A single room only supports the playback of all music files. After multiple calls to this function, only the last call will take effect. <br>
     *       + Please call this method inside the room.
     */
    virtual int StartAudioMixing(const char* file_path, bool loopback, bool replace, int cycle) = 0;

    /** 
     * @hidden
     * @deprecated since 325.1, use IAudioMixingManager instead
     * @type api
     * @region Audio Mixing
     * @brief Stop playing music files and mixes
     * @notes   <br>
     *       + Call the StartAudioMixing{@link #IAudioMixingManager#StartAudioMixing}
     * After the method starts playing music files and mixes, call this method to stop playing music files and mixes. <br>
     *       + Please call this method inside the room.
     */
    virtual void StopAudioMixing() = 0;

    /** 
     * @hidden
     * @deprecated since 325.1, use IAudioMixingManager instead
     * @type api
     * @region Audio Mixing
     * @brief Pause playing music files
     * @return   <br>
     *         + 0: Success <br>
     *         + < 0: Failure
     * @notes   <br>
     *       + Call StartAudioMixing{@link #IAudioMixingManager#StartAudioMixing} method to start playing music After the file and mixing, you can call this method to pause playing the music file. <br>
     *       + After calling this method to pause playing the music file, you can call the ResumeAudioMixing{@link #IAudioMixingManager#ResumeAudioMixing} method to resume playing. <br>
     *       + Please call this method inside the room.
     */
    virtual int PauseAudioMixing() = 0;

    /** 
     * @hidden
     * @deprecated since 325.1, use IAudioMixingManager instead
     * @type api
     * @region  mixing
     * @brief  Resume playing music files
     * @return   <br>
     *        + 0: success <br>
     *        + < 0: failure
     * @notes   <br>
     *       + Call PauseAudioMixing{@link #IAudioMixingManager#PauseAudioMixing} method after pausing playing music files , you can resume playing by calling this method. <br>
     *       + Please call this method in the room
     */
    virtual int ResumeAudioMixing() = 0;

    /** 
     * @hidden
     * @deprecated since 325.1, use IAudioMixingManager instead
     * @type api
     * @region  mixing
     * @brief  To adjust the file volume of music files. In order to ensure better sound quality, it is recommended to set the volume value to [0,100].
     * @param  [in] volume Music file playback volume range is 0~ 400. <br>
     *        + 0: mute <br>
     *        + 100: original volume <br>
     *        + 400: maximum adjustable volume (with overflow protection)
     * @return   <br>
     *        + 0: success <br>
     *        + < 0: failure
     * @notes   <br>
     *       + Call this method can be adjusted at the same time is local And remote playback volume. AdjustAudioMixingPlayoutVolume {@link
     *  #AdjustAudioMixingPlayoutVolume} Can be used to adjust the local volume only, and AdjustAudioMixingPublishVolume {@link
     *  #AdjustAudioMixingPublishVolume} Can be used to adjust the remote volume only. <br>
     *       + The volume adjustment effect of this method on AdjustAudioMixingPlayoutVolume{@link #AdjustAudioMixingPlayoutVolume} and
     * AdjustAudioMixingPublishVolume{@link #AdjustAudioMixingPublishVolume} is a product relationship. <br>
     *       + Please call this method inside the room.
     */
    virtual int AdjustAudioMixingVolume(int volume) = 0;

    /** 
     * @hidden
     * @deprecated since 325.1, use IAudioMixingManager instead
     * @type api
     * @region  mixing
     * @brief  To adjust the local playback volume of music files. In order to ensure better sound quality, it is recommended to set the volume value to [0,100].
     * @param  [in] volume Music file playback volume range is 0~ 400. <br>
     *        + 0: mute <br>
     *        + 100: original volume <br>
     *        + 400: maximum adjustable volume (with overflow protection)
     * @return   <br>
     *        + 0: success <br>
     *        + < 0: failure
     * @notes   <br>
     *       + Call this method to adjust the mixed The volume at which the music file is played locally. If you need to adjust the local and remote playback volume at the same time, you can use the
     * AdjustAudioMixingVolume{@link #AdjustAudioMixingVolume} method. <br>
     *       + Please call this method inside the room.
     */
    virtual int AdjustAudioMixingPlayoutVolume(int volume) = 0;

    /** 
     * @hidden
     * @deprecated since 325.1, use IAudioMixingManager instead
     * @type api
     * @region  mixing
     * @brief  To adjust the remote playback volume of the music file. In order to ensure better sound quality, it is recommended to set the volume value to [0,100].
     * @param  [in] volume Music file playback volume range is 0~ 400. <br>
     *        + 0: mute <br>
     *        + 100: original volume <br>
     *        + 400: maximum adjustable volume (with overflow protection)
     * @return   <br>
     *        + 0: success <br>
     *        + < 0: failure
     * @notes   <br>
     *       + This method adjusts the music file of the mix in The volume of the remote playback. If you need to adjust the local and remote playback volume at the same time, you can use the
     * AdjustAudioMixingVolume{@link #AdjustAudioMixingVolume} method. <br>
     *       + Please call this method inside the room.
     */
    virtual int AdjustAudioMixingPublishVolume(int volume) = 0;

    /** 
     * @hidden
     * @deprecated since 325.1, use IAudioMixingManager instead
     * @type api
     * @region  mix
     * @brief  Get music file duration
     * @return   <br>
     *        + > 0: Success, music file duration, in milliseconds. <br>
     *        + < 0: Failed
     * @notes  <br>
     *       Please call this method inside the room.
     */
    virtual int GetAudioMixingDuration() = 0;

    /** 
     * @hidden
     * @deprecated since 325.1, use IAudioMixingManager instead
     * @type api
     * @region  mixing
     * @brief  Get the music file playback progress
     * @return   <br>
     *        + > 0: Success, the music file playback progress, in milliseconds. <br>
     *        + < 0: Failed
     * @notes  <br>
     *       Please call this method inside the room.
     */
    virtual int GetAudioMixingCurrentPosition() = 0;

    /** 
     * @hidden
     * @deprecated since 325.1, use IAudioMixingManager instead
     * @type api
     * @region  mixing
     * @brief  To set the playback location of the audio file
     * @param  [in] pos
     *         Integer. Progress bar position in milliseconds.
     * @return   <br>
     *        + 0: Success <br>
     *        + < 0: Failure
     * @notes   <br>
     *       This method can set the playback location of the audio file, so that you can play the music file from the specified location according to the actual situation, without playing a music file completely from beginning to end.
     */
    virtual int SetAudioMixingPosition(int pos) = 0;

    /** 
     * @hidden
     * @deprecated since 325.1, use IAudioMixingManager instead
     * @type api
     * @region Audio Mixing
     * @brief Get the file volume of the specified sound file
     * @param  [in] sound_id <br>
     *        Sound ID, apply caller maintenance, please ensure uniqueness. Make sure the sound_id here is the same as the sound_id set by StartAudioMixing{@link #IAudioMixingManager#StartAudioMixing}
     * .
     * @return   <br>
     *        + > 0: Success, file volume, volume range 0~ 400. <br>
     *        + < 0: Failed
     * @notes   <br>
     *       Please call this method inside the room.
     */
    virtual int GetEffectVolume(int sound_id) = 0;

    /** 
     * @hidden
     * @deprecated since 325.1, use IAudioMixingManager instead
     * @type api
     * @region Audio Mixing
     * @brief Adjust the file volume of the specified sound file
     * @param  [in] sound_id <br>
     *        Sound ID, apply caller maintenance, please ensure uniqueness. Make sure the sound_id here is the same as the sound_id set by StartAudioMixing{@link #IAudioMixingManager#StartAudioMixing}
     * .
     * @param  [in] volume <br>
     *        Music file playback volume range is 0~ 400. Default 100 is the original file volume
     * @return   <br>
     *        + 0: Success <br>
     *        + < 0: Failure
     * @notes
     *        + This method adjusts the volume of audio files played locally and remotely. <br>
     *        + Please call this method inside the room.
     */
    virtual int SetVolumeOfEffect(int sound_id, int volume) = 0;

    /** 
     * @hidden
     * @deprecated since 325.1, use IAudioMixingManager instead
     * @type api
     * @region  mixing
     * @brief  To start playing the specified sound effect file. In this method, you can set the number of times the sound effect file is played, the volume, and whether the remote user can hear the sound effect.
     * @param  [in] sound_id <br>
     *         Sound ID, application caller maintenance, please ensure uniqueness. If the sound effect has been loaded into memory through PreloadEffect{@link #PreloadEffect}
     * , Make sure that the sound_id here is the same as the sound_id set by PreloadEffect{@link #PreloadEffect}.
     * @param  [in] file_path <br>
     *         Specify the audio file name and file path name that need to be mixed. The following audio formats are supported: mp3, aac, m4a, 3gp, wav
     * @param  [in] loopback <br>
     *        + True: Only the sound effect can be heard locally. <br>
     *        + False: The sound effect can be heard by both local and remote users.
     * @param  cycle  Specifies the number of times the audio file is cycled. <br>
     *        + > 0: The number of cycles. <br>
     *        + -1: Infinite loop until it stops after calling StopEffect{@link #StopEffect} or StopAllEffects{@link #StopAllEffects}.
     * @param  [in] volume <br>
     *        The sound file playback volume range is 0~ 400. The default is 100 for the original file volume.
     * @return   <br>
     *        + 0: Success <br>
     *        + < 0: Failure
     * @notes   <br>
     *       + After calling this method to play the sound effect, the application will receive an OnAudioMixingStateChanged{@link #IRtcEngineLiteEventHandler#OnAudioMixingStateChanged} callback. <br>
     *       + This method can be called many times by passing in the sound_id and
     * File_path of different sound files to realize simultaneous playback of multiple sound files and sound effect superposition. <br>
     *       + You can call the StopEffect{@link #StopEffect} method to stop playing the specified sound effect file. <br>
     *       + Please call this method inside the room.
     */
    virtual int PlayEffect(int sound_id, const char* file_path, bool loopback, int cycle, int volume) = 0;

    /** 
     * @hidden
     * @deprecated since 325.1, use IAudioMixingManager instead
     * @type api
     * @region Audio Mixing
     * @brief  To preload the specified sound effect file.
     * @param  [in] sound_id <br>
     *         Sound ID, application caller maintenance, please ensure uniqueness.
     * @param  [in] file_path <br>
     *         Specify the audio file name and file path name that need to be mixed. The following audio formats are supported: mp3, aac, m4a, 3gp, wav.
     * @return   <br>
     *         + 0: Success <br>
     *         + < 0: Failure
     * @notes   <br>
     *        +
     * Call this method to preload and play the specified sound effect file. The preload operation can be completed before entering the room (all mixing-related interfaces, if not marked, can be called in the room when not entering the room). If the sound file is very long, the loading operation may take a long time. It is recommended that application developers put the preloading operation in a child thread.
     *  <br>
     *        + This method only preloads to play the specified sound effect file, and only calls the StartAudioMixing{@link #IAudioMixingManager#StartAudioMixing}
     *  Method to start playing the specified sound effect file. <br>
     *        + This method preloads the specified sound effect file, which can be unloaded via UnloadEffect{@link #UnloadEffect}.
     */
    virtual int PreloadEffect(int sound_id, const char* file_path) = 0;

    /** 
     * @hidden
     * @deprecated since 325.1, use IAudioMixingManager instead
     * @type api
     * @region Audio Mixing
     * @brief Uninstall the specified sound file
     * @param  [in] sound_id <br>
     *        Sound ID, apply caller maintenance, please ensure uniqueness.
     * @return   <br>
     *        + 0: Success <br>
     *        + < 0: Failure
     * @notes   <br>
     *        + Call this method to uninstall the specified sound file. <br>
     *        + If the sound file is not unloaded when the StopEffect{@link #StopEffect} method is called, the SDK will automatically call the method to uninstall the sound file.
     */
    virtual int UnloadEffect(int sound_id) = 0;

    /** 
     * @hidden
     * @deprecated since 325.1, use IAudioMixingManager instead
     * @type api
     * @region Audio Mixing
     * @brief Stop playing the specified sound file
     * @param  [in] sound_id <br>
     *        Sound ID, apply caller maintenance, please ensure uniqueness. If the sound effect has been loaded into memory through PreloadEffect{@link #PreloadEffect}
     * , Make sure that the sound_id here is the same as the sound_id set by PreloadEffect{@link #PreloadEffect}.
     * @return   <br>
     *        + 0: Success <br>
     *        + < 0: Failure
     * @notes   <br>
     *        + Call this method to stop playing the specified sound file. <br>
     *        + UnloadEffect{@link #UnloadEffect} is actively called inside this method to uninstall the specified sound effect text.
     */
    virtual int StopEffect(int sound_id) = 0;

    /** 
     * @hidden
     * @deprecated since 325.1, use IAudioMixingManager instead
     * @type api
     * @region Audio Mixing
     * @brief Pause playing the specified sound file
     * @param  [in] sound_id <br>
     *        Sound ID, apply caller maintenance, please ensure uniqueness
     * @return   <br>
     *        + 0: Success <br>
     *        + < 0: Failed
     * @notes   <br>
     *       Call this method to pause the playback of the specified sound effect file. You can call the ResumeEffect{@link #ResumeEffect} method to resume playback.
     */
    virtual int PauseEffect(int sound_id) = 0;

    /** 
     * @hidden
     * @deprecated since 325.1, use IAudioMixingManager instead
     * @type api
     * @region Audio Mixing
     * @brief Resume playback of the specified sound file
     * @param  [in] sound_id <br>
     *        Sound ID, apply caller maintenance, please ensure uniqueness.
     * @return   <br>
     *        + 0: Success <br>
     *        + < 0: Failure
     * @notes   <br>
     *       + After calling the PauseEffect{@link #PauseEffect} method to pause playing the specified sound effect file, you can resume playing through this method. <br>
     *       + After calling the PauseAllEffects{@link #PauseAllEffects}
     *  Method to pause the playback of all sound effects files, you can also use this method to resume the playback of individual sound effects files.
     */
    virtual int ResumeEffect(int sound_id) = 0;

    /** 
     * @hidden
     * @deprecated since 325.1, use IAudioMixingManager instead
     * @type api
     * @region Audio Mixing
     * @brief Adjust the file volume of all sound files
     * @param  [in] volume <br>
     *        Music file playback volume range is 0~ 400. The default is 100 for the original file volume.
     * @notes   <br>
     *       + This method adjusts the volume of all sound files of the mix to be played locally and remotely. <br>
     *       + Please call this method inside the room.
     */
    virtual void SetEffectsVolume(int volume) = 0;

    /** 
     * @hidden
     * @deprecated since 325.1, use IAudioMixingManager instead
     * @type api
     * @region Audio Mixing
     * @brief Stop playing all sound files
     * @notes   <br>
     *      This method stops playing all sound files.
     */
    virtual void StopAllEffects() = 0;

    /** 
     * @hidden
     * @deprecated since 325.1, use IAudioMixingManager instead
     * @type api
     * @region  mixing
     * @brief  To pause the playback of all sound files.
     * @notes   <br>
     *       Call this method to pause the playback of all sound effects files, and you can call the ResumeAllEffects{@link #ResumeAllEffects} method to resume playback.
     */
    virtual void PauseAllEffects() = 0;

    /** 
     * @hidden
     * @deprecated since 325.1, use IAudioMixingManager instead
     * @type api
     * @region Audio Mixing
     * @brief Resume playback of all sound files
     * @notes   <br>
     *      + Call this method to resume playback of all sound files. After PauseAllEffects{@link #PauseAllEffects}
     *  Is called to pause playback of all sound files, this method can be called to resume playback of all sound files. <br>
     *      + After calling the PauseEffect{@link #PauseEffect}
     *  Method to pause the playback of a single specified sound effect file, you can also call this method to resume playback, but the efficiency is too low to recommend this use.
     */
    virtual void ResumeAllEffects() = 0;

    /** 
     * @hidden
     * @deprecated since 325.1, use IAudioMixingManager instead
     * @type api
     * @region  mix
     * @brief  Uninstall all sound files
     * @notes   <br>
     *       + Call this method to uninstall all sound files. If the sound files were not uninstalled when the engine was destroyed, the SDK will automatically call this method to uninstall all sound files.
     *  <br>
     *       + Call LeaveRoom{@link #LeaveRoom}
     *  After leaving the room, this method will not be called automatically to uninstall all sound files. If you need to uninstall all sound files after leaving the room, you need the app to call this method actively.
     */
    virtual void UnloadAllEffects() = 0;

    /** 
     * @hidden
     * @deprecated since 325.1, use IAudioMixingManager instead
     * @type api
     * @region  mixing
     * @brief  Start/stop external audio stream mixing,  and set the mixing data format
     * @param  [in] enable <br>
     *        + True: Enable external audio stream mixing. <br>
     *        + False: Stop mixing external audio streams.
     * @param  [in] sample_rate <br>
     *         Audio sample rate in HZ. Currently supported sampling rates are: 8000HZ, 16000HZ, 32000HZ, 44100HZ, 48000HZ.
     * @param  [in] channel_num <br>
     *         Number of audio channels. Currently supports single channel (1), dual channel (2)
     * @notes   <br>
     *       + Call this method to set the PCM format of the external audio mix, that is, the
     * Audio_frame audio data format of PushAudioMixingStreamData{@link #PushAudioMixingStreamData}. <br>
     *       + Call the data source external cache of the method mix, and the audio format is PCM; StartAudioMixing{@link #IAudioMixingManager#StartAudioMixing}
     *  The method mixes the data source external file, and the supported file formats are: mp3, aac, m4a, 3gp, wav. These two mixing methods can coexist. <br>
     *       + Enable is false to stop the external audio stream mixing. Or when the engine is released, the SDK will stop mixing internally. <br>
     *       + The external audio stream mix initiated by this method does not support volume adjustment, pause, pause and resume operations.
     */
    virtual void SetAudioPlayoutMixStream(bool enable, int sample_rate, int channel_num) = 0;

    /** 
     * @hidden
     * @deprecated since 325.1, use IAudioMixingManager instead
     * @type api
     * @region Audio Mixing
     * @brief Get the SDK current buffered data
     * @return   <br>
     *        + > 0: Success, the number of buffered audio sample points. <br>
     *        + 0: Failure
     * @notes
     *       Call this method to obtain the amount of cached data in real time, and adjust the data push rhythm based on this data to avoid out of memory
     */
    virtual int GetAudioMixingStreamCachedFrameNum() = 0;

    /** 
     * @hidden
     * @deprecated since 325.1, use IAudioMixingManager instead
     * @type api
     * @region Audio Mixing
     * @brief Push the audio data of the mix to the SDK
     * @param  [in] audio_frame <br>
     *        PCM audio data, and its format is consistent with SetAudioPlayoutMixStream{@link #SetAudioPlayoutMixStream}.
     * @param  [in] frame_num <br>
     *         The number of sample points should be the sample_rate/100 set in SetAudioPlayoutMixStream{@link #SetAudioPlayoutMixStream}.
     *         When the sample rate is set to 48000, 480 sample points should be pushed each time
     * @return
     *        + True: success <br>
     *        + False: failure
     * @notes
     *       + Before calling this method, you must use the SetAudioPlayoutMixStream{@link #SetAudioPlayoutMixStream} method to set the data format of audio_frame
     * . <br>
     *       + Before calling this method, get the cached data amount through GetAudioMixingStreamCachedFrameNum
     * , And adjust the rhythm of the push data to avoid internal cache overflow causing push failure. <br>
     *       + Use reference suggestions: For the first time to push data, please cache certain data (such as 200 milliseconds) on the application side first,
     *  And then push it all at once; after that, the push operation is timed once in 10 milliseconds, and each time The amount of audio data is 10 milliseconds.
     */
    virtual bool PushAudioMixingStreamData(int8_t* audio_frame, int frame_num) = 0;

#ifndef ByteRTC_AUDIO_ONLY

    /** 
     * @hidden (Linux)
     * @type api
     * @region Video management
     * @brief Enable internal video capture immediately. The default is off.   <br>
     *        Internal video capture refers to: use the RTC SDK built-in video capture module to capture.<br>
     *        The local client will be informed via OnVideoDeviceStateChanged{@link #IRtcEngineLiteEventHandler#OnVideoDeviceStateChanged} after starting video capture by calling this API.
     *        The remote clients in the room will be informed of the state change via OnUserStartVideoCapture{@link #IRTCRoomEventHandler#OnUserStartVideoCapture} after the visible client starts video capture by calling this API.<br>
     * @notes   <br>
     *        + Call StopVideoCapture{@link #IRtcEngineLite#StopVideoCapture} to stop the internal video capture. Otherwise, the internal video capture will sustain until you destroy the engine instance.<br>
     *        + Once you create the engine instance, you can start internal video capture regardless of the video publishing state. The video stream will start publishing only after the video capture starts. <br>
     *        + To switch from custom to internal video capture, stop publishing before disabling the custom video capture module and then call this API to enable the internal video capture.<br>
     */
    virtual void StartVideoCapture() = 0;

    /** 
     * @hidden (Linux)
     * @type api
     * @region Video management
     * @briefDisable internal video capture immediately. The default is off.   <br>
     *        Internal video capture refers to: use the RTC SDK built-in video capture module to capture.<br>
     *        The local client will be informed via OnVideoDeviceStateChanged{@link #IRtcEngineLiteEventHandler#OnVideoDeviceStateChanged} after stopping video capture by calling this API.<br>
     *        The remote clients in the room will be informed of the state change via OnUserStopVideoCapture{@link #IRTCRoomEventHandler#OnUserStopVideoCapture} after the visible client stops video capture by calling this API.<br>
     * @notes   <br>
     *        + Call StartVideoCapture{@link #IRtcEngineLite#StartVideoCapture} to enable the internal video capture. <br>
     *        + Without calling this API the internal video capture will sustain until you destroy the engine instance.
     */
    virtual void StopVideoCapture() = 0;

   /** 
    * @hidden(Linux)
    * @type api
    * @region  Video Management
    * @brief  Set the video capture parameters for internal capture of the RTC SDK. <br>
    *         If your project uses the SDK internal capture module, you can specify the video capture parameters including preference, resolution and frame rate through this interface.
    * @param videoCaptureConfig  Video capture parameters. See: VideoCaptureConfig{@link #VideoCaptureConfig}.
    * @return   <br>
    *         + 0: Success; <br>
    *         + < 0: Failure; <br>
    * @notes   <br>
    *  + This interface can be called after the engine is created and takes effect immediately after being called. It is recommended to call this interface before calling StartVideoCapture{@link #IRtcEngineLite#StartVideoCapture}.
    *  + It is recommended that different Engines on the same device use the same video capture parameters.
    *  + If you used the internal module to start video capture before calling this interface, the capture parameters default to Auto.
    */
    virtual int SetVideoCaptureConfig(const VideoCaptureConfig& videoCaptureConfig) = 0;
    /** 
     * @type api
     * @region Video Management
     * @brief Enables/Disables the mode of publishing multiple video streams with different encoding configuration.
     * @param [in] enabled Whether to enable the mode of publishing multiple video streams:  <br>
     *        + True: Yes  <br>
     *        + False: No(Default setting)
     * @return API call result:  <br>
     *        + 0: Success  <br>
     *        + <0: Failure
     * @notes <br>
     *        + You can call this API  <br>
     *            - Before entering the room, or  <br>
     *            - After entering the room but before publishing streams.  <br>
     *        + After setting this API to "True", you can call [SetVideoEncoderConfig](#IRtcEngineLite-setvideoencoderconfig-1) to set encoding configuration for each stream.  <br>
     *        + If this function is off, or if this function is on but you don't set the configuration of any stream, only one stream will be sent with a resolution of 640px × 360px and a frame rate of 15fps.
     */
    virtual int EnableSimulcastMode(bool enabled) = 0;
    /**
     * {en}
     * @type api
     * @region Video Management
     * @brief <span id="IRtcEngineLite-setvideoencoderconfig-1"></span>
     *        Video publisher call this API to set the parameters of the maximum resolution video stream that is expected to be published, including resolution, frame rate, bitrate, scale mode, and fallback strategy in poor network conditions.
     *        You can only set configuration for one stream with this API. If you want to set configuration for multiple streams, Call [SetVideoEncoderConfig](#IRtcEngineLite-setvideoencoderconfig-2).
     * @param [in] max_solution The maximum video encoding parameter. See VideoEncoderConfig{@link #VideoEncoderConfig}.
     * @return  API call result: <br>
     *        + 0: Success <br>
     *        + ! 0: Failure <br>
     * @notes  <br>
     *        + If you call this API after enabling the mode of publishing multiple streams with EnableSimulcastMode{@link #IRtcEngineLite#EnableSimulcastMode}, SDK will automatically adjust the number of streams published and the parameters of each published stream according to the settings of subscribers. Up to 4 streams will be published, and the resolution you set in this API will be considered as the largest resolution among these 4 streams, see [Publish Multiple Streams](https://www.volcengine.com/docs/6348/70139) for details. Until you enable the mode of publishing multiple streams, SDK will only publish the stream you set.  <br>
     *        + Without calling this API, SDK will only publish one stream for you with a resolution of 640px × 360px and a frame rate of 15fps.  <br>
     *        + This API is applicable to the video stream captured by the camera, see SetScreenVideoEncoderConfig{@link #IRtcEngineLite#SetScreenVideoEncoderConfig} for setting parameters for screen sharing video stream.
     */
    virtual int SetVideoEncoderConfig(const VideoEncoderConfig& max_solution) = 0;
    /** 
     * @type api
     * @region Video Management
     * @brief <span id="IRtcEngineLite-setvideoencoderconfig-2"></span>
     *        Video publisher call this API to set the configurations of each stream published, including resolution, frame rate, bitrate, scale mode, and fallback strategy in poor network conditions.
     * @param [in] channel_solutions List of configurations for multiple streams. You can set parameters for up to 3 streams, SDK will always take the value of the first 3 streams when parameters for more streams are set.
     *        You should note that the resolution you set is the maximum resolution of each stream, and must be arranged from largest to smallest. See VideoEncoderConfig{@link #VideoEncoderConfig}.
     * @param [in] solution_num Numbers of streams
     * @return  API call result: <br>
     *        + 0: Success <br>
     *        + ! 0: Failure <br>
     * @notes  <br>
     *        + If you call this API after enabling the mode of publishing multiple streams with EnableSimulcastMode{@link #IRtcEngineLite#EnableSimulcastMode}, SDK will publish streams as you set, otherwise only the stream with the largest resolution you set will be published.  <br>
     *        + Without calling this API to set parameters for multiple video streams, the SDK only publishes one video stream with a resolution of 640px × 360px and a frame rate of 15fps.  <br>
     *        + After setting parameters for multiple video streams, SDK will automatically match the streams to be published based on the desired subscription parameters set by subscribers, see [Publish Multiple Streams](https://www.volcengine.com/docs/6348/70139) for details.  <br>
     *        + This API is applicable to the video stream captured by the camera, see SetScreenVideoEncoderConfig{@link #IRtcEngineLite#SetScreenVideoEncoderConfig} for setting parameters for screen sharing video stream.
     */
    virtual int SetVideoEncoderConfig(const VideoEncoderConfig* channel_solutions, int solution_num) = 0;
    /** 
     * @type api
     * @region Screen Shareing Management
     * @brief Video publisher call this API to set the expected configurations for the screen sharing video stream, including resolution, frame rate, bitrate, scale mode, and fallback strategy in poor network conditions.
     * @param [in] screen_solution You expected configurations for screen sharing video stream. See VideoEncoderConfig{@link #VideoEncoderConfig}.
     * @return  API call result: <br>
     *        + 0: Success <br>
     *        + ! 0: Failure <br>
     * @notes Without calling this API, the default encoding parameters for screen sharing video streams are: resolution 1920px × 1080px, frame rate 15fps.
     */
    virtual int SetScreenVideoEncoderConfig(const VideoEncoderConfig& screen_solution) = 0;

    /** 
     * @hidden
     * @deprecated since 336, using SetVideoEncoderConfig instead.
     * @type api
     * @region  Video management
     * @brief  Sets the video encoder configuration of each individual stream for simulcasting, including resolution, frame rate, bitrate, scale mode, fallback strategy under poor network conditions, etc.
     * @param  [in] index Stream type. See StreamIndex{@link #StreamIndex}.
     * @param  [in] solutions    Video parameters of each individual stream for simulcasting. See VideoSolution{@link #VideoSolution}.
     *                           The maximum resolution is 4096px × 4096px. If the resolution exceeds the limit or the video cannot be encoded, the simulcasting will fail.
     * @param  [in] solution_num The length of `solutions`. <br>
     *                           The length of `solutions` is no more than 4. The resolutions must be in descending sort. <br>
     * @return   <br>
     *         + 0: Success <br>
     *         +! 0: Failure <br>
     * @notes   <br>
     *        + When using the internal capture, the resolution and frame rate will be adapted to the maximum resolution and frame rate for encoding. <br>
     *        + The video streams are encoded by default with resolution of 640px × 360px and frame rate of 15fps. <br>
     *        When you call this API, it will take effect immediately, which may cause the camera to restart. <br>
     *        + The screen stream takes the first set of parameters.
     */
    virtual int SetVideoEncoderConfig(StreamIndex index, const VideoSolution* solutions, int solution_num) = 0;

    /** 
     * @hidden
     * @deprecated since 329.1, use SetVideoEncoderConfig instead
     * @type api
     * @region Video Management
     * @brief Start pushing multiple video streams, and set the video parameters when pushing multiple streams,
     *        Including resolution, Frame rate, bit rate, zoom mode, fallback strategy when the network is poor, etc.
     * @param  [in] solutions The first address of the video parameter array. See VideoSolution{@link #VideoSolution}.
     * @param  [in] solution_num Video parameter array length. <br>
     *                           Up to 3 parameters are supported. When multiple parameters are set, the resolution must be sequentially reduced, arranged from large to small. <br>
     *                           There is no limit to the maximum resolution. However, if the set resolution cannot be encoded, the encoding push stream will fail.
     * @return   <br>
     *         + 0: Success <br>
     *         +! 0: Failure <br>
     * @notes   <br>
     *         + When using internal capture, the resolution and frame rate of video capture will be adapted according to encoding parameters. <br>
     *         + The default video encoding parameters are: resolution 640px × 360px, frame rate 15fps. <br>
     *         + Changing the encoding resolution takes effect immediately, which may cause the camera to restart.
     */
    virtual int SetVideoEncoderConfig(const VideoSolution* solutions, int solution_num) = 0;

    /** 
     * @hidden (Linux)
     * @type api
     * @region Video Management
     * @brief Sets the view to be used for local video rendering and the rendering mode.  <br>
     * @param [in] index Video stream type. See StreamIndex{@link #StreamIndex}.
     * @param [in] canvas View information and rendering mode. See VideoCanvas{@link #VideoCanvas}.
     * @return  <br>
     *        + 0: Success <br>
     *        + -1: Failure <br>
     * @notes  <br>
     *       + You should bind your stream to a view before joining the room. This setting will remain in effect after you leave the room. <br>
     *       + If you need to unbind the local video stream from the current view, you can call this API and set the videoCanvas to `null`.
     */
    virtual int SetLocalVideoCanvas(StreamIndex index, const VideoCanvas& canvas) = 0;
    /** 
     * @hidden(Linux)
     * @type api
     * @region Video management
     * @brief Update the render mode and background color of local video rendering
     * @param [in] index See StreamIndex{@link #StreamIndex}.
     * @param [in] renderMode See RenderMode{@link #RenderMode}.
     * @param [in] backgroundColor See VideoCanvas{@link #VideoCanvas}.background_color.
     * @notes Calling this API during local video rendering will be effective immediately.
     */
    virtual void UpdateLocalVideoCanvas(StreamIndex index, const enum RenderMode renderMode, const uint32_t backgroundColor) = 0;

    /** 
     * @type api
     * @hidden(Linux)
     * @region Video Management
     * @brief Set the view and mode for rendering the specific video stream. <br>
     *        If you need to unbind the canvas for the video stream, set the canvas to `Null`.
     * @param  [in] stream_key See RemoteStreamKey{@link #RemoteStreamKey}.
     * @param  [in] canvas canvas and rendering mode. See VideoCanvas{@link #VideoCanvas}
     * @return   <br>
     *         + 0: Success <br>
     *         +!0: Failure <br>
     * @notes   <br>
     *        + Set the canvas after receiving OnUserJoined{@link #IRTCRoomEventHandler#OnUserJoined} or OnFirstRemoteVideoFrameDecoded{@link #IRTCRoomEventHandler#OnFirstRemoteVideoFrameDecoded}. The difference between these two callbacks is that if the video recording function is enabled, the video recording service will join the room as a dumb client side, so other client sides will receive OnUserJoined{@link #IRTCRoomEventHandler#OnUserJoined} callback. <br>
     *       + When the local user leaves the room, the setting will be invalid. The remote user leaving the room does not affect the setting.
     */
    virtual void SetRemoteStreamVideoCanvas(RemoteStreamKey stream_key, const VideoCanvas& canvas) = 0;
    /** 
     * @hidden(Linux)
     * @type api
     * @region Video management
     * @brief Update the render mode and background color of remote video rendering
     * @param [in] stream_key See RemoteStreamKey{@link #RemoteStreamKey}.
     * @param [in] renderMode See RenderMode{@link #RenderMode}.
     * @param [in] backgroundColor See VideoCanvas{@link #VideoCanvas}.background_color.
     * @notes Calling this API during remote video rendering will be effective immediately.
     */
    virtual void UpdateRemoteStreamVideoCanvas(RemoteStreamKey stream_key, const enum RenderMode renderMode, const uint32_t backgroundColor) = 0;

    /** 
     * @hidden (Linux)
     * @type api
     * @region Custom Video Capturing & Rendering
     * @brief Binds the local video stream to a custom renderer.
     * @param [in] index Video stream type. See StreamIndex{@link #StreamIndex}.
     * @param [in] video_sink Custom video renderer. See IVideoSink{@link #IVideoSink}.
     * @param [in] required_format Video frame encoding format that applys to custom rendering. See PixelFormat{@link #PixelFormat}.
     * @notes  <br>
     *        + RTC SDK uses its own renderer (internal renderer) for video rendering by default.  <br>
     *        + Joining or leaving the room will not affect the binding state. <br>
     *        + If you need to unbind the video stream from the custom renderer, you must set video_sink to `null`.  <br>
     *        + You should call this API before joining the room, and after receiving OnFirstLocalVideoFrameCaptured{@link #IRTCRoomEventHandler#OnFirstLocalVideoFrameCaptured} which reports that the first local video frame has been successfully captured.
     */
    virtual void SetLocalVideoSink(
            StreamIndex index, IVideoSink* video_sink, IVideoSink::PixelFormat required_format) = 0;

    /** 
     * @hidden (Linux)
     * @type api
     * @region Custom Video Capturing & Rendering
     * @brief Binds the remote video stream to a custom renderer.
     * @param [in] stream_key Remote stream information which specifys the source and type of the video stream to be rendered. See RemoteStreamKey{@link #RemoteStreamKey}.
     * @param [in] video_sink Custom video renderer. See IVideoSink{@link #IVideoSink}.
     * @param [in] required_format Encoding format which applys to the custom renderer. See PixelFormat{@link #PixelFormat}.
     * @notes   <br>
     *        + RTC SDK uses its own renderer (internal renderer) for video rendering by default.  <br>
     *        + Joining or leaving the room will not affect the binding state. <br>
     *         + This API can be called before and after entering the room. To call before entering the room, you need to get the remote stream information before joining the room; if you cannot get the remote stream information in advance, you can call the API after joining the room and getting the remote stream information via OnUserPublishStream{@link #IRTCRoomEventHandler#OnUserPublishStream}.
     *         + If you need to unbind, you must set videoSink to null.
     */
    virtual void SetRemoteVideoSink(RemoteStreamKey stream_key, IVideoSink* video_sink,
                                   IVideoSink::PixelFormat required_format) = 0;

    /** 
     * @hidden
     * @deprecated since 336.1, use UnpublishStream PublishStream instead.
     * @type api
     * @region Video management
     * @brief Enable/ Disable the transmission of the local video stream.  The video stream is not sending by default.<br>
     *        Call this API to start sending local video streams when using either internal capture module or custom video source. <br>
     *         Remote users in the room will be informed via OnUserMuteVideo{@link #IRTCRoomEventHandler#OnUserMuteVideo} after the visible user changes the transmission state of the video streams by calling this API.
     * @param   [in] muteState An identifier denoting whether the local video stream is sending. Refer to MuteState{@link #MuteState} for more details. <br>
     * @notes  <br>
     *         + Calling this API toggles the transmission state of the local video streams instead of the capture state.
     *         + You can enable video transmission regardless of the video capture state. Enable the transmission before capture, thus the video stream can be sent once the capture starts.
     */
    virtual void MuteLocalVideo(MuteState muteState) = 0;

    /** 
     * @hidden
     * @type api
     * @region Audio management
     * @brief Send screen share local audio data
     * @param  [in] pData
     *        Audio data
     * @param  [in] nSamples
     *        Sample number
     * @param  [in] nBytesPerSample
     *        Single sample data length
     * @param  [in] nChannels
     *        Audio channel number
     * @param  [in] sampleRate
     *        Sample rate
     * @return   <br>
     *         + True: success; <br>
     *         + False: failure; <br>
     * @notes   <br>
     *        + Only video frames in YUV420P format are supported for the time being. <br>
     *        + This function runs in the user calling thread. Before destroying the RtcEngine instance, please stop calling this function to push screen shared data <br>
     */
    virtual void PushScreenAudioFrame(const void* pData, const size_t nSamples, const size_t nBytesPerSample,
            const size_t nChannels, const uint32_t sampleRate) = 0;

    /** 
     * @hidden (macOS, Windows, Linux)
     * @type api
     * @region Video management
     * @brief Toggle the front/postcondition camera used for internal video capture  <br>
     *        After calling this interface, OnVideoDeviceStateChanged{@link #IRtcEngineLiteEventHandler#OnVideoDeviceStateChanged} callback will be triggered locally.
     * @param   [in] Camera ID, mobile end camera. See CameraID{@link #CameraID}.
     * @return   <br>
     *         + 0: Success <br>
     *         +! 0: Failed <br>
     * @notes   <br>
     *        + Default uses front-facing camera.
     *        + If you are using the camera for video capture, the toggle operation will take effect immediately; if the camera is not activated, the set camera will be turned on when the internal capture is turned on later.
     *        + If you have multiple cameras locally and want to select a specific working camera, you can control it through IVideoDeviceManager{@link #IVideoDeviceManager}. <br>
     */
    virtual int SwitchCamera(CameraID camera_id) = 0;

    /** 
     * @type api
     * @region Screen sharing
     * @brief Push screen video frame.
     * @param  [in] frame Set the screen video frame, see: IVideoFrame{@link #IVideoFrame}. <br>
     * @return   <br>
     *         + True: success; <br>
     *         + False: failure; <br>
     * @notes   <br>
     *        + Only video frames in YUV420P format are supported for the time being. <br>
     *        + This function runs in the user calling thread. Before destroying the RtcEngine instance, please stop calling this function to push screen shared data <br>
     */
    virtual bool PushScreenFrame(IVideoFrame* frame) = 0;

    /** 
     * @hidden
     * @deprecated since 327.1, use StartScreenVideoCapture and PublishScreen instead
     * @type api
     * @region Screen Sharing
     * @brief This method shares a window or part of the window. The user needs to specify the window id that he wants to share in this method.
     * @param  [in] window_id
     *        Specify the window id to be shared, see: view_t{@link #view_t}
     * @param  [in] region_rect
     *        Specify the position of the shared area relative to the entire window, and share the entire window when the value is 0, see: Rectangle{@link #Rectangle}
     * @param  [in] capture_params
     *        Encoding parameter configuration for screen sharing, see: DesktopCaptureParameters {@link # DesktopCaptureParameters}
     * @return   <br>
     *         + 0: Success; <br>
     *         + -1: Failure; <br>
     * @notes   <br>
     *        + This function and PublishScreen{@link #IRtcRoom#PublishScreen} are mutually exclusive. When calling StopScreenCapture {@link
     *  #StopScreenCapture}, PublishScreen{@link #IRtcRoom#PublishScreen} can be used again.  <br>
     *        + This function and StopScreenCapture{@link #StopScreenCapture} are called in pairs. <br>
     *        + After receiving the OnFirstRemoteVideoFrameRendered{@link #IRTCRoomEventHandler#OnFirstRemoteVideoFrameRendered}
     *          Event, set the remote screen sharing view by calling the SetRemoteVideoCanvas{@link #SetRemoteVideoCanvas} or SetRemoteVideoSink{@link #SetRemoteVideoSink}
     *          Function. <br>
     *        + Locally, you can call the SetLocalVideoCanvas{@link #IRtcEngineLite#SetLocalVideoCanvas} or SetLocalVideoSink {@link
     *  #SetLocalVideoSink} Function to set the local screen share view. <br>
     *        + You can also get the original data source by registering the RegisterVideoFrameObserver{@link #RegisterVideoFrameObserver} video callback object, listening for the
     * OnLocalScreenFrame{@link #IVideoFrameObserver#OnLocalScreenFrame} local screen video callback event and the
     * OnRemoteScreenFrame{@link #IVideoFrameObserver#OnRemoteScreenFrame} remote screen share video callback event. <br>
     */
    virtual int StartScreenCaptureByWindowId(
            view_t window_id, const Rectangle& region_rect, const DesktopCaptureParameters& capture_params) = 0;

    /** 
     * @hidden
     * @deprecated since 327.1, use StartScreenVideoCapture and PublishScreen instead
     * @type api
     * @region Screen Sharing
     * @brief Share the screen by specifying an area to share a screen or part of the screen. The user needs to specify the screen area that he wants to share in this method.
     * @param  [in] screen_rect
     *         Specify the position of the screen to be shared relative to the virtual screen. The virtual screen refers to the image area composed of all screens. For details, see: Rectangle{@link #Rectangle}
     * @param  [in] region_rect
     *         Specify the position of the area to be shared relative to the screen_rect. For details, see: Rectangle{@link #Rectangle}
     * @param  [in] capture_params
     *         The encoding parameter configuration of screen sharing, see: DesktopCaptureParameters {@Link #DesktopCaptureParameters}
     * @return   <br>
     *         + 0: Success; <br>
     *         + -1: Failure; <br>
     * @notes   <br>
     *        + This function and PublishScreen{@link #IRtcRoom#PublishScreen} are mutually exclusive to use when calling StopScreenCapture {@link
     *  #StopScreenCapture} Before using PublishScreen{@link #IRtcRoom#PublishScreen}. <br>
     *        + This function and StopScreenCapture{@link #StopScreenCapture} are called in pairs <br>
     *        + After receiving the OnFirstRemoteVideoFrameRendered{@link #IRTCRoomEventHandler#OnFirstRemoteVideoFrameRendered}
     *          Event, set the remote screen sharing view by calling the
     * SetRemoteVideoCanvas{@link #SetRemoteVideoCanvas} or SetRemoteVideoSink{@link #SetRemoteVideoSink}
     *  Function. <br>
     *        + Locally, you can call the SetLocalVideoCanvas{@link #IRtcEngineLite#SetLocalVideoCanvas} or SetLocalVideoSink {@link
     *  #SetLocalVideoSink} Function to set the local screen share view. <br>
     *        + You can also get the original data source by registering the RegisterVideoFrameObserver{@link #RegisterVideoFrameObserver} video callback object, listening for the
     * OnLocalScreenFrame{@link #IVideoFrameObserver#OnLocalScreenFrame} local screen video callback event and the
     * OnRemoteScreenFrame{@link #IVideoFrameObserver#OnRemoteScreenFrame} remote screen share video callback event. <br>
     */
    virtual int StartScreenCaptureByScreenRect(const Rectangle& screen_rect, const Rectangle& region_rect,
            const DesktopCaptureParameters& capture_params) = 0;

    /** 
     * @hidden
     * @deprecated since 327.1, use StartScreenVideoCapture and PublishScreen instead
     * @type api
     * @region screen sharing
     * @brief Share the screen through the screen id, share a screen or part of the screen. Users need to specify the screen id they want to share in this method.
     * @param  [in] display_id
     *         Specify the screen id to be shared.
     * @param  [in] region_rect
     *         Specify the location of the area to be shared relative to the entire screen, see: Rectangle{@link #Rectangle}
     * @param  [in] capture_params
     *        The encoding parameter configuration for screen sharing, see: DesktopCaptureParameters{@link #DesktopCaptureParameters}
     * @return   <br>
     *         + 0: Success; <br>
     *         + -1: Failure; <br>
     * @notes  Share a window or part of the window. You need to specify the window id you want to share in this method.
     */
    virtual int StartScreenCaptureByDisplayId(
            unsigned int display_id, const Rectangle& region_rect, const DesktopCaptureParameters& capture_params) = 0;

    /** 
     * @hidden (macOS, Windows,iOS)
     * @deprecated since 327.1, use StartScreenVideoCapture and PublishScreen instead
     * @type api
     * @region screen sharing
     * @brief Open screen sharing through the incoming Context.
     * @param  [in] context
     *        Android platform is MediaProjection object, obtained by the business party after applying for system recording permission, not used on the iOS, just pass empty.
     * @param  [in] group_id
     *        IOS platform needs to pass in the group id used by App and Extension, and the Android platform can pass empty.
     * @param  [in] capture_params
     *         Screen sharing encoding parameter configuration, see: DesktopCaptureParameters{@link #DesktopCaptureParameters}
     * @return   <br>
     *         + 0: success; <br>
     *         + -1: Failure; <br>
     * @notes  Turn on screen sharing, mobile end dedicated interface.
     */
    virtual int StartScreenCapture(
            void* context, const char* group_id, const DesktopCaptureParameters& capture_params) = 0;

    /** 
     * @hidden (iOS, Android, Linux)
     * @type api
     * @region Screen sharing
     * @brief Update the capture area when capturing screen video streams through the capture module provided by the RTC SDK.
     * @param   [in] region_rect Acquisition area. See Rectangle{@link #Rectangle} <br>
     *                           This parameter describes the acquisition area after calling this interface, and the relative relationship between the 'source_info' setting area in StartScreenVideoCapture{@link #StartScreenVideoCapture}.
     * @notes Before calling this interface, internal screen stream capture must have been turned on by calling StartScreenVideoCapture{@link #StartScreenVideoCapture}.
     */
    virtual void UpdateScreenCaptureRegion(const Rectangle& region_rect) = 0;

    /** 
     * @hidden
     * @deprecated since 327.1, use SetVideoEncoderConfig, UpdateScreenCaptureHighlightConfig,
     *                  UpdateScreenCaptureFilterConfig, UpdateScreenCaptureMouseCursor instead
     * @type api
     * @region ScreenShare
     * @brief Update the configuration of the encoding parameters of the screen share.
     * @param  [in] capture_params
     *         Screen share encoding parameter configuration, see: DesktopCaptureParameters{@link #DesktopCaptureParameters}
     * @return   <br>
     *         + 0: success; <br>
     *         + -1: Failure; <br>
     */
    virtual int UpdateScreenCaptureParameters(const DesktopCaptureParameters& capture_params) = 0;

    /** 
     * @hidden (iOS, Android, Linux)
     * @type api
     * @region  screen sharing
     * @brief  Capture screen video stream for sharing. Screen video stream includes: content displayed on the screen, in the application window, or in the virtual screen. <br>
     *        The content displayed in the virtual screen is only supported on Windows.
     * @param  [in] source_info Screen capture source information. See ScreenCaptureSourceInfo{@link #ScreenCaptureSourceInfo}. <br>
     *                          Call GetScreenCaptureSourceList{@link #IRtcEngineLite#GetScreenCaptureSourceList} to get all the screen sources that can be shared.
     * @param  [in] capture_params Screen capture parameters. See ScreenCaptureParameters{@link #ScreenCaptureParameters}.
     * @return   <br>
     *         + 0: Success; <br>
     *         + -1: Failure; <br>
     * @notes   <br>
     *        + This API only starts screen capturing but does not publish the captured video. Call PublishScreen{@link #IRtcRoom#PublishScreen} to publish the captured video. <br>
     *        + To turn off screen video capture, call StopScreenVideoCapture{@link #IRtcEngineLite#StopScreenVideoCapture}. <br>
     *        + Local users will receive OnVideoDeviceStateChanged{@link #IRtcEngineLiteEventHandler#OnVideoDeviceStateChanged} on the state of screen capturing such as start, pause, resume, and error. <br>
     *        + After successfully calling this API, local users will receive OnFirstLocalVideoFrameCaptured{@link #IRTCRoomEventHandler#OnFirstLocalVideoFrameCaptured}. <br>
     *        + Before calling this API, you can call SetScreenVideoEncoderConfig{@link #IRtcEngineLite#SetScreenVideoEncoderConfig} to set the frame rate and encoding resolution of the screen video stream. <br>
     *        + After receiving OnFirstLocalVideoFrameCaptured{@link #IRTCRoomEventHandler#OnFirstLocalVideoFrameCaptured}, you can set the local screen sharing view by calling SetLocalVideoCanvas{@link #IRtcEngineLite#SetLocalVideoCanvas} or SetLocalVideoSink {@link #SetLocalVideoSink}. <br>
     *        + You can also register an observer by calling RegisterVideoFrameObserver{@link #RegisterVideoFrameObserver} and receive the captured screen video frame through OnLocalScreenFrame{@link #IVideoFrameObserver#OnLocalScreenFrame}.<br>
     */
    virtual int StartScreenVideoCapture(const ScreenCaptureSourceInfo& source_info, const ScreenCaptureParameters& capture_params) = 0;

    /** 
     * @hidden (macOS, Windows, iOS, Linux)
     * @deprecated since 336.1, use StartScreenCapture instead
     * @type api
     * @region  Screen sharing
     * @brief  Open screen sharing through the incoming Context.
     * @param  [in] context
     *        Android platform incoming MediaProjection object, obtained by the business party after applying for system recording permission.
     * @return   <br>
     *         + 0: Success; <br>
     *         + -1: Failure; <br>
     * @notes  Enable screen sharing, Android-specific interface.
     */
    virtual int StartScreenVideoCapture(void* context) = 0;

    /** 
     * @hidden (Linux)
     * @type api
     * @region screen sharing
     * @brief Stop screen video streaming.
     * @notes   <br>
     *        + To turn on screen video stream capture, calling StartScreenVideoCapture{@link #StartScreenVideoCapture} <br>
     *        + Local users will receive the OnVideoDeviceStateChanged{@link #IRtcEngineLiteEventHandler#OnVideoDeviceStateChanged} callback. <br>
     *        + Calling this interface does not affect screen video stream publishing. <br>
     */
    virtual void StopScreenVideoCapture() = 0;

    /** 
     * @hidden (iOS, Android, Linux)
     * @type api
     * @region Screen sharing
     * @brief Update border highlighting settings when capturing screen video streams through the capture module provided by the RTC SDK. The default display table box.
     * @param  [in] highlight_config Border highlighting settings. See HighlightConfig{@link #HighlightConfig}
     * @notes  Before calling this interface, you must have turned on internal screen flow collection by calling StartScreenVideoCapture{@link #StartScreenVideoCapture}.
     */
    virtual void UpdateScreenCaptureHighlightConfig(const HighlightConfig& highlight_config) = 0;

    /** 
     * @hidden (iOS, Android, Linux)
     * @type api
     * @region Screen sharing
     * @brief Update the processing settings for the mouse when capturing screen video streams through the capture module provided by the RTC SDK. Default acquisition mouse.
     * @param  [in] capture_mouse_cursor See MouseCursorCaptureState{@link #MouseCursorCaptureState}
     * @notes  Before calling this interface, internal screen stream capture must have been turned on by calling StartScreenVideoCapture{@link #StartScreenVideoCapture}.
     */
    virtual void UpdateScreenCaptureMouseCursor(MouseCursorCaptureState capture_mouse_cursor) = 0;

    /** 
     * @hidden (iOS, Android, Linux)
     * @type api
     * @region Screen sharing
     * @brief When capturing screen video streams through the capture module provided by the RTC SDK, set the window that needs to be filtered.
     * @param  [in] ScreenFilterConfig Window filtering settings. See ScreenFilterConfig{@link #ScreenFilterConfig} <br>
     * @notes  <br>
     *        + Before calling this interface, internal screen stream capture must have been turned on by calling StartScreenVideoCapture{@link #StartScreenVideoCapture}. <br>
     *        + This function only works when the screen source category is a screen rather than an application form. See: ScreenCaptureSourceType{@link #ScreenCaptureSourceType}
     */
    virtual void UpdateScreenCaptureFilterConfig(const ScreenFilterConfig& filter_config) = 0;

    /** 
     * @hidden (Linux)
     * @brief  Get the list of shared objects (application windows and desktops),  after using it, you need to call the corresponding release interface to release
     * @region  screen sharing
     * @notes   <br>
     *        + The enumerated form can be used as an input parameter when opening screen sharing, see: StartScreenVideoCapture{@link #IRtcEngineLite#StartScreenVideoCapture} <br>
     *        + This function For PC side use only
     */
    virtual IScreenCaptureSourceList* GetScreenCaptureSourceList() = 0;

    /** 
     * @hidden (Linux)
     * @brief  Get the shared object thumbnail
     * @region  screen share
     * @param type Type of the screen capture object. Refer to ScreenCaptureSourceType{@link #ScreenCaptureSourceType} for more details.
     * @param source_id ID of the screen-shared object. Refer to view_t{@link #view_t} for more details.
     * @param max_width Maximum width
     * @param max_height Maximum height
     */
    virtual IVideoFrame* GetThumbnail(
            ScreenCaptureSourceType type, view_t source_id, int max_width, int max_height) = 0;

    /** 
     * @hidden
     * @deprecated since 324.1, use SetVideoSourceType instead
     * @type api
     * @region Video Management
     * @brief Set whether custom video capture is enabled
     * @param   [in] enable <br>
     *        + True: On <br>
     *        + False: Off
     * @notes   <br>
     *        + 1. This method can be called before and after entering the room. <br>
     *        + 2. When you have called StartVideoCapture{@link #IRtcEngineLite#StartVideoCapture} to turn on internal collection, and then call this method to switch to custom collection, the SDK will automatically turn off internal collection. <br>
     *        + 3. When you call this method to turn on custom collection, to switch to internal collection, you must first call this method to turn off custom collection, and then call StartVideoCapture{@link #IRtcEngineLite#StartVideoCapture} to manually turn on internal collection.
     */
    virtual void SetExternalVideoSource(bool enable) = 0;

    /** 
     * @hidden
     * @deprecated since 329.1, use SetVideoSourceType instead
     * @type api
     * @region Video management
     * @brief Switch video capture methods (internal capture/custom capture)
     * @param   [in] type Video capture methods. See VideoSourceType{@link #VideoSourceType}
     * @notes   <br>
     *        + Default uses internal capture. Internal capture refers to video capture using the video capture mechanism built into the RTC SDK. <br>
     *        + This method can be called before and after entering the room. <br>
     *        + When you have called StartVideoCapture{@link #IRtcEngineLite#StartVideoCapture} to turn on internal collection, and then call this method to switch to custom collection, the SDK will automatically turn off internal collection. <br>
     *        + When you call this method to turn on custom collection, in order to switch to internal collection, you must first call this method to turn off custom collection, and then call StartVideoCapture{@link #IRtcEngineLite#StartVideoCapture} to manually turn on internal collection.
     */
    virtual void SetVideoSourceType(VideoSourceType type) = 0;

    /** 
     * @type api
     * @region Video management
     * @brief Set the video source of the RTC SDK.
     *        The internal video capture is the default, which refers to capturing video using the built-in module.
     * @param  [in] stream_index Stream index. Refer to StreamIndex{@link #StreamIndex} for more details.
     * @param  [in] type Video source type. Refer to VideoSourceType{@link #VideoSourceType} for more details.
     * @notes   <br>
     *         + You can call this API whether the user is in a room or not. <br>
     *         + Calling this API to switch to the custom video source will stop the enabled internal video capture.
     *         + To switch to internal video capture, call this API to stop custom capture and then call StartVideoCapture{@link #IRtcEngineLite#StartVideoCapture} to enable internal video capture.
     *         + To push custom encoded video frames to the SDK, call this API to switch `VideoSourceType` to `VideoSourceTypeEncodedWithAutoSimulcast` or `VideoSourceTypeEncodedWithoutAutoSimulcast`.
     */
    virtual void SetVideoSourceType(StreamIndex stream_index, VideoSourceType type) = 0;

    /** 
     * @type api
     * @region  Video Management
     * @brief  Send external source video data
     * @param  [in] frame Set the video frame. See IVideoFrame{@link #IVideoFrame}. <br>
     * @notes   <br>
     *        + Only video frames in YUV420P format are supported for the time being. <br>
     *        + This function runs in the user calling thread <br>
     * @notes  Before pushing external video frames, you must call SetVideoSourceType{@link #SetVideoSourceType} to turn on external video source capture.
     */
    virtual bool PushExternalVideoFrame(IVideoFrame* frame) = 0;

    /**
     * @hidden
     */
    virtual void SetupDynamicLayoutRender(IVideoSink* render) = 0;
#endif

    /** 
     * @hidden (macOS, Windows)
     * @type api
     * @region Audio Facility Management
     * @brief Set the audio playback device, and use speakers by default.   <br>
     *         When the audio playback device changes, you will receive an OnAudioPlaybackDeviceChanged {@link
     *  #IRtcEngineLiteEventHandler #OnAudioPlaybackDeviceChanged} Callback.
     * @param  [in] device Audio playback device. See AudioPlaybackDevice{@link #AudioPlaybackDevice} <br>
     * @return  Method call result   <br>
     *         + 0: Success. <br>
     *         + < 0: failure <br>
     * @notes   <br>
     *        + 1. This interface is only available for mobile devices. <br>
     *        + 2. This method only supports setting audio & video playback devices as earphones or speakers. When the app connects to a wired or Bluetooth audio playback device, the SDK
     *  Automatically switches to a wired or Bluetooth audio playback device. Active setting as a wired or Bluetooth audio playback device will return Failure. <br>
     *        + 3.
     * If you connect a wired or Bluetooth audio playback device, setting the audio playback device as a speaker or earpiece will successfully call, but it will not immediately switch to the speaker or earpiece. After the wired or Bluetooth audio playback device is removed, it will automatically switch to the earpiece or speaker according to the settings.
     *  <br>
     *        + 4. This method can be called before and during a call. <br>
     */
    virtual int SetAudioPlaybackDevice(AudioPlaybackDevice device) = 0;

    /** 
     * @hidden (macOS, Windows)
     * @type api
     * @region Audio Facility Management
     * @brief Set the audio playback device, and use speakers by default.   <br>
     *         When the audio playback device changes, you will receive an OnAudioRouteChanged {@link
     *  #IRtcEngineLiteEventHandler #OnAudioRouteChanged} Callback.
     * @param  [in] device Audio playback device. See AudioRoute{@link #AudioRoute} <br>
     * @return  Method call result   <br>
     *         + 0: Success. <br>
     *         + < 0: failure <br>
     * @notes   <br>
     *        + 1. This interface is only available for mobile devices. <br>
     *        + 2. This method only supports setting audio & video playback devices as earphones or speakers. When the
     * app connects to a wired or Bluetooth audio playback device, the SDK Automatically switches to a wired or
     * Bluetooth audio playback device. Active setting as a wired or Bluetooth audio playback device will return
     * Failure. <br>
     *        + 3.
     * If you connect a wired or Bluetooth audio playback device, setting the audio playback device as a speaker or
     * earpiece will successfully call, but it will not immediately switch to the speaker or earpiece. After the wired
     * or Bluetooth audio playback device is removed, it will automatically switch to the earpiece or speaker according
     * to the settings. <br>
     *        + 4. This method can be called before and during a call.
     *        + 5. SetAudioRoute to kAudioRouteUnknown must be failed.<br>
     */
    virtual int SetAudioRoute(AudioRoute device) = 0;

    /** 
     * @hidden (macOS, Windows)
     * @type api
     * @region Audio Facility Management
     * @brief get current using audio playback device.   <br>
     *         When the audio playback device changes, you will receive an OnAudioRouteChanged{@link
     * #IRtcEngineLiteEventHandler #OnAudioRouteChanged} Callback.
     * @return  device current using Audio playback device. See AudioRoute{@link #AudioRoute}
     * @notes   <br>
     *        + 1. This interface is only available for mobile devices. <br>
     *        + 2. This method can be called before and during a call. <br>
     */
    virtual AudioRoute GetAudioRoute() = 0;

    /** 
     * @type api
     * @region Multiple rooms
     * @brief Create a room for the call <br>
     *        Each call of this API creates one IRtcRoom{@link #IRtcRoom} instance. Call this API as many times as the number of rooms you need.  And then call JoinRoom{@link #IRtcRoom#JoinRoom} of each `IRTCRoom` instance to join multiple rooms at the same time.  These enable Multi-room mode. In Multi-room mode, a user can subscribe to media streams in the joined rooms at the same time.<br>
     * @param  [in] room_id roomId is an non-empty string within 128 bytes. Each room created by calling this API requires a unique roomid. So that the room can be distinguished from each other.  <br>
     *        The following character sets are supported: <br>
     *        + 26 uppercase letters A~ Z <br>
     *        + 26 lowercase letters a~ z <br>
     *        + 10 numbers: 0~ 9 <br>
     *        + Underscore "_", at sign "@" , and minus sign "-" <br>
     * @return `IRtcRoom` instance <br>
     * @notes  To forward streams to the other rooms, call StartForwardStreamToRooms{@link #IRtcRoom#StartForwardStreamToRooms} instead of enabling Multi-room mode.
     */
    virtual IRtcRoom* CreateRtcRoom(const char* room_id) = 0;
    /** 
     * @type api
     * @region Audio & Video Fallback
     * @brief Sets the fallback option for published audio & video streams.   <br>
     *        You can call this API to set whether to automatically lower the resolution you set of the published streams under limited network conditions.
     * @param [in] option Fallback option, see PublishFallbackOption{@link #PublishFallbackOption}.
     * @return  API call result <br>
     *         + 0: Success <br>
     *         + < 0: Failure
     * @notes  <br>
     *         + This API only works after you call EnableSimulcastMode{@link #IRtcEngineLite#EnableSimulcastMode} to enable the mode of publishing multiple streams. <br>
     *         + You must call this API before entering the room.  <br>
     *         + After you allow video stream to fallback, your stream subscribers will receive OnSimulcastSubscribeFallback{@link #IRTCRoomEventHandler#OnSimulcastSubscribeFallback} when the resolution of your published stream are lowered or restored.  <br>
     *         + You can alternatively set fallback options with distrubutions from server side, which is of higher priority.
     */
    virtual int SetPublishFallbackOption(PublishFallbackOption option) = 0;
    /** 
     * @type api
     * @region Audio & Video Fallback
     * @brief Sets the fallback option for subscribed RTC streams.   <br>
     *        You can call this API to set whether to lower the resolution of currently subscribed stream under limited network conditions.
     * @param [in] option Fallback option, see SubscribeFallbackOption{@link #SubscribeFallbackOption} for more details.
     * @return API call result:  <br>
     *        + 0:  Success  <br>
     *        + < 0: Failure
     *  @notes  <br>
     *         + You must call this API before enterting the room.  <br>
     *         + After you enables the fallback, you will receive OnSimulcastSubscribeFallback{@link #IRTCRoomEventHandler#OnSimulcastSubscribeFallback} and OnRemoteVideoSizeChanged{@link #IRTCRoomEventHandler#OnRemoteVideoSizeChanged} when the resolution of your subscribed stream is lowered or restored.  <br>
     *         + You can alternatively set fallback options with distrubutions from server side, which is of higher priority.
     */
    virtual int SetSubscribeFallbackOption(SubscribeFallbackOption option) = 0;

    /** 
     * @type api
     * @region  audio & video fallback
     * @brief  Set user priority
     * @param  [in] user_id Remote user's ID
     * @param  [in] priority Remote user's requirement priority. See enumeration type RemoteUserPriority{@link #RemoteUserPriority}
     * @return 0:  Success. <br>
     *         + < 0: failure <br>
     * @notes   <br>
     *         + 1. This method is used with SetSubscribeFallbackOption{@link #SetSubscribeFallbackOption}. <br>
     *         + 2. If the subscription flow fallback option is turned on, weak connections or insufficient performance will give priority to ensuring the quality of the flow received by high-priority users. <br>
     *         + 3. This method can be used before and after entering the room, and the priority of the remote user can be modified. <br>
     */
    virtual int SetRemoteUserPriority(const char* user_id, RemoteUserPriority priority) = 0;

    /** 
     * @type api
     * @region Engine management
     * @brief  Sets the business ID   <br>
     *         You can use businessId to distinguish different business scenarios. You can customize your businessId to serve as a sub AppId, which can share and refine the function of the AppId, but it does not need authentication.
     * @param  [in] business_id <br>
     *         Your customized businessId
     *         BusinessId is a tag, and you can customize its granularity.
     * @return   <br>
     *         + 0: Success <br>
     *         + < 0: Failure. See BusinessCheckCode{@link #BusinessCheckCode} for specific reasons. <br>
     *         + -6001: The user is already in the room. <br>
     *         + -6002: The input is invalid. Legal characters include all lowercase letters, uppercase letters, numbers, and four other symbols, including '.', '-','_', and '@'.
     * @notes   <br>
     *         + You must call this API before the JoinRoom{@link #IRtcEngine#JoinRoom} API, otherwise it will be invalid.
     */
    virtual int SetBusinessId(const char* business_id) = 0;

    /** 
     * @hidden
     * @deprecated since 332.1, use EnableVirtualBackground instead
     * @type api
     * @region Video Effects
     * @brief Turn on the background and set the background mode and split mode
     * @param  [in] mode Background mode. See BackgroundMode{@link #BackgroundMode}. <br>
     * @param  [in] divide_model Segmentation model. See DivideMode{@link #DivideMode}. <br>
     * @return   <br>
     *         + 0: Success <br>
     *         +! 0: Failed <br>
     */
    virtual int ReplaceBackground(BackgroundMode mode, DivideMode divide_model) = 0;

    /** 
     * @hidden
     * @deprecated since 332.1, use DisableVirtualBackground instead
     * @type api
     * @region Video Effects
     * @brief  Close the background
     */
    virtual void DisableBackground() = 0;

    /** 
     * @hidden
     * @deprecated Since 329.1, use SetLocalVideoMirrorType instead
     * @type api
     * @region Video Management
     * @brief Set whether to turn on mirror mode when front-facing camera capture is used.  <br>
     * @param   [in] mirrorMode Whether to turn on mirror mode
     */
    virtual void SetLocalVideoMirrorMode(MirrorMode mirrorMode) = 0;

    /** 
     * @type api
     * @hidden(Windows,MacOS,Linux)
     * @brief Set the orientation of the video capture. By default, the App direction is used as the orientation reference.<br>
     *        During rendering, the receiving client rotates the video in the same way as the sending client did.<br>
     * @param [in] rotationMode Rotation reference can be the orientation of the App or gravity. Refer to VideoRotationMode{@link #VideoRotationMode} for details.
     * @return <br>
     *        + 0: Success
     *        + <0: Failure
     * @notes <br>
 *        + The orientation setting is effective for internal video capture only. That is, the orientation setting is not effective to the custom video source or the screen-sharing stream.
 *        + If the video capture is on, the setting will be effective once you call this API. If the video capture is off, the setting will be effective on when capture starts.
     */
    virtual int SetVideoRotationMode(VideoRotationMode rotationMode) = 0;

    /** 
     * @type api
     * @region Video management
     * @brief Sets the mirror mode for the captured video stream.
     * @param  [in] mirrorType Mirror type. See MirrorType{@link #MirrorType}.
     * @notes  <br>
     *         + Switching video streams does not affect the settings of the mirror type. <br>
     *         + This API is not applicable to screen-sharing streams. <br>
     *         + Before you call this API, the initial states of each video stream are as follows: <br>
     *         <table>
     *            <tr><th></th><th>Front-facing camera</th><th>Back-facing camera</th><th>Custom capturing</th><th>Built-in camera</th></tr>
     *            <tr><td>Mobile device</td><td>The preview is mirrored. The published video stream is not mirrored.</td><td>The preview and the published video stream are not mirrored.</td><td>The preview and the published video stream are not mirrored.</td><td>/</td></tr>
     *            <tr><td>PC</td><td>/</td><td>/</td><td>The preview and the published video stream are not mirrored.</td><td>The preview is mirrored. The published video stream is not mirrored.</td></tr>
     *         </table>
     */
    virtual void SetLocalVideoMirrorType(MirrorType mirrorType) = 0;

    /** 
     * @hidden
     * @type api
     * @region Video Effects
     * @brief  Get video effects interface
     * @return  Video effects interface pointer
     */
    virtual IVideoEffect* GetVideoEffectInterface() = 0;

    /** 
     * @hidden
     * @type api
     * @region  video management
     * @brief  Get camera control interface
     * @return  Camera control interface pointer
     */
    virtual ICameraControl* GetCameraControl() = 0;

    /** 
     * @type api
     * @region Encryption
     * @brief Set the way to use built-in encryption when transmitting  <br>
     * @param encrypt_type Built-in encryption algorithm. See EncryptType{@link #EncryptType}
     * @param  [in] encrypt_type Encryption type. See EncryptType{@link #EncryptType}
     * @param  [in] key Encryption key, the length is limited to 36 bits, and the excess will be Truncate
     * @param  [in] key_size The length of the parameter key
     * @notes   <br>
     *        + Use this method when using the built-in encryption on transfer; if you need to use custom encryption on transfer. See OnEncryptData {@link #IEncryptHandler#OnEncryptData}. Built-in encryption and custom encryption are mutually exclusive, and the transmission is determined to be encrypted according to the last method called. <br>
     *        + This method must be called before entering the room, and can be called repeatedly, taking the last called parameter as the effective parameter. <br>
     */
    virtual void SetEncryptInfo(EncryptType encrypt_type, const char* key, int key_size) = 0;

    /** 
     * @type api
     * @region Encryption
     * @brief Set custom encryption   <br>
     *        You need to implement the corresponding encryption/decryption method, refer to EncryptType{@link #EncryptType}. <br>
     * @param  [in] handler Custom encryption handler, you need to implement the handler's encryption and decryption method <br>
     * @notes   <br>
     *        + This method is mutually exclusive with SetEncryptInfo{@link #SetEncryptInfo}, you can only select custom encryption method or default encryption method. The encryption method that takes effect depends on the last method called. <br>
     *        + This method must be called before entering the room, and can be called repeatedly, taking the last called parameter as the effective parameter. <br>
     *        + Whether encrypted or decrypted, the length modification of the original data source needs to be controlled between 90% and 120%, i.e. If the input data is 100 bytes, the processed data must be between 90 and 120 bytes. Between bytes, if the encrypted or decrypted result exceeds the length limit, the audio & video frame will be discarded. <br>
     *        + Data encryption/decryption is performed serially, so depending on the implementation, it may affect the final rendering efficiency. Whether to use this method needs to be carefully evaluated by users.
     */
    virtual void SetCustomizeEncryptHandler(IEncryptHandler* handler) = 0;

    /** 
     * @type api
     * @region Audio Data Callback
     * @brief Enables audio data callback.
     * @param [in] method Audio data callback method. See AudioFrameCallbackMethod{@link #AudioFrameCallbackMethod}. <br>
     *               If `method` is set as `kAudioFrameCallbackRecord`, `kAudioFrameCallbackPlayback`, `kAudioFrameCallbackMixed`, `kAudioFrameCallbackRecordScreen`, set `format` to the accurate value listed in the audio parameters format.
     *               If `method` is set as `kAudioFrameCallbackRemoteUser`, set `format` to `auto`.
     * @param [in] format Audio parameters format. See AudioFormat{@link #AudioFormat}.
     * @notes After calling this API and RegisterAudioFrameObserver{@link #IRtcEngineLite#RegisterAudioFrameObserver}, IAudioFrameObserver{@link #IAudioFrameObserver} will receive the corresponding audio data callback. However, these two APIs are independent of each other and the calling order is not restricted. <br>
     */
    virtual void EnableAudioFrameCallback(AudioFrameCallbackMethod method, AudioFormat format) = 0;

    /** 
     * @type api
     * @region Audio Data Callback
     * @brief Disables audio data callback.
     * @param [in] method Audio data callback method. See AudioFrameCallbackMethod{@link #AudioFrameCallbackMethod}.
     * @notes Call this API after calling EnableAudioFrameCallback{@link #IRtcEngineLite#EnableAudioFrameCallback}.
     */
    virtual void DisableAudioFrameCallback(AudioFrameCallbackMethod method) = 0;

    /** 
     * @type api
     * @region Audio Data Callback
     * @brief Register an audio frame observer.  <br>
     * @param [in] observer Audio data callback observer. See IAudioFrameObserver{@link #IAudioFrameObserver}. Use `null` to cancel the registration.
     * @return  <br>
     *        + 0: Success. <br>
     *        + < 0: Failure. <br>
     * @notes After calling this API and EnableAudioFrameCallback{@link #IRtcEngineLite#EnableAudioFrameCallback}, IAudioFrameObserver{@link #IAudioFrameObserver} will receive the corresponding audio data callback.
     */
    virtual void RegisterAudioFrameObserver(IAudioFrameObserver* observer) = 0;


    /** 
     * @type api
     * @region  Audio Processing
     * @brief  Set up a custom audio processor.   <br>
     *        Using this processor, you can call ProcessAudioFrame{@link #IAudioProcessor#ProcessAudioFrame} to customize the audio frames captured by the RTC SDK and use the processed audio frames for RTC audio & video communication. <br>
     *        SDK only holds weak references to the processor, you should guarantee its Life Time.
     * @param  [in] processor Custom audio processor. See IAudioProcessor{@link #IAudioProcessor}. If null is passed in, the audio frames captured by the RTC SDK are not customized.
     * @param  [in] audioFormat Customize the audio parameter format. See AudioFormat{@link #AudioFormat}, the SDK will give the audio frame according to the specified settings.
     * @notes When this interface is repeatedly called, only the last call takes effect. The effects do not stack.
     */
    virtual void RegisterLocalAudioProcessor(IAudioProcessor* processor, AudioFormat audioFormat) = 0;

    /** 
     * @type api
     * @region Video Data Callback
     * @brief Register a video data callback observer
     * @param [in] observer Video data callback observer, see IVideoFrameObserver{@link #IVideoFrameObserver}. Cancel the registration by setting it to `null`.
     * @notes We recommended you call this API before calling JoinRoom{@link #IRtcEngine#JoinRoom}.
     */
    virtual void RegisterVideoFrameObserver(IVideoFrameObserver* observer) = 0;

    /** 
      * @type api
      * @region  Video Processing
      * @brief  Set up a custom video preprocessor. <br>
      *         Using this video preprocessor, you can call ProcessVideoFrame{@link #IVideoProcessor#ProcessVideoFrame} to preprocess the video frames collected by the RTC SDK, and use the processed video frames for RTC audio & video communication.
      * @param  [in] processor Custom video processor. See IVideoProcessor{@link #IVideoProcessor}. If null is passed in, the video frames captured by the RTC SDK are not preprocessed. <br>
      *        SDK only holds weak references to the processor, you should guarantee its Life Time.
      * @param  [in] config Customize the settings applicable to the video pre-processor. See VideoPreprocessorConfig{@link #VideoPreprocessorConfig}. <br>
      *                Currently, the 'required_pixel_format 'in'config' only supports: 'kVideoPixelFormatI420' and'kVideoPixelFormatUnknown ': <br>
      *                + When set to'kVideoPixelFormatUnknown', the RTC SDK gives the format of the video frame for processing by the processor, that is, the format of the acquisition.
      *                 You can get the actual captured video frame format and pixel type by frame_type{@link #IVideoFrame#frame_type} and pixel_format{@link #IVideoFrame#pixel_format}. <br>
      *                + When set to'kVideoPixelFormatI420 ', the RTC SDK converts the captured video frames into the corresponding format for pre-processing. This method call fails when <br>
      *                + Is set to another value.
      * @return  <br>
      *          + 0: Success. <br>
      *          +! 0: failure <br>
      * @notes   <br>
      *         + For Windows platforms, the preprocessed video frame format returned only supports'kVideoPixelFormatI420 '<br>
      *         + For Windows platforms, setting the required_pixel_format in'config' to'kVideoPixelFormatI420 'can bring some performance optimization by avoiding format conversion. <br>
      *         + When this interface is repeatedly called, only the last call takes effect. The effects do not stack.
      */
    virtual int RegisterLocalVideoProcessor(IVideoProcessor* processor, VideoPreprocessorConfig config) = 0;

    /** 
     * @hidden
     * @deprecated since 326.1, use SendSEIMessage instead
     * @type api
     * @region  video data callback
     * @brief  To register metadata observers to receive or send metadata. The underlying layer implements this function by adding SEI data to the video frame. <br>
     *         After registering an observer, there is no SEI information in the video frame sent, which will trigger OnReadyToSendMetadata {@link
     *  #IMetadataObserver #OnReadyToSendMetadata} Callback. <br> After registering an observer, the received video frame contains SEI information, which will trigger a receive
     * OnMetadataReceived{@link #IMetadataObserver#OnMetadataReceived} callback. <br>
     * @param  [in] observer Metadata observer, see: IMetadataObserver{@link #IMetadataObserver}
     * @notes   <br>
     *       + When using video custom capture and rendering, you can directly add and obtain metadata in the video frame. This interface is not recommended.
     *       + This interface supports dynamic unregistration, setting the parameter to nullptr to unregister. <br>
     */
    virtual void RegisterMetadataObserver(IMetadataObserver* observer) = 0;

    /** 
     * @type api
     * @region Video Management
     * @brief When video communication, SEI data is sent through video frames.
     * @param  [in] stream_index Media stream type. See StreamIndex{@link #StreamIndex}
     * @param  [in] message SEI message. Length not exceeding 4 kB. <br>
     * @param  [in] length SEI message length. <br>
     * @param  [in] repeat_count Number of times the message was sent repeatedly. The value range is [0,30]. <br>
     *                    When this interface is called, SEI data is added to consecutive'repeatCount 'video frames starting with the current video frame.
     * @return  <br>
     *         + > = 0: The number of SEIs to be added to the video frame <br>
     *         + < 0: Send failed
     * @notes  <br>
     *         + You can use this interface to add SEI to the video frame collected internally by the RTC SDK. For video frames obtained by custom acquisition: if the original video frame does not add SEI data, you can call this interface to add SEI information to it for encoding and transmission; if the original video frame has added SEI data, then Call this interface does not take effect. <br>
     *         + If there is no video frame with SEI within 2s after calling this interface (for example, video capture and transmission are not turned on), then SEI data will not be added to the video frame.
     *         + After the message is sent successfully, the remote will receive an OnSEIMessageReceived{@link #IRtcEngineLiteEventHandler#OnSEIMessageReceived} callback.
     */
    virtual int SendSEIMessage(StreamIndex stream_index, const uint8_t* message, int length, int repeat_count) = 0;

    /** 
     * @hidden
     * @deprecated
     * @type api
     * @region room management
     * @brief Set the engine to automatically publish properties
     * @param  [in] engine
     *       For the engine to be set, see: IRtcEngine{@link #IRtcEngine}
     * @param  [in] auto_publish
     *       Whether to automatically publish streams, the default is true to automatically publish, when setting false, you can use engine- > publish (), engine- > unpublish () to publish and Unpublish local stream
     * @notes
     * Set whether the engine automatically publishes the user's local stream, which must be called before joinChannel. The default engine auto_publish is true. The engine- > publish (),
     * Engine- > unpublish () interfaces should not be called when the auto_publish is true.
     */
    virtual void EnableAutoPublish(bool auto_publish) = 0;

    /** 
     * @hidden (Linux)
     * @type api
     * @region Video Facility Management
     * @brief Create a video Facility Management instance
     * @return Video Facility Management instance. See IVideoDeviceManager{@link #IVideoDeviceManager}
     * @notes When the IVideoDeviceManager instance is no longer used, call the instance's Release{@link #Release}
     * Interface to avoid Resource leakage
     */
    virtual IVideoDeviceManager* GetVideoDeviceManager() = 0;

    /** 
     * @hidden (Linux)
     * @type api
     * @region audio management
     * @brief Device audio management interface creation
     * @return Audio Facility Management interface
     */
    virtual IAudioDeviceManager* GetAudioDeviceManager() = 0;

    /** 
     * @hidden
     * @type api
     * @region audio data callback
     * @brief Register an audio data callback observer.
     * @param  [in] observer Audio data callback observer. See IRemoteAudioFrameObserver{@link #IRemoteAudioFrameObserver}
     * @notes   <br>
     *          Register the callback to receive PCM data from a single remote user.
     */
    virtual void RegisterRemoteAudioFrameObserver(IRemoteAudioFrameObserver* observer) = 0;

    /** 
     * @type api
     * @region Local recording
     * @brief This method records the audio & video data during the call to a local file.
     * @param  [in] type Stream attribute, specify whether to record mainstream or screen stream. See StreamIndex{@link #StreamIndex}
     * @param  [in] config Local recording parameter configuration. See RecordingConfig{@link #RecordingConfig}
     * @param [in] recording_type Local recording media type. See RecordingType{@link #RecordingType}.
     * @return   <br>
     *         + 0: normal
     *         + -1: Parameter setting exception
     *         + -2: The current version of the SDK does not support this Feature, please contact technical support staff
     * @notes   <br>
     *         + After calling this method, you will receive an OnRecordingStateUpdate{@link #IRtcEngineLiteEventHandler#OnRecordingStateUpdate} callback. <br>
     *         + If the recording is normal, the system will notify the recording progress through OnRecordingProgressUpdate{@link #IRtcEngineLiteEventHandler#OnRecordingProgressUpdate}
     *  Callback every second.
     */
    virtual int StartFileRecording(StreamIndex type, RecordingConfig config, RecordingType recording_type) = 0;

    /** 
     * @type api
     * @region local recording
     * @brief Stop local recording
     * @param  [in] type Stream attribute, specify to stop mainstream or screen stream recording. See StreamIndex{@link #StreamIndex}
     * @notes   <br>
     *         + Call StartFileRecording{@link #IRtcEngineLite#StartFileRecording} After starting local recording, you must call this method to stop recording. <br>
     *         + After calling this method, you will receive an OnRecordingStateUpdate{@link #IRtcEngineLiteEventHandler#OnRecordingStateUpdate} callback prompting you to record the result.
     */
    virtual void StopFileRecording(StreamIndex type) = 0;

    /** 
     * @type api
     * @region Engine management
     * @brief Set runtime parameters
     * @param  [in] json_string String after json serialization
     * @notes
     */
    virtual void SetRuntimeParameters(const char * json_string) = 0;

    /** 
     * @hidden
     * @type api
     * @region speech recognition service
     * @brief Enables automatic speech recognition service. This method converts the recognized user's voice into text and calls it back to the user through the OnMessage{@link #IRTCASREngineEventHandler#OnMessage}
     *  Event.
     * @param  [in] asr_config Check information. See RTCASRConfig{@link #RTCASRConfig}
     * @param  [in] handler Speech recognition service usage status callback. See IRTCASREngineEventHandler{@link #IRTCASREngineEventHandler}
     */
    virtual void StartASR(const RTCASRConfig& asr_config, IRTCASREngineEventHandler* handler) = 0;

    /** 
     * @hidden
     * @type api
     * @region  speech recognition service
     * @brief  Turn off speech recognition service
     */
    virtual void StopASR() = 0;

    /** 
     * @type api
     * @region  engine management
     * @brief  This call quality rating evaluation
     * @param  [in] data Reported data. JSON format, must contain the following 4 fields: <br>
     *         + Types: preset set of problem types, optional. For specific types, refer to ProblemFeedbackOption{@link #ProblemFeedbackOption} <br>
     *         + Problem_desc: specific descriptions of other issues other than the preset problem <br>
     *         + Os_version: system version <br>
     *         + Network_type: network type: including WiFi, 2g, 3g, 4g, 5g. If it is a desktop, fill in pc. <br>
     * @return  <br>
     *         + 0: Report successfully <br>
     *         + -1: Report failed, haven't joined the room yet <br>
     *         + -2: Report failed, incoming JSON resolution failed <br>
     *         + -3: Report failed, incoming JSON field missing <br>
     * @notes  If If the user is in the room when reporting, the problem will be located in one or more rooms where the user is currently located;
     *        If the user is not in the room when reporting, the problem will be located in the room where the engine previously exited.
     */
    virtual int Feedback(const char* data) = 0;

    /** 
     * @type api
     * @region  mixing
     * @brief  Mixing management interface creation
     * @return  Mixing management instance. See IAudioMixingManager{@link #IAudioMixingManager}
     */
    virtual IAudioMixingManager* GetAudioMixingManager() = 0;

    /** 
     * @type api
     * @region Real-time messaging
     * @brief Log in to send P2P messages or send messages to a server without joining the RTC room.<br>
     *        To log out, call Logout{@link #IRtcEngineLite#Logout}. <br>
     * @param  [in] token <br>
     *        Token is required during login for authentication. <br>
     *        This Token is different from that required by calling joinRoom. You can assign any value even null to `roomId` to generate a login token. During developing and testing, you can use temporary tokens generated on the console. Deploy the token generating application on your server.
     * @param  [in] uid <br>
     *        User ID <br>
     *        User ID is unique within one appid.
     * @return <br>
     *        + `0`: Success<br>
     *        + `-1`: Failure due to invalid parameter<br>
     *        + `-2`: Invalid call. RTC will return this value when you call this API after the user has been logged in to the server. <br>
     * @notes  You will receive OnLoginResult{@link #IRtcEngineLiteEventHandler#OnLoginResult} after calling this API and log in successfully. But remote users will not receive notification about that.
     */
    virtual int64_t Login(const char* token, const char* uid) = 0;
    /** 
     * @type After api
     * @region  real-time messaging
     * @brief  Calls this interface to log out, it is impossible to call methods related to out-of-room messages and end-to-server messages or receive related callbacks.
     * @notes   <br>
     *        + Before calling this interface to log out, you must call Login{@link #IRtcEngineLite#Login} to log in. <br>
     *        + After local users call this method to log out, they will receive the result of the OnLogout{@link #IRtcEngineLiteEventHandler#OnLogout} callback notification, and remote users will not receive the notification.
     */
    virtual void Logout() = 0;
    /** 
     * @type api
     * @region Real-time messaging
     * @brief Update the Token <br>
     *        Token used by the user to log in has a certain valid period. When the Token expires, you need to call this method to update the login Token information. <br>
     *         When calling the Login{@link #IRtcEngineLite#Login} method to log in, if an expired token is used, the login will fail and you will receive an OnLoginResult{@link #IRtcEngineLiteEventHandler#OnLoginResult} callback notification with an error code of kLoginErrorCodeInvalidToken. You need to reacquire the token and call this method to update the token.
     * @param  [in] token <br>
     *        Updated dynamic key
     * @notes   <br>
     *        + If the token is invalid and the login fails, call this method. After updating the token, the SDK will automatically log back in, and the user does not need to call the Login{@link #IRtcEngineLite#Login} method. <br>
     *        + Token expires, if you have successfully logged in, it will not be affected. An expired Token error will be notified to the user the next time you log in with an expired Token, or when you log in again due to a disconnection due to poor local network conditions.
     */
    virtual void UpdateLoginToken(const char* token) = 0;
    /** 
     * @type api
     * @region Real-time messaging
     * @brief Set application server parameters   <br>
     *        Client side calls SendServerMessage{@link #IRtcEngineLite#SendServerMessage} or SendServerBinaryMessage{@link #IRtcEngineLite#SendServerBinaryMessage} Before sending a message to the application server, a valid signature and application server address must be set.
     * @param  [in] signature <br>
     *        Dynamic signature <br>
     *        The service server uses this signature to authenticate the request.
     * @param  [in] url <br>
     *         The address of the application server
     * @notes   <br>
     *        + The user must call Login{@link #IRtcEngineLite#Login} to log in before calling this interface. <br>
     *        + After calling this interface, the SDK will use OnServerParamsSetResult{@link #IRtcEngineLiteEventHandler#OnServerParamsSetResult} to return the corresponding result.
     */
    virtual void SetServerParams(const char* signature, const char* url) = 0;
    /** 
     * @type api
     * @region Real-time messaging
     * @brief Query the login status of the opposite or local user
     * @param  [in] peer_user_id <br>
     *        User ID to be queried
     * @notes   <br>
     *        + Login{@link #IRtcEngineLite#Login} must be called before this interface can be called. <br>
     *        + After calling this interface, the SDK notifies the query result using the OnGetPeerOnlineStatus{@link #IRtcEngineLiteEventHandler#OnGetPeerOnlineStatus} callback. <br>
     *        + Before sending an out-of-room message, the user can know whether the peer user is logged in through this interface to decide whether to send a message. You can also check your login status through this interface.
     */
    virtual void GetPeerOnlineStatus(const char* peer_user_id) = 0;
    /** 
     * @type api
     * @region Real-time messaging
     * @brief Send a text message (P2P) to a specified user outside the room
     * @param  [in] uid <br>
     *        Message receiving user's ID
     * @param  [in] message <br>
     *        Text message content sent <br>
     *        Message does not exceed 62KB.
     * @return   <br>
     *         + > 0: Sent successfully, return the number of the sent message, increment from 1 <br>
     *         + -1: Sent failed, RtcEngine instance not created <br>
     *         + -2: Sent failed, uid is empty
     * @notes   <br>
     *        + You must call Login{@link #IRtcEngineLite#Login} to complete the login before you can send a message in a foreign text of the room. <br>
     *        + After the user calls this interface to send a text message, he will receive an OnUserMessageSendResultOutsideRoom{@link #IRtcEngineLiteEventHandler#OnUserMessageSendResultOutsideRoom} callback to know whether the message was successfully sent. <br>
     *        + If the text message is sent successfully, the user specified by uid receives the message via the OnUserMessageReceivedOutsideRoom{@link #IRtcEngineLiteEventHandler#OnUserMessageReceivedOutsideRoom} callback.
     */
    virtual int64_t SendUserMessageOutsideRoom(const char* uid, const char* message) = 0;
    /** 
     * @type api
     * @region Real-time messaging
     * @brief Send binary messages (P2P) to a specified user outside the room
     * @param  [in] uid <br>
     *        ID of the user receiving the message
     * @param  [in] length <br>
     *        Length of the binary string
     * @param  [in] message <br>
     *        Content of the binary message sent <br>
     *        Message does not exceed 46KB.
     * @return   <br>
     *         + > 0: sent successfully, return the number of the sent message, increment from 1 <br>
     *         + -1: Sent failed, RtcEngine instance not created <br>
     *         + -2: Sent failed, uid is empty
     * @notes   <br>
     *        + Login{@link #IRtcEngineLite#Login} must be called before sending out-of-room binary messages. <br>
     *        + After the user calls this interface to send a binary message, he will receive an OnUserMessageSendResultOutsideRoom{@link #IRtcEngineLiteEventHandler#OnUserMessageSendResultOutsideRoom} callback to notify whether the message was sent successfully. <br>
     *        + If the binary message is sent successfully, the user specified by uid will receive the message through the OnUserBinaryMessageReceivedOutsideRoom{@link #IRtcEngineLiteEventHandler#OnUserBinaryMessageReceivedOutsideRoom} callback.
     */
    virtual int64_t SendUserBinaryMessageOutsideRoom(const char* uid, int length, const uint8_t* message) = 0;
    /** 
     * @type api
     * @region Real-time messaging
     * @brief The client side sends a text message to the application server (P2Server)
     * @param  [in] message <br>
     *        The content of the text message sent <br>
     *        The message does not exceed 62KB.
     * @return   <br>
     *         + > 0: Sent successfully, return the number of the sent message, increment from 1 <br>
     *         + -1: Sent failed, RtcEngine instance not created
     * @notes   <br>
     *        + Before sending a text message to the application server, you must first call Login{@link #IRtcEngineLite#Login} to complete the login, and then call SetServerParams{@link #IRtcEngineLite#SetServerParams} Set up the application server. <br>
     *        + After calling this interface, you will receive an OnServerMessageSendResult{@link #IRtcEngineLiteEventHandler#OnServerMessageSendResult} callback to inform the message sender whether the message was sent successfully. <br>
     *        + If the text message is sent successfully, the application server that previously called SetServerParams{@link #IRtcEngineLite#SetServerParams} will receive the message.
     */
    virtual int64_t SendServerMessage(const char* message) = 0;
    /** 
     * @type api
     * @region Real-time messaging
     * @brief Client side sends binary messages to the application server (P2Server)
     * @param  [in] length <br>
     *        Length of binary string
     * @param  [in] message <br>
     *        Binary message content sent <br>
     *        Message does not exceed 46KB.
     * @return   <br>
     *         + > 0: Sent successfully, return the number of the sent message, increment from 1 <br>
     *         + -1: Sent failed, RtcEngine instance not created
     * @notes   <br>
     *        + Before sending a binary message to the application server, you must first call Login{@link #IRtcEngineLite#Login} to complete the login, and then call SetServerParams{@link #IRtcEngineLite#SetServerParams} Set up the application server. <br>
     *        + After calling this interface, you will receive an OnServerMessageSendResult{@link #IRtcEngineLiteEventHandler#OnServerMessageSendResult} callback to inform the message sender that the sending succeeded or failed. <br>
     *        + If the binary message is sent successfully, the application server that previously called SetServerParams{@link #IRtcEngineLite#SetServerParams} will receive the message.
     */
    virtual int64_t SendServerBinaryMessage(int length, const uint8_t* message) = 0;

    /** 
     * @type api
     * @region Pre-call network detection
     * @brief Pre-call network detection
     * @param  [in] is_test_uplink Whether to detect uplink bandwidth
     * @param  [in] expected_uplink_bitrate Expected uplink bandwidth, unit: kbps<br>Range: {0, [100-10000]}, `0`: Auto, that RTC will set the highest bite rate.
     * @param  [in] is_test_downlink Whether to detect downlink bandwidth
     * @param  [in] expected_downlink_biterate Expected downlink bandwidth, unit: kbps<br>Range: {0, [100-10000]}, `0`: Auto, that RTC will set the highest bite rate.
     * @return   <br>
     *         + > 0: Start probe successful <br>
     *         + -1: Start probe failed. Parameter error, both uplink and downlink probes are false, or the expected bandwidth exceeds the range [100,10000] <br>
     *         + -2: Failed to start probing. Push-pull flow has started <br>
     *         + -3: Probe has started <br>
     * @notes   <br>
     *        + After calling this interface, you will receive OnNetworkDetectionResult{@link #IRtcEngineLiteEventHandler#OnNetworkDetectionResult} callback notifying the probe result; <br>
     *        + If the probe stops, you will receive OnNetworkDetectionStopped{@link #IRtcEngineLiteEventHandler#OnNetworkDetectionStopped} notify that probing has stopped.
     */
    virtual NetworkDetectionStartReturn StartNetworkDetection(bool is_test_uplink, int expected_uplink_bitrate,
                                   bool is_test_downlink, int expected_downlink_biterate) = 0;

    /** 
     * @type api
     * @region Pre-call network probe
     * @brief Stop pre-call network probe
     * @notes   <br>
     *        + After calling this interface, you will receive OnNetworkDetectionStopped{@link #IRtcEngineLiteEventHandler#OnNetworkDetectionStopped} notifying that the the probing has stopped.
     */
    virtual void StopNetworkDetection() = 0;

    /** 
     * @hidden (Android, iOS)
     * @type api
     * @region Screen sharing
     * @brief Set the screen audio acquisition method (internal acquisition/custom acquisition)
     * @param  [in] source_type Screen audio input source type. See AudioSourceType{@link #AudioSourceType}
     * @notes   <br>
     *       + The default acquisition method is internal to the RTC SDK Collect. <br>
     *       + You should call this method before PublishScreen{@link #IRtcRoom#PublishScreen}. Otherwise, you will receive an error from OnWarning{@link #IRtcEngineLiteEventHandler#OnWarning}: 'kWarningCodeSetScreenAudioSourceTypeFailed' <br>
     *       + If it is set to internal collection, you must call StartScreenAudioCapture{@link #IRtcEngineLite#StartScreenAudioCapture} to start collection; <br>
     *       + If it is set to custom collection, you must call PushScreenAudioFrame{@link #IRtcEngineLite#PushScreenAudioFrame} to customize the collected screen audio frame Push to the RTC SDK. <br>
     *       + Whether it is an internal capture or a custom capture, you must call PublishScreen{@link #IRtcRoom#PublishScreen} to push the captured screen audio to the remote end.
     */
    virtual void SetScreenAudioSourceType(AudioSourceType source_type) = 0;

    /** 
     * @type api
     * @region Screen sharing
     * @brief Set the mixing mode of the screen audio stream and the audio stream collected by the microphone when the screen is shared
     * @param  [in] index The mixing mode. See StreamIndex{@link #StreamIndex} <br>
     *         + 'kStreamIndexMain': Mix the audio stream collected by the screen audio stream and the microphone <br>
     *         + 'KStreamIndexScreen ': Divide the screen audio stream and the audio stream collected by the microphone into two audio streams
     * @notes You should call this method before PublishScreen{@link #IRtcRoom#PublishScreen}. Otherwise, you will receive an error from OnWarning{@link #IRtcEngineLiteEventHandler#OnWarning} ': kWarningCodeSetScreenAudioStreamIndexFailed'
     */
    virtual void SetScreenAudioStreamIndex(StreamIndex index) = 0;

    /** 
     * @hidden (Android, iOS, Linux)
     * @type api
     * @region screen sharing
     * @brief When sharing the screen, start using RTC SDK internal collection method to collect screen audio
     * @notes  <br>
     *         + After collection, you also need to call PublishScreen{@link #IRtcRoom#PublishScreen} to collect the screen audio Push to the far end. <br>
     *         + To turn off screen audio internal capture, call StopScreenAudioCapture{@link #IRtcEngineLite#StopScreenAudioCapture}.
     */
    virtual void StartScreenAudioCapture() = 0;

   /** 
    * @hidden (Android, iOS, Linux)
    * @type api
    * @region Screen sharing
    * @brief Stop using RTC SDK internal acquisition method to capture screen audio during screen sharing.
    * @notes To start the screen audio internal capture, call StartScreenAudioCapture{@link #IRtcEngineLite#StartScreenAudioCapture}.
    */
    virtual void StopScreenAudioCapture() = 0;

    /** 
     * @type api
     * @region Screen Sharing
     * @brief Using a custom capture method, when capturing screen audio during screen sharing, push the audio frame to the RTC SDK for encoding and other processing.
     * @param  [in] frame Audio data frame. See IAudioFrame{@link #IAudioFrame}
     * @return  Method call result   <br>
     *         + 0: Setup succeeded <br>
     *         + < 0: Setup failed <br>
     * @notes   <br>
     *         + Before calling this interface to push custom collected audio data during screen sharing, you must call SetScreenAudioSourceType{@link #IRtcEngineLite#SetScreenAudioSourceType} Enable custom capture of screen audio. <br>
     *         + You should call this method every 10 milliseconds to push a custom captured audio frame. A push audio frame should contain frame.sample _rate/100 audio sample points. For example, if the sampling rate is 48000Hz, 480 sampling points should be pushed each time. <br>
     *         + The audio sampling format is S16. The data format in the audio buffer must be PCM data, and its capacity size should be samples × frame.channel × 2. <br>
     *         + This function runs within user-level threads. If other processes are running at the same time, this process will be interrupted. <br>
     *         + After calling this interface to push the custom captured audio frame to the RTC SDK, you must call PublishScreen{@link #IRtcRoom#PublishScreen} to push the captured screen audio to the remote end. Audio frame information pushed to the RTC SDK is lost before calling PublishScreen{@link #IRtcRoom#PublishScreen}.
     */
    virtual int PushScreenAudioFrame(IAudioFrame* frame) = 0;


    /** 
     * @hidden
     * @type api
     * @deprecated since 327.1, use StopScreenVideoCapture instead
     * @region Screen sharing
     * @brief Stop screen or window sharing.
     * @notes   <br>
     *        + This function must be called after StartScreenVideoCapture{@link #StartScreenVideoCapture} <br>
     *        + This function cannot be mixed with PublishScreen{@link #IRtcRoom#PublishScreen}, UnpublishScreen{@link #IRtcRoom#UnpublishScreen}. <br>
     */
    virtual void StopScreenCapture() = 0;

    /** 
     * @hidden (macOS,Windows,Android,Linux)
     * @type api
     * @region  screen sharing
     * @brief  set extension configuration
     * @param groupId  App and extension should belong to the same app group. Group ID needs to be passed in here
     * @notes this function must be called immediately after the sharedenginewithappid function <br>
     *                If it is not called or the call time is late, local users will receive a callback to rtcEngine: onMediaDeviceStateChanged: device_type: device_state: device_error: {@link #ByteRTCEngineDelegate #rtcEngine: onMediaDeviceStateChanged: device_type: device_state: device_error:}. <br>
     *                Parameter  device_state value is ByteRTCMediaDeviceStateStopped, device_error value is ByteRTCMediaDeviceErrorNotFindGroupId
     */
    virtual void SetExtensionConfig(const char* group_id) = 0;

    /** 
     * @hidden (macOS,Windows,Android,Linux)
     * @type api
     * @region  screen sharing
     * @brief send screen sharing extension program message
     * @param message:  Message content sent to extension program
     * @param size : size of message
     * @notes This function must be called after the startScreenCapture function
     */
    virtual void SendScreenCaptureExtensionMessage(const char* message, size_t size) = 0;

    /** 
     * @hidden (macOS,Windows,Android,Linux)
     * @type api
     * @region  screen sharing
     * @brief  start local screen shared data collection
     * @param type screen shared data collection type
     * @param bundleId the bundle ID of the broadcast upload extension passed in, which is used to display the extension preferentially in settings
     * @notes when the extension is started abnormally from the IOS control center, you don't need to call this function <br>
     *       local users will receive a callback to rtcEngine: onMediaDeviceStateChanged: device_type: device_state: device_error: {@link #ByteRTCEngineDelegate #rtcEngine: onMediaDeviceStateChanged: device_type: device_state: device_error:}. <br>
     *       Parameter  device_state value is ByteRTCMediaDeviceStateStarted, device_error value is ByteRTCMediaDeviceErrorOK
     */
    virtual void StartScreenCapture(ScreenMediaType type, const char* bundle_id) = 0;

    /** 
     * @hidden (macOS, Windows, iOS, Linux)
     * @type api
     * @region  Screen sharing
     * @brief  Start to capture screen audio/video through the incoming Type and Context.
     * @param [in] type
     *        specifying the media type of screen capture，see ScreenMediaType{@link #ScreenMediaType}
     * @param  [in] context
     *        Android platform incoming Intent object, obtained by the business party after applying for system recording permission.
     * @notes  <br>
     *        + After collection, you also need to call PublishScreen {@link #IRtcRoom #PublishScreen} to collect the screen audio/vieo Push to the far end. <br>
     *        + Enable internal screen audio/video capture, Android-specific interface.
     */
    virtual void StartScreenCapture(ScreenMediaType type, void* context) = 0;

    /** 
     * @hidden (macOS,Windows,Linux)
     * @type api
     * @region  screen sharing
     * @brief  update screen acquisition data type
     * @param type screen acquisition data type
     * @notes  this function must be called after the startScreenCapture function.
     *        local users will receive a callback to rtcEngine: onMediaDeviceStateChanged: device_type: device_state: device_error: {@link #ByteRTCEngineDelegate #rtcEngine: onMediaDeviceStateChanged: device_type: device_state: device_error:}. <br>
     *        Parameter  device_state value is ByteRTCMediaDeviceStateStarted or ByteRTCMediaDeviceStateStopped, device_error value is ByteRTCMediaDeviceErrorOK
     */
    virtual void UpdateScreenCapture(ScreenMediaType type) = 0;


     /** 
      * @type api
      * @region Audio Management
      * @brief Enable audio information prompts.   <br>
      * @param config See AudioPropertiesConfig{@link #AudioPropertiesConfig}
      * @notes  <br>
      *         After enable the prompt, you can:   <br>
      *        + Get the information of the audio stream collected by the local microphone and screen audio stream through OnLocalAudioPropertiesReport{@link #IRtcEngineLiteEventHandler#OnLocalAudioPropertiesReport}; <br>
      *        + Get the information of the subscribed remote audio streams through OnRemoteAudioPropertiesReport{@link #IRtcEngineLiteEventHandler#OnRemoteAudioPropertiesReport}.
      */
    virtual void EnableAudioPropertiesReport(const AudioPropertiesConfig& config) = 0;
    /** 
     * @type api
     * @region Audio Management
     * @brief Enables/disables the loudness equalization function.  <br>
     *        If you call this API with the parameter set to True, the loudness of user's voice will be adjusted to -16lufs. If then you also call SetAudioMixingLoudness{@link #IAudioMixingManager#SetAudioMixingLoudness} and import the original loudness of the audio data used in audio mixing, the loudness will be adjusted to -20lufs when the audio data starts to play.
     * @param [in] enable Whether to enable loudness equalization funcion:  <br>
     *        + True: Yes <br>
     *        + False: No
     * @notes You must call this API before starting to play the audio file with StartAudioMixing{@link #IAudioMixingManager#StartAudioMixing}.
     */
    virtual void EnableVocalInstrumentBalance(bool enable) = 0;
    /** 
     * @type api
     * @region Audio Management
     * @brief Enables/disables the playback ducking function. This function is usually used in scenarios such as "Watch Videos Together".  <br>
     *        With the function on, if remote voice is detected, the local media volume will be lowered to ensure the clarity of the remote voice. If remote voice disappears, the local media volume restores.
     * @param enable Whether to enable playback ducking:  <br>
     *        + True: Yes  <br>
     *        + False: No
     */
    virtual void EnablePlaybackDucking(bool enable) = 0;
    /** 
     * @type api
     * @region Video Data Callback
     * @brief Register a local video frame observer.   <br>
     *        This method applys to both internal capturing and custom capturing.  <br>
     *        After calling this API, SDK triggers OnLocalEncodedVideoFrame{@link #ILocalEncodedVideoFrameObserver#OnLocalEncodedVideoFrame} whenever a video frame is captured.
     * @param [in] observer Local video frame observer. See ILocalEncodedVideoFrameObserver{@link #ILocalEncodedVideoFrameObserver}. You can cancel the registration by setting it to `nullptr`.
     * @notes You can call this API before or after entering the RTC room. Calling this API before entering the room ensures that video frames are monitored and callbacks are triggered as early as possible.
     */
    virtual void RegisterLocalEncodedVideoFrameObserver(ILocalEncodedVideoFrameObserver* observer) = 0;

    /** 
     * @type api
     * @region  video management
     * @brief  Video data callback after registering remote encoding.   <br>
     *         After registration, when the SDK detects a remote encoded video frame, it will trigger the OnRemoteEncodedVideoFrame{@link #IRemoteEncodedVideoFrameObserver#OnRemoteEncodedVideoFrame} callback
     * @param  [in] observer Remote encoded video data monitor. See IRemoteEncodedVideoFrameObserver{@link #IRemoteEncodedVideoFrameObserver}
     * @notes   <br>
     *        + This method needs to be called before entering the room. <br>
     *        + The engine needs to be unregistered before it is destroyed. Call this method to set the parameter to nullptr.
     */
    virtual void RegisterRemoteEncodedVideoFrameObserver(IRemoteEncodedVideoFrameObserver* observer) = 0;

    /** 
     * @type api
     * @region Video management
     * @brief Register custom encoded frame push event callback
     * @param  [in] encoder_handler Custom encoded frame callback class. See IExternalVideoEncoderEventHandler{@link #IExternalVideoEncoderEventHandler}
     * @notes   <br>
     *        + This method needs to be called before entering the room. <br>
     *        + The engine needs to be unregistered before it is destroyed. Call this method to set the parameter to nullptr.
     */
    virtual void SetExternalVideoEncoderEventHandler(IExternalVideoEncoderEventHandler* encoder_handler) = 0;

    /** 
     * @type api
     * @region  video management
     * @brief  Push custom encoded video stream
     * @param  [in] index The attributes of the encoded stream that need to be pushed. See StreamIndex{@link #StreamIndex}
     * @param  [in] video_index The corresponding encoded stream subscript, starting from 0, if you call SetVideoEncoderConfig{@link #IRtcEngineLite#SetVideoEncoderConfig} to set multiple streams, the number here must be consistent with it
     * @param  [in] video_stream Encoded stream video frame information. See IEncodedVideoFrame{@link #IEncodedVideoFrame}. <br>
     * @notes   <br>
     *         + Currently, only video frames in H264 and ByteVC1 formats are supported, and the video stream protocol must be in an Annex B format. <br>
     *         + This function runs within the user calling thread <br>
     *         + Before pushing a custom encoded video frame, you must call SetVideoSourceType{@link #IRtcEngineLite#SetVideoSourceType} to switch the video input source to the custom encoded video source.
     */
    virtual bool PushExternalEncodedVideoFrame(StreamIndex index, int video_index, IEncodedVideoFrame* video_stream) = 0;

    /** 
     * @type api
     * @region Video Management
     * @brief Before subscribing to the remote video stream, set the remote video data decoding method
     * @param  [in] key The remote stream information, that is, which video stream is decoded. See RemoteStreamKey{@link #RemoteStreamKey}.
     * @param  [in] config Video decoding method. See VideoDecoderConfig{@link #VideoDecoderConfig}.
     * @notes   <br>
     *         + This method is suitable for manual subscription mode and is used before subscribing to remote flows. <br>
     *         + When you want to custom decode a remote stream, you need to call RegisterRemoteEncodedVideoFrameObserver{@link #IRtcEngineLite#RegisterRemoteEncodedVideoFrameObserver} to register the remote video stream monitor, and then call the interface to set the decoding method to custom decoding. The monitored video data will be called back through OnRemoteEncodedVideoFrame{@link #IRemoteEncodedVideoFrameObserver#OnRemoteEncodedVideoFrame}.
     */
    virtual void SetVideoDecoderConfig(RemoteStreamKey key, VideoDecoderConfig config) = 0;

    /** 
     * @type api
     * @region Video Management
     * @brief After subscribing to the remote video stream, request the key frame to the remote
     * @param  [in] stream_info The remote stream information. See RemoteStreamKey{@link #RemoteStreamKey}.
     * @notes   <br>
     *         + This method is only suitable for manual subscription mode and is used after successful subscription to the remote flow. <br>
     *         + This method is suitable for calling SetVideoDecoderConfig{@link #IRtcEngineLite#SetVideoDecoderConfig} to turn on the custom decoding function, and the custom decoding fails
     */
    virtual void RequestRemoteVideoKeyFrame(const RemoteStreamKey& stream_info) = 0;

    /** 
     * @type api
     * @region Audio management
     * @brief Send audio stream synchronization information. The message is sent to the remote end through the audio stream and synchronized with the audio stream. After the interface is successfully called, the remote user will receive a OnStreamSyncInfoReceived{@link #IRtcEngineLiteEventHandler#OnStreamSyncInfoReceived} callback.
     * @param  [in] data Message content.
     * @param  [in] length Message length. Message length must be [1,16] bytes.
     * @param  [in] config For configuration of media stream information synchronization. See StreamSycnInfoConfig{@link #StreamSycnInfoConfig}.
     * @return   <br>
     *         + > = 0: Message sent successfully. Returns the number of successful sends. <br>
     *         + -1: Message sending failed. Message length greater than 16 bytes. <br>
     *         + -2: Message sending failed. The content of the incoming message is empty. <br>
     *         + -3: Message sending failed. This screen stream was not published when the message was synchronized through the screen stream. <br>
     *         + -4: Message sending failed. This audio stream is not yet published when you synchronize messages with an audio stream captured by a microphone or custom device, as described in ErrorCode{@link #ErrorCode}. <br>
     * @notes When using `kRoomProfileTypeLiveBroadcasting` as room profile, the data will be delivered. For other coom profiles, the data may be lost when the local user is muted.
     */
    virtual int SendStreamSyncInfo(const uint8_t* data, int32_t length, const StreamSycnInfoConfig& config) = 0;

    /** 
     * @type api
     * @region Audio Mixing
     * @brief Change local voice to a different key, mostly used in Karaoke scenarios.  <br>
     *        You can adjust the pitch of local voice such as ascending or descending with this method.
     * @param [in] pitch The value that is higher or lower than the original local voice within a range from -12 to 12. The default value is 0, i.e. No adjustment is made.  <br>
     *        The difference in pitch between two adjacent values within the value range is a semitone, with positive values indicating an ascending tone and negative values indicating a descending tone, and the larger the absolute value set, the more the pitch is raised or lowered.  <br>
     *        Out of the value range, the setting fails and triggers OnWarning{@link #IRtcEngineLiteEventHandler#OnWarning} callback, indicating `WARNING_CODE_SET_SCREEN_STREAM_INVALID_VOICE_PITCH` for invalid value setting with WarningCode{@link #WarningCode}.
     */
    virtual void SetLocalVoicePitch(int pitch) = 0;

    /** 
     * @type api
     * @region Media stream management
     * @brief Control the playinging status of the local audio stream: play/not play   <br>
     * @param  [in] mute_state Playinging status to identify whether to play the local audio stream, see: MuteState {@link #MuteState}
     * @notes This method only controls the local audio The playinging state of the stream does not affect the local audio play device.
     */
    virtual void MuteAudioPlayback(MuteState mute_state) = 0;

    /** 
     * @type api
     * @brief Subscribe the public stream<br>
     *        A user can call this method to subscribe a public stream whether he/she has joined the room or not.
     * @param [in] public_stream_id ID of the public stream. If the stream has not been published then, the local client will receive the public stream once it starts publishing.
     * @return <br>
     *        + 0: Success. You will also be informed by OnPlayPublicStreamResult{@link #IRtcEngineLiteEventHandler#OnPlayPublicStreamResult}.  <br>
     *        + !0: Failure because of invalid parameter or empty parameters.<br>
     * @notes  <br>
     *        + We recommend to bind a view for the public stream before calling this API to subscribe a public stream. <br>
     *              - Internal renderer: By calling SetPublicStreamVideoCanvas{@link #IRtcEngineLite# SetPublicStreamVideoCanvas} or <br>
     *              - Custom renderer: By calling SetPublicStreamVideoSink{@link #IRtcEngineLite#SetPublicStreamVideoSink}<br>
     *        + After calling this API, you will be informed once the first frame has been decoded successfully by OnFirstPublicStreamVideoFrameDecoded{@link #IRtcEngineLiteEventHandler#OnFirstPublicStreamVideoFrameDecoded} and OnFirstPublicStreamAudioFrame{@link #IRtcEngineLiteEventHandler#OnFirstPublicStreamAudioFrame}.<br>
     *        + If the public stream contains SEI information, you will be informed by OnPublicStreamSEIMessageReceived{@link #IRtcEngineLiteEventHandler#OnPublicStreamSEIMessageReceived}.<br>
     *        + Call StopPlayPublicStream{@link #IRtcEngineLite#StopPlayPublicStream} to cancel subscribing the public stream.
     */
    virtual int StartPlayPublicStream(const char* public_stream_id) = 0;
    /** 
     * @type api
     * @brief Cancel subscribing the public stream.<br>
     *        Call this method to cancel subscribing to the public stream by calling StartPlayPublicStream{@link #IRtcEngineLite#StartPlayPublicStream}.
     * @param [in] public_stream_id ID of the public stream
     * @return
     *        + 0: Success<br>
     *        + !0: Failure<br>
     */

    virtual int StopPlayPublicStream(const char* public_stream_id) = 0;
    /** 
     * @type api
     * @brief Assign a internal render view to the public stream
     * @param [in] public_stream_id ID of the public stream
     * @param [in] canvas Internal render view. Refer to VideoCanvas{@link #VideoCanvas} for more details.
     * @return
     *        + 0: Success<br>
     *        + !0: Failure<br>
     */
    virtual int SetPublicStreamVideoCanvas(const char* public_stream_id, const VideoCanvas& canvas) = 0;
    /** 
     * @type api
     * @brief Assign a custom renderer to the public stream
     * @param [in] public_stream_id ID of the public stream
     * @param [in] video_sink Custom renderer. Set to be `null` when you want to release the renderer. Refer to IVideoSink{@link #IVideoSink} for more details.
     * @return
     *        + 0: Success<br>
     *        + !0: Failure<br>
     */
    virtual void SetPublicStreamVideoSink(const char* public_stream_id, IVideoSink* video_sink,
                                          IVideoSink::PixelFormat format) = 0;

    /** 
     * @hidden(Linux)
     * @type api
     * @region Audio & Video Processing
     * @brief Adds watermark to designated video stream.
     * @param [in] streamIndex Targeting stream index of the watermark. See StreamIndex{@link #StreamIndex}.
     * @param [in] image_path File path of the watermark image. You can use the absolute path or the asset path(/assets/xx.png). The path should be less than 512 bytes. <br>
     *        The watermark image should be in PNG or JPG format.
     * @param [in] config Watermark configurations. See RTCWatermarkConfig{@link #RTCWatermarkConfig}.
     * @notes <br>
     *        + Call ClearVideoWatermark{@link #IRtcEngineLite#ClearVideoWatermark} to remove the watermark on the designated video stream. <br>
     *        + You can only add one watermark to one video stream. The newly added watermark replaces the previous one. You can call this API multiple times to add watermarks to different streams. <br>
     *        + You can call this API before and after joining room. <br>
     *        + If you mirror the preview, or the preview and the published stream, the watermark will also be mirrored locally, but the published watermark will not be mirrored. <br>
     *        + When you enable simulcast mode, the watermark will be added to all video streams, and it will scale down to smaller encoding configurations accordingly. <br>
     */
    virtual void SetVideoWatermark(StreamIndex streamIndex, const char* image_path, RTCWatermarkConfig config) = 0;

    /** 
     * @hidden(Linux)
     * @type api
     * @region Audio & Video Processing
     * @brief  Removes video watermark from designated video stream.
     * @param  [in] streamIndex Targeting stream index of the watermark. See StreamIndex{@link #StreamIndex}.
     */
    virtual void ClearVideoWatermark(StreamIndex streamIndex) = 0;

    /** 
     * @type api
     * @region  cloud proxy
     * @brief  Start cloud proxy
     * @param  [in] configuration cloud proxy informarion list. See CloudProxyConfiguration{@link #CloudProxyConfiguration}.
     * @notes   <br>
     *         + Call this API before joining the room.<br>
     *         + Start pre-call network detection after starting cloud proxy. <br>
     *         + After starting cloud proxy and connects the cloud proxy server successfully, receives OnCloudProxyConnected{@link #IRtcEngineLiteEventHandler#OnCloudProxyConnected}. <br>
     *         + To stop cloud proxy, call StopCloudProxy{@link #IRtcEngineLite#StopCloudProxy}.
     */
    virtual void StartCloudProxy(const CloudProxyConfiguration& configuration) = 0;

    /** 
     * @type api
     * @region  cloud proxy
     * @brief  Stop cloud proxy
     * @notes To start cloud proxy, call StartCloudProxy{@link #IRtcEngineLite#StartCloudProxy}.
     */
    virtual void StopCloudProxy() = 0;
};

}  // namespace bytertc

#endif  // BYTE_RTC_LITE_INTERFACE_H__
