/*
 * Copyright (c) 2020 The BytePlusRTC project authors. All Rights Reserved.
 * @brief BytePlusRTC Room Event Handler Interface
*/

#pragma once

#ifndef BYTE_RTC_ROOM_EVENT_HANDLER_INTERFACE_H__
#define BYTE_RTC_ROOM_EVENT_HANDLER_INTERFACE_H__

#include "bytertc_defines.h"

namespace bytertc {
/** 
 * @type callback
 * @brief  Audio & video room event callback interface
 */
class IRTCRoomEventHandler {
public:

    /** 
     * @hidden
     * @brief  Destructor
     */
    virtual ~IRTCRoomEventHandler() {
    }

     /** 
     * @type callback
     * @region Multi-room
     * @brief Callback on room state changes. You will receive this callback when you join the room, leave the room, or receive room relating warnings and errors.
     * @param [in] room_id  Room ID.
     * @param [in] uid  User ID.
     * @param [in] state Room state code. <br>
     *              + 0: Success. <br>
     *              + !0: Failure. See ErrorCode{@link #ErrorCode} and WarningCode{@link #WarningCode} for specific indications.
     * @param [in] extra_info Extra information.
     *                 `join_type` indicates the type of room the user joins. `0` means the user joins the room for the first time, and `1` means the user rehoins the room. <br>
     *                 `elapsed` indicates the time interval from calling JoinRoom{@link #IRtcRoom#JoinRoom} to successfully joining room, in ms.
     */
    virtual void OnRoomStateChanged(
            const char* room_id, const char* uid, int state, const char* extra_info) {
        (void)room_id;
        (void)state;
        (void)extra_info;
    }
    
    /** 
     * @type callback
     * @region Room management
     * @brief Callback on stream state changes. You will receive this callback when you receive stream relating warnings and errors.
     * @param [in] room_id  Room ID.
     * @param [in] uid  User ID.
     * @param [in] state Room state code. See ErrorCode{@link #ErrorCode} and WarningCode{@link #WarningCode} for specific indications.
     * @param [in] extra_info Extra information. Currently unavailable.
     */
    virtual void OnStreamStateChanged(
            const char* room_id, const char* uid, int state, const char* extra_info) {
        (void)room_id;
        (void)state;
        (void)extra_info;
    }

    /** 
      * @type callback
      * @region Room management
      * @brief After leaving the room successfully, receives the callback.   <br>
      *        When the user calls the LeaveRoom{@link #IRtcRoom#LeaveRoom}, the SDK will stop all publishing subscription streams and release all call-related media resources. After that, the user receives this callback . <br>
      * @param [in] stats Statistics for this call. See RtcRoomStats{@link #RtcRoomStats}. <br>
      * @notes   <br>
      *        + If calling LeaveRoom{@link #IRtcRoom#LeaveRoom} and then DestroyRtcEngine{@link #DestroyRtcEngine} immediately after, the user will not receive this callback. <br>
      *        + If the app needs to use the media resources of the device for purposes other than RTC, you should init the resources after receiving this callback. Receiving this callback ensures that the resources are not occupied by RTC SDK.
      */
    virtual void OnLeaveRoom(const RtcRoomStats& stats) {
        (void)stats;
    }

    /** 
     * @hidden
     * @deprecated since 341.1 by OnRoomStateChanged and OnStreamStateChanged
     * @type callback
     * @region Room Management
     * @brief SDK Warning callback occurred. When a recoverable error is encountered inside the <br>
     *        SDK, the user is notified via this callback while attempting automatic recovery. This callback event is only used as a notification. <br>
     * @param  [in] warn Code. See WarningCode{@link #WarningCode}. <br>
     */
    virtual void OnRoomWarning(int warn) {
        (void)warn;
    }

    /** 
     * @hidden
     * @deprecated since 341.1 by OnRoomStateChanged and OnStreamStateChanged
     * @type callback
     * @region Room management
     * @brief SDK Error callback occurred. <br>
     *        SDK internal encountered an unrecoverable error, will notify the App through this callback, requires the App to operate according to the error code or prompt the user. <br>
     * @param  [in] err Error code. See ErrorCode{@link #ErrorCode}. <br>
     */
    virtual void OnRoomError(int err) {
        (void)err;
    }
    /** 
     * @type callback
     * @brief You will be informed 30 sec before the expiration via this callback.
     *        Call UpdateToken{@link #IRtcRoom#UpdateToken} to renew the Token. If the Token expired, the user would be removed from the room and not be able to continue the call.
     */
    virtual void OnTokenWillExpire() {

    }

    /** 
     * @hidden
     * @deprecated Since 329.1, use OnLocalAudioPropertiesReport/onRemoteAudioPropertiesReport instead
     * @type callback
     * @region audio event callback
     * @brief Prompt all users in the room to send a callback for volume.
     * @param  [in] speakers The ID and volume of the local user and the remote user subscribed to.
     *                       See: AudioVolumeInfo{@link #AudioVolumeInfo}.
     * @param  [in] speaker_number Length of speakers array.
     * @param  [in] total_remote_volume The sum of all audio volumes contained in speakers, the value range is: [0,255]
     * @notes
     *         + For local users: As long as local audio collection is performed, the callback will contain the volume information of the local audio stream. <br>
     *         + For remote users: The local must subscribe to the audio stream of a remote user before the callback contains the volume information of the audio stream it sent.
     */
    virtual void OnAudioVolumeIndication(const AudioVolumeInfo* speakers, unsigned int speaker_number, int total_remote_volume) {
        (void)speakers;
        (void)speaker_number;
        (void)total_remote_volume;
    }


    /** 
     * @type callback
     * @region  data statistics
     * @brief  In-room call statistics callback.    <br>
     *        After the user enters the room and starts the call, he receives this callback every 2s.
     * @param  [in] stats Current RtcEngine statistics. See RtcRoomStats{@link #RtcRoomStats}
     */
    virtual void OnRoomStats(const RtcRoomStats& stats) {
        (void)stats;
    }
    /** 
     * @type callback
     * @region Statistics
     * @brief After the published stream is successful, you receive this callback every 2s to understand the network quality information of the published stream during this cycle.
     * @param  [in] stats Current RtcEngine statistics. See LocalStreamStats{@link #LocalStreamStats}
     */
    virtual void OnLocalStreamStats(const LocalStreamStats& stats) {
        (void)stats;
    }
    /** 
     * @type callback
     * @region Statistics
     * @brief Receive this callback every 2s to learn the network quality information of the stream published by the subscribed remote user during this cycle.
     * @param  [in] stats Current RtcEngine statistics. See RemoteStreamStats{@link #RemoteStreamStats}
     */
    virtual void OnRemoteStreamStats(const RemoteStreamStats& stats) {
        (void)stats;
    }

    /** 
      * @type callback
      * @region Room management
      * @brief You will receive this callback in following cases: <br>
      *        + The remote user calls SetUserVisibility{@link #IRtcRoom#SetUserVisibility} turns visible and joins your room. <br>
      *        + The remote visible user is disconnected and then reconnected to your room. <br>
      *        + The invisible remote user in your room calls SetUserVisibility{@link #IRtcRoom#SetUserVisibility} and turns visible. <br>
      *        + You join the room when there are visible users in the room.
      * @param [in] user_info See UserInfo{@link #UserInfo}. <br>
      * @param [in] elapsed Reserved parameter.
      */
    virtual void OnUserJoined(const UserInfo& user_info, int elapsed) {
        (void)user_info;
        (void)elapsed;
    }

    /** 
     * @type callback
     * @brief This callback is triggered when a remote user is disconnected or turns invisible.
     * @param uid ID of the user who leaves the room, or switches to invisible. <br>
     * @param reason Reason to leave the room: <br>
     *               + 0: The remote client calls LeaveRoom{@link #IRtcRoom#LeaveRoom} to leave the room. <br>
     *               + 1: The remote client is disconnected because of poor network connection or expired Token. <br>
     *               + 2: The remote client calls SetUserVisibility{@link #IRtcRoom#SetUserVisibility} to turn invisible. <br>
     *               + 3: The remote user has been removed from the room by the administrator via a OpenAPI call.
     */
    virtual void OnUserLeave(const char* uid, UserOfflineReason reason) {
        (void)uid;
        (void)reason;
    }

    /** 
     * @hidden
     * @type callback
     * @region Media Stream Management
     * @brief When a user in the room calls MuteAllRemoteAudio{@link #IRtcRoom#MuteAllRemoteAudio},
     *        Changing the state of receiving all remote audio streams, other users in the room receive this callback.
     * @param  [in] user_id Change the user ID of the receiving state
     * @param  [in] mute_state The receiving state, see: MuteState{@link #MuteState}
     */
    virtual void OnMuteAllRemoteAudio(const char* user_id, MuteState mute_state) {
        (void)user_id;
        (void)mute_state;
    }


    /** 
     * @hidden
     * @type callback
     * @region Media Stream Management
     * @brief When the user in the room pauses/resumes receiving all video streams, other users in the room receive this callback. See MuteAllRemoteVideo{@link #MuteAllRemoteVideo}.
     * @param  [in] uid Pause/resume the user ID of the received video stream.
     * @param   [in] mute Pauses/resumes the state of the received video stream. See MuteState{@link #MuteState}.
     */
    virtual void OnMuteAllRemoteVideo(const char* uid, MuteState mute) {
        (void)uid;
        (void)mute;
    }


    /** 
     * @hidden
     * @deprecated since 336.1, use OnUserPublishStream, OnUserPublishScreen, OnUserUnPublishStream and OnUserUnPublishScreen instead.
     * @type callback
     * @region Media Stream Management
     * @brief Transmission state change of an audio stream in the room triggers this callback. Refer to MuteLocalAudio{@link #IRtcEngineLite#MuteLocalAudio} for more details.
     * @param  [in] user_id The user who changes his/her transmission state of the audio streams
     * @param  [in] mute_state Transmission state of the audio streams. Refer to MuteState{@link #MuteState} for more details.
     */
    virtual void OnUserMuteAudio(const char* user_id, MuteState mute_state) {
        (void)user_id;
        (void)mute_state;
    }


    /** 
     * @hidden
     * @deprecated since 323.1, use OnUserStartAudioCapture instead
     * @type callback
     * @region Audio event callback
     * @brief When the remote user turns on/off audio device capture, others in the room will receive this callback.
     *        The callback is triggered by the remote user calling the StartAudioCapture{@link #IRtcEngineLite#StartAudioCapture} method.
     * @param  [in] uid Remote user ID
     * @param  [in] enabled Whether the user has audio device capture <br>
     *        + True: the user has audio device capture <br>
     *        + False: the user has audio device capture closed <br>
     *        + This interface is deprecated, please use OnUserStartAudioCapture{@link #IRTCRoomEventHandler#OnUserStartAudioCapture} And OnUserStopAudioCapture{@link #IRTCRoomEventHandler#OnUserStopAudioCapture} <br>
     */
    virtual void OnUserEnableLocalAudio(const char* uid, bool enabled) {
        (void)uid;
        (void)enabled;
    }

    /** 
     * @type callback
     * @region Audio event callback
     * @brief The remote clients in the room will be informed of the state change via this callback after the visible user starts audio capture by calling StartAudioCapture{@link #IRtcEngineLite#StartAudioCapture}.
     * @param  [in] user_id The user who starts the internal audio capture
     */
    virtual void OnUserStartAudioCapture(const char* user_id) {
        (void)user_id;
    }

    /** 
     * @type callback
     * @region Audio event callback
     * @brief The remote clients in the room will be informed of the state change via this callback after the visible user stops audio capture by calling StopAudioCapture{@link #IRtcEngineLite#StopAudioCapture}.
     * @param  [in] user_id The user who stops the internal audio capture
     */
    virtual void OnUserStopAudioCapture(const char* user_id) {
        (void)user_id;
    }

    /** 
     * @type callback
     * @region Audio event callback
     * @brief Receive the callback when the first audio frame is locally collected
     * @param  [in] index Audio stream properties. See StreamIndex{@link #StreamIndex}
     */
    virtual void OnFirstLocalAudioFrame(StreamIndex index) {
        (void)index;
    }


    /** 
     * @type callback
     * @region Audio event callback
     * @brief Receives the callback when the first frame from a remote audio stream is received.
     * @param  [in] key Remote audio stream information. See RemoteStreamKey{@link #RemoteStreamKey}
     * @notes The callback is received when the user has just received each audio stream in the room.
     */
    virtual void OnFirstRemoteAudioFrame(const RemoteStreamKey& key) {
        (void)key;
    }


    /** 
     * @hidden
     * @deprecated since 336.1, use OnUserUnPublishStream and OnUserUnPublishScreen instead.
     * @type callback
     * @region Room Management
     * @brief Local users will receive this callback when a remote user in the room stops publishing audio & video streams. Properties of the
     * @param  [in] stream Stream. See MediaStreamInfo{@link #MediaStreamInfo}.
     * @param  [in] reason The reason for the removal of the remote stream. See StreamRemoveReason{@link #StreamRemoveReason}.
     */
    virtual void OnStreamRemove(const MediaStreamInfo& stream, StreamRemoveReason reason) {
        (void)stream;
    }


    /** 
     * @hidden
     * @deprecated since 336.1, use OnUserPublishStream and OnUserPublishScreen instead.
     * @type callback
     * @region Room Management
     * @brief When users in the room post a new audio & video stream, other users in the room will receive this callback. Includes streams that are removed and republished.   <br>
     * @param  [in] stream Stream properties. See MediaStreamInfo{@link #MediaStreamInfo}. <br>
     */
    virtual void OnStreamAdd(const MediaStreamInfo& stream) {
        (void)stream;
    }
    /** 
     * @type callback
     * @region Room management
     * @brief Callback on new media streams captured by camera/microphone in the room.
     * @param [in] uid The ID of the remote user who published the stream.
     * @param [in] type Media stream type. See MediaStreamType{@link #MediaStreamType}.
     * @notes You will receive this callback after a remote user successfully published media streams captured by camera/microphone in the room with PublishStream{@link #IRtcRoom#PublishStream}. Then you can choose whether to call SubscribeStream{@link #IRtcRoom#SubscribeStream} to subscribe to the streams or not.
     */
    virtual void OnUserPublishStream(const char* uid, MediaStreamType type) {
        (void)uid;
        (void)type;
    }
    /** 
     * @type callback
     * @region Room management
     * @brief Callback on removal of remote media stream captured by camera/microphone.
     * @param [in] uid The ID of the remote user who removed the stream.
     * @param [in] type Media stream type. See MediaStreamType{@link #MediaStreamType}.
     * @param [in] reason The reason for the removal, see StreamRemoveReason{@link #StreamRemoveReason}.
     * @notes After receiving this callback, you can choose whether to call UnsubscribeStream{@link #IRtcRoom#UnsubscribeStream} to unsubscribe from the streams or not.
     */
    virtual void OnUserUnPublishStream(const char* uid, MediaStreamType type,StreamRemoveReason reason) {
        (void)uid;
        (void)type;
        (void)reason;
    }
    /** 
     * @type callback
     * @region Room management
     * @brief Callback on new screen sharing media streams from remote users in the room.
     * @param [in] uid The ID of the remote user who published the stream.
     * @param [in] type Media stream type. See MediaStreamType{@link #MediaStreamType}.
     * @notes You will receive this callback after a remote user successfully published screen sharing streams in the room with PublishScreen{@link #IRtcRoom#PublishScreen}. Then you can choose whether to call SubscribeScreen{@link #IRtcRoom#SubscribeScreen} to subscribe to the streams or not.
     */
    virtual void OnUserPublishScreen(const char* uid, MediaStreamType type) {
        (void)uid;
        (void)type;
    }
    /** 
     * @type callback
     * @region Room management
     * @brief Callback on removal of screen sharing media streams from remote users in the room.
     * @param [in] uid The ID of the remote user who removed the stream.
     * @param [in] type Media stream type. See MediaStreamType{@link #MediaStreamType}.
     * @param [in] reason The reason for the removal, see StreamRemoveReason{@link #StreamRemoveReason}.
     * @notes After receiving this callback, you can choose whether to call UnsubscribeScreen{@link #IRtcRoom#UnsubscribeScreen} to unsubscribe from the streams or not.
     */
    virtual void OnUserUnPublishScreen(const char* uid, MediaStreamType type,StreamRemoveReason reason) {
        (void)uid;
        (void)type;
        (void)reason;
    }

    /** 
     * @type callback
     * @region Room management
     * @brief Callback on subscription status of media streams
     * @param [in] state_code Subscription status of media streams, see SubscribeState{@link #SubscribeState}.
     * @param [in] user_id The ID of the user who published the stream.
     * @param [in] info Configurations of stream subscription, see SubscribeConfig{@link #SubscribeConfig}.
     * @notes  <br>
     *        Local users will receive this callback:  <br>
     *        + After calling SubscribeStream{@link #IRtcRoom#SubscribeStream}/UnsubscribeStream{@link #IRtcRoom#UnsubscribeStream} to change the subscription status of remote media streams captured by camera/microphone.  <br>
     *        + After calling SubscribeScreen{@link #IRtcRoom#SubscribeScreen}/UnsubscribeScreen{@link #IRtcRoom#UnsubscribeScreen} to change the subscription status of remote screen sharing streams.
     */
    virtual void OnStreamSubscribed(SubscribeState state_code, const char* user_id, const SubscribeConfig& info) {
        (void)state_code;
        (void)user_id;
        (void)info;
    }

    /** 
     * @type callback
     * @region  audio & video rollback
     * @brief Once the Fallback option is set, Fallback or reverting from a Fallback of a media stream will trigger this callback.
     * @param  [in] event Information of stream switching. Refer to RemoteStreamSwitch{@link #RemoteStreamSwitch} for more details.
     * @notes Call SetSubscribeFallbackOption{@link #IRtcEngineLite#SetSubscribeFallbackOption} or configure Fallback options on the server side to register this callback.
     */
    virtual void OnSimulcastSubscribeFallback(const RemoteStreamSwitch& event) {
        (void)event;
    }

    /** 
     * @hidden
     * @type callback
     * @region Room management
     * @brief Callback the event when the release flow is successful.
     * @param  [in] user_id The user ID of the published stream.
     * @param  [in] is_screen Whether the stream is a screen stream. <br>
     *              + Ture: Screen stream; <br>
     *              + False: Non-screen stream. <br>
     */
    virtual void OnStreamPublishSuccess(const char* user_id, bool is_screen) {
        (void)is_screen;
    }

    /** 
     * @type callback
     * @region Real-time messaging
     * @brief Receives a callback for broadcast messages in the room.
     * @param  [in] uid <br>
     *         Message sender ID
     * @param  [in] message <br>
     *         Received message content
     * @notes Other users in the same room will receive this callback when they call SendRoomMessage{@link #IRtcRoom#SendRoomMessage} to send a broadcast message.
     */
    virtual void OnRoomMessageReceived(const char* uid, const char* message) {
        (void)uid;
        (void)message;
    }

    /** 
     * @type callback
     * @region Real-time messaging
     * @brief Receives a callback to a binary broadcast message in the room.
     * @param  [in] uid <br>
     *         Message sender ID
     * @param  [in] size <br>
     *         Binary message length received
     * @param  [in] message <br>
     *        Binary message content received
     * @notes Other users in the same room call SendRoomBinaryMessage{@link #IRtcRoom#SendRoomBinaryMessage} Receive this callback when sending a binary broadcast message.
     */
    virtual void OnRoomBinaryMessageReceived(const char* uid, int size, const uint8_t* message) {
        (void)uid;
        (void)size;
        (void)message;
    }
    /** 
     * @type callback
     * @region Real-time messaging
     * @brief Receive this callback when you receive a text message (P2P) from another user in the room.
     * @param  [in] uid <br>
     *         Message sender ID.
     * @param  [in] message <br>
     *         The content of the received text message.
     * @notes   <br>
     *         + You must call the SendUserMessage{@link #IRtcRoom#SendUserMessage} method before you can receive the callback.
     */
    virtual void OnUserMessageReceived(const char* uid, const char* message) {
        (void)uid;
        (void)message;
    }

    /** 
     * @type callback
     * @region Real-time Message Communication
     * @brief A single user receives a callback (P2P) of binary messages from other uid-owned users in the same room.
     * @param  [in] uid <br>
     *         Message sender ID.
     * @param  [in] message <br>
     *         The content of the received binary message.
     * @notes   <br>
     *         + When other users in the same room call SendUserBinaryMessage{@link #IRtcRoom#SendUserBinaryMessage} to send binary messages to local users, local users will receive the callback.
     */
    virtual void OnUserBinaryMessageReceived(const char* uid, int size, const uint8_t* message) {
        (void)uid;
        (void)size;
        (void)message;
    }

    /** 
     * @type callback
     * @region Real-time messaging
     * @brief After sending a text or binary message (P2P) to a single user in the room, the message sender receives a callback with the result of the message.
     * @param  [in] msgid <br>
     *             The ID of this message.
     * @param  [in] error <br>
     *             Text or binary message sending results. See UserMessageSendResult{@link #UserMessageSendResult}
     * @notes   <br>
     *         + You must first call the SendUserMessage{@link #IRtcRoom#SendUserMessage} or SendUserBinaryMessage{@link #IRtcRoom#SendUserBinaryMessage} interface to receive this callback.
     */
    virtual void OnUserMessageSendResult(int64_t msgid, int error) {
        (void)msgid;
        (void)error;
    }

    /** 
     * @type callback
     * @region Real-time messaging
     * @brief When the SendRoomMessage{@link #IRtcRoom#SendRoomMessage} function is called to send a message, the sending result of this message (feedback) is called back.
     * @param  [in] msgid <br>
     *         ID of this message
     * @param  [in] error <br>
     *         Message sending result <br>
     *         See RoomMessageSendResult{@link #RoomMessageSendResult}
     */
    virtual void OnRoomMessageSendResult(int64_t msgid, int error) {
        (void)msgid;
        (void)error;
    }

    /*
     * @hidden
     * @deprecated
     */
    virtual void OnRoomModeChanged(RtcRoomMode mode) {
        (void)mode;
    }

#ifndef ByteRTC_AUDIO_ONLY
    /** 
     * @type callback
     * @region Video Management
     * @brief RTC SDK receives this callback when the first video frame or screen video frame capture is completed locally.
     * @param [in] index Stream property. See StreamIndex{@link #StreamIndex}.
     * @param [in] info Video information. See VideoFrameInfo{@link #VideoFrameInfo}.
     * @notes For captured local video frames, you can call SetLocalVideoCanvas{@link #IRtcEngineLite#SetLocalVideoCanvas} or SetLocalVideoSink{@link #IRtcEngineLite#SetLocalVideoSink} to render locally.
     */
    virtual void OnFirstLocalVideoFrameCaptured(StreamIndex index, VideoFrameInfo info) {
        (void)index;
        (void)info;
    }


    /** 
     * @type callback
     * @region video management
     * @brief Receive this callback when the local video size or rotation configuration changes.
     * @param [in] index See StreamIndex{@link #StreamIndex}。
     * @param [in] info Video frame information. See VideoFrameInfo{@link #VideoFrameInfo}.
     */
    virtual void OnLocalVideoSizeChanged(StreamIndex index, const VideoFrameInfo& info) {
        (void)index;
        (void)info;
    }


    /** 
     * @type callback
     * @region Video Management
     * @brief Users in the room who subscribe to this video stream receive this callback when the remote video size or rotation configuration changes.
     * @param  [in] key Remote stream information. See RemoteStreamKey{@link #RemoteStreamKey}.
     * @param  [in] info Video frame information. See VideoFrameInfo{@link #VideoFrameInfo}.
     */
    virtual void OnRemoteVideoSizeChanged(RemoteStreamKey key, const VideoFrameInfo& info) {
        (void)key;
        (void)info;
    }


    /** 
     * @type callback
     * @region Video Management
     * @brief  Receive this callback after the first frame of remote video stream is locally rendered by SDK.
     * @param  [in] key Remote stream information. See RemoteStreamKey{@link #RemoteStreamKey}.
     * @param  [in] info Video frame information. See VideoFrameInfo{@link #VideoFrameInfo}.
     */
    virtual void OnFirstRemoteVideoFrameRendered(const RemoteStreamKey key, const VideoFrameInfo& info) {
        (void)key;
        (void)info;
    }

    /** 
     * @type callback
     * @region Video Management
     * @brief Receive this callback after the first frame of remote video stream is received and decoded by SDK.
     * @param  [in] key Remote stream information. See RemoteStreamKey {@link #RemoteStreamKey}.
     * @param  [in] info Video frame information. See VideoFrameInfo {@link #VideoFrameInfo}.
     */
    virtual void OnFirstRemoteVideoFrameDecoded(const RemoteStreamKey key, const VideoFrameInfo& info) {
        (void)key;
        (void)info;
    }

    /** 
     * @hidden
     * @deprecated since 336.1, use OnUserPublishStream, OnUserPublishScreen, OnUserUnPublishStream and OnUserUnPublishScreen instead.
     * @type callback
     * @region Media Stream Management
     * @brief Transmission state change of an video stream in the room triggers this callback. Refer to MuteLocalVideo{@link #IRtcEngineLite#MuteLocalVideo}.
     * @param  [in] uid The user who changes his/her transmission state of the video streams
     * @param  [in] mute Transmission state of the video streams. Refer to MuteState{@link #MuteState} for more details.
     */
    virtual void OnUserMuteVideo(const char* uid, MuteState mute) {
        (void)uid;
        (void)mute;
    }

    /** 
     * @hidden
     * @deprecated since 325.1, use OnUserStartVideoCapture instead
     * @type callback
     * @region Video management
     * @brief This event is called back when the remote user turns on/off local video capture. When the remote user calls EnableLocalVideo, other users will trigger this callback.
     * @param [in] uid User ID, which prompts which user's video stream. Whether the local video feature is enabled. <br>
     * @param [in] enabled Whether to enable local video function.  <br>
     *       + True: This user has enabled the local video function. When enabled, other users can receive the user's video stream. <br>
     *       + false: This user has turned off the video function. After closing, the user can still receive video streams from other users, but other users cannot receive video streams from the user. <br>
     * @notes There are other users who have enabled/disabled the local video feature
     *       + This API is deprecate, please use OnUserStartVideoCapture{@link #IRTCRoomEventHandler#OnUserStartVideoCapture} and OnUserStopVideoCapture{@link #IRTCRoomEventHandler#OnUserStopVideoCapture} instead. <br>
     */
    virtual void OnUserEnableLocalVideo(const char* uid, bool enabled) {
        (void)uid;
        (void)enabled;
    }

    /** 
     * @type callback
     * @region Video event callback
     * @brief UThe remote clients in the room will be informed of the state change via this callback after the visible user starts video capture by calling StartVideoCapture{@link #IRtcEngineLite#StartVideoCapture}.
     * @param  [in] user_id The user who starts the internal video capture
     */
    virtual void OnUserStartVideoCapture(const char* user_id) {
        (void)user_id;
    }

    /** 
     * @type callback
     * @region Video event callback
     * @brief The remote clients in the room will be informed of the state change via  this callback after the visible user stops video capture by calling StopVideoCapture{@link #IRtcEngineLite#StopVideoCapture}.
     * @param  [in] user_id The user who stops the internal video capture
     */
    virtual void OnUserStopVideoCapture(const char* user_id) {
        (void)user_id;
    }

    /** 
     * @type callback
     * @region Audio event callback
     * @brief When the state of the local audio changes, the callback notifies the current local audio state.
     * @param  [in] state The state of the local audio device, see: LocalAudioStreamState{@link #LocalAudioStreamState}
     * @param  [in] error Error code when the local audio state changes, see: LocalAudioStreamError{@link #LocalAudioStreamError}
     */
    virtual void OnLocalAudioStateChanged(LocalAudioStreamState state, LocalAudioStreamError error) {
        (void)state;
        (void)error;
    }

    /** 
     * @type callback
     * @region Audio event callback
     * @brief When the state of the audio stream from the remote user subscribes to changes, this callback will be received to understand the current state of the remote audio stream.
     * @param  [in] key Remote stream information. See RemoteStreamKey{@link #RemoteStreamKey}
     * @param  [in] state Remote audio stream state. See RemoteAudioState{@link #RemoteAudioState}
     * @param  [in] reason For remote audio stream state change. See RemoteAudioStateChangeReason{@link #RemoteAudioStateChangeReason}
     */
    virtual void OnRemoteAudioStateChanged(
            const RemoteStreamKey& key, RemoteAudioState state, RemoteAudioStateChangeReason reason) {
        (void)key;
        (void)state;
        (void)reason;
    }

    /** 
     * @type callback
     * @region video management [in]
     * @brief Receive this event when the state of the local video stream changes.
     * @param [in] index Index audio/video properties. See StreamIndex{@link #StreamIndex}
     * @param [in] state Local video stream status. See LocalVideoStreamState{@link #LocalVideoStreamState}
     * @param [in] error Error code when local video status changes. See LocalVideoStreamError{@link #LocalVideoStreamError}
     */
    virtual void OnLocalVideoStateChanged(StreamIndex index, LocalVideoStreamState state, LocalVideoStreamError error) {
        (void)index;
        (void)state;
        (void)error;
    }


    /** 
     * @type callback
     * @region Video Management
     * @brief When the state of a remote video stream changes, users in the room who subscribe to this stream receive the event.
     * @param  [in] key Information about the remote video stream, room, user ID, stream attributes, etc. See RemoteStreamKey{@link #RemoteStreamKey}.
     * @param  [in] state The remote video stream status. See RemoteVideoState{@link #RemoteVideoState}.
     * @param  [in] reason For the remote video stream state change. See RemoteVideoStateChangeReason{@link #RemoteVideoStateChangeReason}.
     */
    virtual void OnRemoteVideoStateChanged(
            RemoteStreamKey key, RemoteVideoState state, RemoteVideoStateChangeReason reason) {
        (void)key;
        (void)state;
        (void)reason;
    }


    /** 
     * @hidden
     * @type callback
     * @region Video management
     * @brief This callback is triggered when subscribing to the remote user
     * @param  [in] user_id The ID of the remote user
     * @param  [in] is_reconnect Whether it is a reconnection
     */
    virtual void OnSubscribe(const char* user_id, bool is_reconnect) {
        (void)user_id;
        (void)is_reconnect;
    }

    /** 
     * @hidden
     * @type callback
     * @region Video management
     * @brief This callback is triggered when the remote user is unsubscribed
     * @param  [in] user_id The ID of the remote user
     * @param  [in] is_reconnect Whether it is a reconnection
     */
    virtual void OnUnSubscribe(const char* user_id, bool is_reconnect) {
        (void)user_id;
        (void)is_reconnect;
    }

    /** 
     * @type callback
     * @region Room management
     * @brief Audio first frame sending status change callback
     * @param  [in] user Local user information. See RtcUser{@link #RtcUser}
     * @param  [in] state First frame sending status. See FirstFrameSendState{@link #FirstFrameSendState}
     */
    virtual void OnAudioFrameSendStateChanged(const RtcUser& user, FirstFrameSendState state) {
        (void)user;
        (void)state;
    }

    /** 
     * @type callback
     * @region Room management
     * @brief Video first frame sending status change callback
     * @param  [in] user Local user information. See RtcUser{@link #RtcUser}
     * @param  [in] state First frame sending status. See FirstFrameSendState{@link #FirstFrameSendState}
     */
    virtual void OnVideoFrameSendStateChanged(const RtcUser& user, FirstFrameSendState state) {
        (void)user;
        (void)state;
    }

    /** 
     * @type callback
     * @region room management
     * @brief Screen sharing streaming video first frame sending status change callback
     * @param  [in] user Local user information. See RtcUser{@link #RtcUser}
     * @param  [in] state First frame sending status. See FirstFrameSendState{@link #FirstFrameSendState}
     */
    virtual void OnScreenVideoFrameSendStateChanged(const RtcUser& user, FirstFrameSendState state) {
        (void)user;
        (void)state;
    }

    /** 
     * @type callback
     * @region  room management
     * @brief  Audio first frame playback state change callback
     * @param  [in] user Remote user information. See RtcUser{@link #RtcUser}
     * @param  [in] state First frame playback status. See FirstFramePlayState{@link #FirstFramePlayState}
     */
    virtual void OnAudioFramePlayStateChanged(const RtcUser& user, FirstFramePlayState state) {
        (void)user;
        (void)state;
    }

    /** 
     * @type callback
     * @region  room management
     * @brief  Video first frame playback state change callback
     * @param  [in] user Remote user information. See RtcUser{@link #RtcUser}
     * @param  [in] state First frame playback state. See FirstFramePlayState{@link #FirstFramePlayState}
     */
    virtual void OnVideoFramePlayStateChanged(const RtcUser& user, FirstFramePlayState state) {
        (void)user;
        (void)state;
    }

    /** 
     * @type callback
     * @region room management
     * @brief Screen sharing streaming video first frame playback state change callback
     * @param  [in] user Remote user information. See RtcUser{@link #RtcUser}
     * @param  [in] state First frame playback state. See FirstFramePlayState{@link #FirstFramePlayState}
     */
    virtual void OnScreenVideoFramePlayStateChanged(const RtcUser& user, FirstFramePlayState state) {
        (void)user;
        (void)state;
    }

    /*
     * @hidden
     * @brief callback when the maximum screen share fps is changed
     * @param [in] fps
     *           maximum screen share fps
     */
    virtual void OnMaximumScreenShareFpsUpdated(int fps) {
        (void)fps;
    }

    /** 
     * @hidden
     * @brief Callback when the maximum screen shared pixels (resolution) is chang
     * @param  [in] screenPixels
     *        The recommended maximum video width × height value to maintain the frame rate
     *
     */
    virtual void OnMaximumScreenSharePixelsUpdated(int screenPixels) {
        (void)screenPixels;
    }

    /** 
     * @hidden
     * @type callback
     * @region video management
     * @brief Spliced view user requests avatar callback
     * @param  [in] uid User ID
     */
    virtual void OnNeedAvatar(const char* uid) {
        (void)uid;
    }

    /** 
     * @type callback
     * @region Video management
     * @brief This callback is triggered when the server level MuteUser/UnmuteUser method is invoked to disable/unban the sending of a specified user's video stream in a specified room.
     * @param  [in] uid User ID of the video stream that was disabled/unbanned
     * @param  [in] banned Video stream sending status <br>
     *         + True: Video stream sending was disabled <br>
     *         + False: Video stream sending was unbanned
     * @notes   <br>
     *         + When the specified user in the room is disabled/unbanned Video stream sending, all users in the room will receive the callback .. <br>
     *         + If the banned user checks out and then enters the room, it will still be banned status, and everyone in the room will receive the callback again. <br>
     *         + If the banned user is disconnected and reconnected to the room, it will still be banned status, and only the person will receive the callback again. <br>
     *         + After the specified user is banned, other users in the room will check out and enter the room again, and will receive the callback again. <br>
     *         + When the number of calls exceeds 5, only blocked/unblocked users will receive the callback. <br>
     *         + The same room is created again after dissolution, and the state in the room is empty.
     */
    virtual void OnVideoStreamBanned(const char* uid, bool banned) {
        (void)uid;
        (void)banned;
    }

    /** 
     * @type callback
     * @region Multi-Room
     * @brief Stream publisher will receive this callback when the A/V synchronization state changes after SetMultiDeviceAVSync{@link #IRtcRoom#SetMultiDeviceAVSync} is called.
     * @param state A/V synchronization state, see AVSyncState{@link #AVSyncState}.
     */
    virtual void OnAVSyncStateChange(AVSyncState state) {
        (void)state;
    }

#endif  // ByteRTC_AUDIO_ONLY
    /** 
     * @type callback
     * @region Audio event callback
     * @brief This callback is triggered when the server level MuteUser/UnmuteUser method is called to disable/unban the sending of the specified user's audio and video stream in a specified room.
     * @param  [in] uid Disabled/unbanned audio stream user ID
     * @param  [in] banned Audio stream sending status <br>
     *         + True: audio stream sending is disabled <br>
     *         + False: audio stream sending is unbanned
     * @notes   <br>
     *         + Specified users in the room are prohibited/all users in the room when audio stream sending is unbanned Will receive the callback. <br>
     *         + If the banned user checks out and then enters the room, it will still be banned status, and everyone in the room will receive the callback again. <br>
     *         + If the banned user is disconnected and reconnected to the room, it will still be banned status, and only the person will receive the callback again. <br>
     *         + After the specified user is banned, other users in the room will check out and enter the room again, and will receive the callback again. <br>
     *         + When the number of calls exceeds 5, only blocked/unblocked users will receive the callback. <br>
     *         + The same room is created again after dissolution, and the state in the room is empty.
     */
    virtual void OnAudioStreamBanned(const char* uid, bool banned) {
        (void)uid;
        (void)banned;
    }

    /** 
     * @type callback
     * @region Multi-room
     * @brief Callback returning the state and errors during relaying the media stream to each of the rooms
     * @param infos Array of the state and errors of each target room. Refer to ForwardStreamStateInfo{@link #ForwardStreamStateInfo} for more inforamtion.
     * @param info_count The number of the target rooms
     */
    virtual void OnForwardStreamStateChanged(ForwardStreamStateInfo* infos, int info_count) {
    }

    /** 
     * @type callback
     * @region Multi-room
     * @brief Callback returning the events during relaying the media stream to each room
     * @param infos Array of the event of each target room. Refer to ForwardStreamEventInfo{@link #ForwardStreamEventInfo} for more information.
     * @param info_count The number of the target rooms
     */
    virtual void OnForwardStreamEvent(ForwardStreamEventInfo* infos, int info_count) {
    }

    /** 
     * @hidden(Linux)
     * @type callback
     * @brief Report the network quality of the users every 2s after the local user joins the room.
     * @param [in] localQuality Local network quality. Refer to NetworkQualityStats{@link #NetworkQualityStats} for details.
     * @param [in] remoteQualities Network quality of the subscribed users. Refer to NetworkQualityStats{@link #NetworkQualityStats} for details.
     * @param [in] remoteQualityNum Array length of `remoteQualities`
     */
    virtual void OnNetworkQuality(const NetworkQualityStats& localQuality, const NetworkQualityStats* remoteQualities, int remoteQualityNum) {
    }

    /** 
     * @type callback
     * @brief Callback of the result of publishing the public stream<br>
     *        You will be informed of the result of publishing the public stream by this callback after calling StartPushPublicStream{@link #IRtcRoom#StartPushPublicStream}.
     * @param [in] public_stream_id ID of the public stream
     * @param [in] errorCode Code for the result of publishing the public stream<br>
     *        `200`: Success
     */
    virtual void OnPushPublicStreamResult(const char* public_stream_id, int errorCode) {
        (void)public_stream_id;
        (void)errorCode;
    }
};

} // namespace bytertc

#endif // BYTE_RTC_ROOM_EVENT_HANDLER_INTERFACE_H__
