/*
 * Copyright (c) 2020 The BytePlusRTC project authors. All Rights Reserved.
 * @brief BytePlusRTC Room Interface
 */

#pragma once

#ifndef BYTE_RTC_ROOM_INTERFACE_H__
#define BYTE_RTC_ROOM_INTERFACE_H__

#include "bytertc_video_effect_interface.h"  // NOLINT
#include "bytertc_defines.h"
#include "bytertc_room_event_handler_interface.h"
#include "bytertc_transcoder_interface.h"
#include "bytertc_publicstream_interface.h"
#include "bytertc_range_audio_interface.h"
#include "bytertc_spatial_audio_interface.h"

namespace bytertc {

/** 
 * @type api
 * @brief  Room interface
 */
class IRtcRoom {
public:
    /** 
     * @hidden
     * @brief  Destructor
     */
    virtual ~IRtcRoom() {
    }

    /** 
     * @type api
     * @region  multi-room
     * @brief  Leave and destroy the room created by calling CreateRtcRoom{@link #IRtcEngineLite#CreateRtcRoom}.
     */
    virtual void Destroy() = 0;

    /** 
     * @hidden
     * @deprecated since 327.1, use SetUserVisibility instead
     * @type api
     * @region multiroom
     * @brief To set the user persona. The default role is live streaming host.   <br>
     *        By setting different user personas, you can control the user's behavior in the room: <br>
     *         + Whether audio & video streams can be published; <br>
     *         + Whether the user himself is invisible in the room. <br>
     * @param  [in] role User persona. See UserRoleType{@link #UserRoleType}.
     * @notes   <br>
     *        + Users can call this method to set user persona before and after joining the room. <br>
     *        + In the room, after calling this method to successfully switch roles, other users in the room will receive corresponding callback notifications: <br>
     *            -When Switching from an invisible user to a live streaming host, other users in the room will receive OnUserJoined{@link #IRTCRoomEventHandler#OnUserJoined}; <br>
     *            -When Switching from a live streaming host to an invisible user, other users in the room will receive the corresponding callback notification The user receives OnUserLeave{@link #OnUserLeave}.
     */
    virtual void SetUserRole(UserRoleType role) = 0;



    /** 
     * @type api
     * @region Multi-room
     * @brief Set the visibility of the user in the room. Visible by default.   <br>
     * @param enable  Visibility of the user in the room: <br>
     *         + True: The user can publish media streams. And the other users in the room get informed of the behaviors of the user, such as Joining room, starting video capture, and Leaving room.<br>
     *         + False: The user cannot publish media streams. And the other users in the room do not get informed of the behaviors of the user, such as joining, starting video capture, or leaving.<br>
     * @return   <br>
     *          + 0: Success. <br>
     *          + < 0: Failure <br>
     * @notes   <br>
     *        + You can call this API whether the user is in a room or not. <br>
     *        + When you call this API, the other users in the room will be informed via the related callback: <br>
     *            - Switch from `false` to `true`: OnUserJoined{@link #IRTCRoomEventHandler#OnUserJoined}<br>
     *            - Switch from `true` to `false`: OnUserLeave{@link #IRTCRoomEventHandler#OnUserLeave} <br>
     *        + The invisible user will receive the warning code, `kWarningCodePublishStreamForbiden`, when trying to publish media streams.
     */
    virtual void SetUserVisibility(bool enable) = 0;



    /** 
     * @type api
     * @region multiroom
     * @brief By Setting `IRTCRoomEventHandler` you can get informed of the room-related events realted to this `IRtcRoom` instance.
     * @param  [in] room_event_handler Refer to IRTCRoomEventHandler{@link #IRTCRoomEventHandler}.
     */
    virtual void SetRtcRoomEventHandler(IRTCRoomEventHandler* room_event_handler) = 0;

    /** 
     * @hidden
     * @deprecated since 326.1, use function joinRoom with MultiRoomConfig parameter instead
     * @type api
     * @region multiroom
     * @brief To join the room.
     *        The user calls this method to join the room corresponding to this IRtcRoom{@link #IRtcRoom}
     * . Users in a room can talk to each other. Before calling the join room, you need to call the CreateRtcRoom{@link #CreateRtcRoom} method
     *         To create the IRtcRoom{@link #IRtcRoom} object corresponding to the room.
     * @param  [in] token
     *         Dynamic key for authenticating the logged-in user. You need to bring Token to enter the room. You can generate temporary Token
     *  In the console for testing. The official launch requires the use of the key SDK to generate and issue Token at your server level.
     * @param  [in] user_info
     *         User information, including user id, and additional information that the user needs to pass through
     * @param  [in] profileType Room mode. See RoomProfileType{@link #RoomProfileType} enumeration type for details. <br>
     * @return
     *      0: success
     *      Other: failure
     * @notes
     *        1. Apps with different App IDs are not interoperable.
     *        2. After calling this method to successfully join the room, the local end will receive a callback notification of OnRoomStateChanged{@link #IRTCRoomEventHandler#OnRoomStateChanged} with an errorcode of 0
     *  And JoinRoomType{@link #JoinRoomType} as KJoinRoomFirst. 3. In the case of poor local network conditions, the SDK
     *  May lose connection with the server. At this time, the SDK will automatically reconnect. After successful reconnection, you will receive OnRoomStateChanged{@link #IRTCRoomEventHandler#OnRoomStateChanged}
     *  Callback notification, errorcode is 0, JoinRoomType{@link #JoinRoomType} is kJoinRoomTypeReconnected 4. In the same room with the same App ID
     * , The user ID of each user must be unique. If two users have the same user ID, the user who joined the room later will kick the user who joined the
     *           Room first out of the room, and the user who joined the room first will receive an OnStreamStateChanged{@link #IRTCRoomEventHandler#OnStreamStateChanged} callback notification. The
     *           Error type is repeated login kErrorCodeDuplicateLogin{@link #kErrorCodeDuplicateLogin}.
     *        5. Please make sure that the App ID used to generate the Token is the same as the App ID used to create the engine, otherwise it will cause the joining room to fail.
     *        6. This method can only add the corresponding room of this IRtcRoom{@link #IRtcRoom}
     *        . If you need to add other rooms, you need to create the IRtcRoom{@link #IRtcRoom} object of the corresponding room.
     */
    virtual void JoinRoom(const char* token, const UserInfo& user_info, RoomProfileType profile_type) = 0;

    /** 
     * @type api
     * @region Multi-room
     * @brief Create/join a room: create a room when the room does not exist; when the room exists, users who have not joined the room can join the room.   <br>
     *        Users in the same room can talk to each other. <br>
     *         Repeated calls after entering the room are invalid. The user must call LeaveRoom{@link #IRtcRoom#LeaveRoom} to exit the current room before joining the next room. <br>
     * @param  [in] token Dynamic key for authenticating the logged-in user. <br>
     *         You need to bring Token to enter the room. When testing, you can use the console to generate temporary tokens. The official launch requires the use of the key SDK to generate and issue tokens at your server level. <br>
     *        + Apps with different App IDs cannot communicate with each other. <br>
     *        + Make sure that the App ID used to generate the Token is the same as the App ID used to create the engine, otherwise it will cause the join room to fail. <br>
     * @param  [in] room_id Added room ID. <br>
     *        The room ID is a non-empty string with a length of less than 128 bytes, and the following character set ranges are supported: <br>
     *            1. 26 capital letters A~ Z. <br>
     *            2. 26 lowercase letters a~ z. <br>
     *            3. 10 numbers 0~ 9. <br>
     *            4. Underline "_", at character "@", minus sign "-". <br>
     * @param  [in] user_info User information. See UserInfo{@link #UserInfo}. <br>
     * @param  [in] room_config Room parameter configuration, set the room mode and whether to automatically publish or subscribe to the flow. See RTCRoomConfig{@link #RTCRoomConfig} for the specific configuration mode. <br>
     * @return   <br>
     *         + 0: Success <br>
     *         + -1: Room_id/user_info.uid contains invalid parameters. <br>
     *         + -2: Already in the room. After the interface call is successful, as long as the return value of 0 is received and LeaveRoom:{@link #IRtcRoom#LeaveRoom} is not called successfully, this return value is triggered when the room entry interface is called again, regardless of whether the filled room ID and user ID are duplicated. <br>
     * @notes   <br>
     *        + In the same room with the same App ID, the user ID of each user must be unique. If two users have the same user ID, the user who entered the room later will kick the user who entered the room out of the room, and the user who entered the room will receive the OnStreamStateChanged{@link #IRTCRoomEventHandler#OnStreamStateChanged} callback notification. For the error type. See kErrorCodeDuplicateLogin in ErrorCode{@link #ErrorCode}. <br>
     *        + Local users will receive OnRoomStateChanged{@link #IRTCRoomEventHandler#OnRoomStateChanged} callback notification after calling this method to join the room successfully. <br>
     *        + Local users call SetUserVisibility{@link #IRtcRoom#SetUserVisibility} to make themselves visible and then join the room, and remote users will receive OnUserJoined{@link #IRTCRoomEventHandler#OnUserJoined}. <br>
     *        + After the user successfully joins the room, the SDK may lose connection to the server in case of poor local network conditions, and the SDK will automatically reconnect at this time. After a successful reconnection, you will receive a local OnRoomStateChanged{@link #IRTCRoomEventHandler#OnRoomStateChanged} callback notification. <br>
     */
    virtual int JoinRoom(const char* token, const UserInfo& user_info, const MultiRoomConfig& config) = 0;

    /** 
      * @type api
      * @region  room management
      * @brief  Leave RTC room.   <br>
      *        The user calls this method to leave the RTC room, end the call process, and release all call-related resources. <br>
      *        It is an asynchronous operation, and the call returns without actually exiting the room. When actually exiting the room, you will receive OnLeaveRoom{@link #IRTCRoomEventHandler#OnLeaveRoom}. <br>
      * @return   <br>
      *         + 0: Success <br>
      *         + < 0: Failure <br>
      * @notes   <br>
      *        + When visible users leave the room, others in the room will receive OnUserLeave{@link #IRTCRoomEventHandler#OnUserLeave}. <br>
      *        + If the engine is destroyed immediately after this method is called, you will not receive OnLeaveRoom{@link #IRTCRoomEventHandler#OnLeaveRoom}. <br>
      *        + After calling JoinRoom{@link #IRtcRoom#JoinRoom} to join the room, you must call this API to end the call, otherwise you cannot start the next call. You can call this API in or out of the room. Repeated calls cause no negative effect. <br>
      */
    virtual void LeaveRoom() = 0;

    /** 
     * @type api
     * @brief Update Token. <br>
     *        A Token used to join rooms has an expiration time. You will be informed 30 sec before the expiration of the token by OnTokenWillExpire{@link #IRTCRoomEventHandler#OnTokenWillExpire}. Call this API to renew the Token. Or the user will be removed from the room as the Token expires. <br>
     *         If you called JoinRoom{@link #IRtcRoom#JoinRoom} to join a room or reconnected into a room with an invalid Token, it would fail. You would get informed by OnRoomStateChanged{@link #IRTCRoomEventHandler#OnRoomStateChanged}. The error code is the `ERROR_CODE_INVALID_TOKEN` in ErrorCode{@link #ErrorCode}. Reacquire a token and call this API to renew the token. <br>
     * @param token  Valid token.  <br>
     *        If the Token is invalid, SDK will emit `ERROR_CODE_UPDATE_TOKEN_WITH_INVALID_TOKEN` defined in an ErrorCode{@link #ErrorCode}.
     * @return  <br>
     *          + 0: Success. <br>
     *          + < 0: Failure <br>
     * @notes  When the Token expired, the user would be removed from the room. And you would get informed of  `ERROR_CODE_EXPIRED_TOKEN` by OnStreamStateChanged{@link #IRTCRoomEventHandler#OnStreamStateChanged}. Reacquire a token and call JoinRoom{@link #IRtcRoom#JoinRoom} with the token to rejoin the room.
     */
    virtual void UpdateToken(const char* token) = 0;

    /** 
     * @type api
     * @region Multi-room
     * @brief Send a text message (P2P) to the specified user in the room
     * @param  [in] uid <br>
     *        Message Receives the user's ID
     * @param  [in] message <br>
     *        Text message content sent. <br>
     *         Message does not exceed 62KB.
     * @return The number of the message sent this time is incremented from 1.
     * @notes   <br>
     *        + Before sending an in-room text message, you must call JoinRoom{@link #IRtcRoom#JoinRoom} to join the room. <br>
     *        + After calling this function, you will receive an OnUserMessageSendResult{@link #IRTCRoomEventHandler#OnUserMessageSendResult} callback to notify the message sender that the sending was successful or failed; <br>
     *        + If the text message is sent successfully, the user specified by uid will receive OnUserMessageReceived{@link #IRTCRoomEventHandler#OnUserMessageReceived} callback.
     */
    virtual int64_t SendUserMessage(const char* uid, const char* message) = 0;

    /** 
     * @type api
     * @region multi-room
     * @brief Sends a binary message (P2P) to the specified user in the room
     * @param  [in] uid <br>
     *        The ID of the message receiving user
     * @param  [in] length <br>
     *        The length of the binary string.
     * @param  [in] message <br>
     *         The content of the binary message. The message does not exceed 46KB.
     *         The number of the message sent this time is incremented from
     * @return 1.  <br>
     * @notes   + Before sending in-room binary messages, you must call JoinRoom{@link #IRtcRoom#JoinRoom} to join the room. <br>
     *        + After calling this function, you will receive an OnUserMessageSendResult{@link #IRTCRoomEventHandler#OnUserMessageSendResult} callback to notify the sender of the success or failure of the message; <br>
     *        + If the binary message is sent successfully, the user specified by uid will receive the OnUserBinaryMessageReceived{@link #IRTCRoomEventHandler#OnUserBinaryMessageReceived} callback.
     */
    virtual int64_t SendUserBinaryMessage(const char* uid, int length, const uint8_t* message) = 0;

    /** 
     * @type api
     * @region Multi-room
     * @brief Send broadcast messages to all other users in the room.
     * @param  [in] message <br>
     *        The broadcast message sent by the user <br>
     *        Message does not exceed 62KB.
     * @notes   <br>
     *        + Before sending in-room binary messages, you must call JoinRoom{@link #IRtcRoom#JoinRoom} to join the room. <br>
     *        + After calling this function, you get an OnRoomMessageSendResult{@link #IRTCRoomEventHandler#OnRoomMessageSendResult} callback. <br>
     *        + Other users in the same room receive an OnRoomMessageReceived{@link #IRTCRoomEventHandler#OnRoomMessageReceived} callback.
     */
    virtual int64_t SendRoomMessage(const char* message) = 0;
    /** 
     * @type api
     * @region Multi-room
     * @brief Send broadcast messages to all other users in the room.
     * @param  [in] size <br>
     *        The length of the binary message sent
     * @param  [in] message <br>
     *        The binary broadcast message sent by the user <br>
     *        The message does not exceed 46KB.
     * @return The number of the message sent this time is incremented from 1.
     * @notes   <br>
     *        + Before sending in-room binary messages, you must call JoinRoom{@link #IRtcRoom#JoinRoom} to join the room. <br>
     *        + After calling this function, you get an OnRoomMessageSendResult{@link #IRTCRoomEventHandler#OnRoomMessageSendResult} callback. <br>
     *        + Other users in the same room receive an OnRoomBinaryMessageReceived{@link #IRTCRoomEventHandler#OnRoomBinaryMessageReceived} callback.
     */
    virtual int64_t SendRoomBinaryMessage(int size, const uint8_t* message) = 0;

    /** 
     * @hidden
     * @deprecated since 336.1, use PublishStream instead.
     * @type api
     * @region Multi-room
     * @brief Publish media streams in the current room.
     * @notes   <br>
     *        + Call this API to publish media streams in Multi-room mode. In this mode, the Auto-Publish is disabled by default.  <br>
     *        + An invisible user cannot publish media streams. Call SetUserVisibility{@link #IRtcRoom#SetUserVisibility} to change the visibility of a user.  <br>
     *        + Call PublishScreen{@link #IRtcRoom#PublishScreen} to start screen sharing.
     *        + Call StartForwardStreamToRooms{@link #IRtcRoom#StartForwardStreamToRooms} to forward the published streams to the other rooms.
     *        + After publishing media streams by calling this API, the other users in the room will be informed via OnStreamAdd{@link #IRTCRoomEventHandler#OnStreamAdd}.
     */
    virtual void Publish() = 0;

    /** 
     * @type api
     * @region Room Management
     * @brief Publishes media streams captured by camera/microphone in the current room.
     * @param [in] type Media stream type, used for specifying whether to publish audio stream or video stream. See MediaStreamType{@link #MediaStreamType}.
     * @notes  <br>
     *        + Streams are not published automatically in multi-room mode by default, you need to call this API to publish streams manually.  <br>
     *        + An invisible user cannot publish media streams. Call SetUserVisibility{@link #IRtcRoom#SetUserVisibility} to change your visibility in the room.  <br>
     *        + Call PublishScreen{@link #IRtcRoom#PublishScreen} to start screen sharing.  <br>
     *        + Call StartForwardStreamToRooms{@link #IRtcRoom#StartForwardStreamToRooms} to forward the published streams to the other rooms.  <br>
     *        + After you call this API, the other users in the room will receive OnUserPublishStream{@link #IRTCRoomEventHandler#OnUserPublishStream}. Those who successfully received your streams will receive OnFirstRemoteAudioFrame{@link #IRTCRoomEventHandler#OnFirstRemoteAudioFrame}/OnFirstRemoteVideoFrameDecoded{@link #IRTCRoomEventHandler#OnFirstRemoteVideoFrameDecoded} at the same time.  <br>
     *        + Call UnpublishStream{@link #IRtcRoom#UnpublishStream} to stop publishing streams.
     */
    virtual void PublishStream(MediaStreamType type) = 0;

    /** 
     * @hidden
     * @deprecated since 336.1, use UnpublishStream instead.
     * @type api
     * @region Multi-room
     * @brief Stop publishing media streams in the room.
     * @notes   <br>
     *        + In Multi-room mode, call this API to stop publishing media streams after starting by calling Publish{@link #IRtcRoom#Publish}. <br>
     *        + After stop publishing media streams by calling this API, the other users in the room will be informed via OnStreamRemove{@link #IRTCRoomEventHandler#OnStreamRemove} callback notification.
     */
    virtual void Unpublish() = 0;

    /** 
     * @type api
     * @region Room Management
     * @brief Stops publishing media streams captured by camera/microphone in the current room.
     * @param [in] type Media stream type, used for specifying whether to stop publishing audio stream or video stream. See MediaStreamType{@link #MediaStreamType}.
     * @notes  <br>
     *         + After calling PublishStream{@link #IRtcEngineLite#PublishStream}, call this API to stop publishing streams. <br>
     *         + After calling this API, the other users in the room will receive OnUserUnPublishStream{@link #IRTCRoomEventHandler#OnUserUnPublishStream}.
     */
    virtual void UnpublishStream(MediaStreamType type) = 0;


    /** 
     * @hidden
     * @deprecated since 327.1, use PublishScreen without params instead
     * @type api
     * @region multiroom
     * @brief To publish local screen share streams to rooms.
     * @param  [in] parameters
     *        Screenshare stream encoding parameters. See ScreenParameters{@link #ScreenParameters}
     * @notes   <br>
     *        + This method is suitable for users who call the SetUserVisibility{@link #IRtcRoom#SetUserVisibility} method to make themselves visible. <br>
     *        + After this method is called to publish the local screen sharing flow, the user also needs to periodically call the PushScreenFrame {@link
     *  #IRtcEngineLite #PushScreenFrame} Method by setting a timed task to push the screen data that needs to be sent to the SDK. <br>
     *        + The remote receives the OnFirstRemoteVideoFrameRendered{@link #IRTCRoomEventHandler#OnFirstRemoteVideoFrameRendered}
     * , OnStreamAdd{@link #IRTCRoomEventHandler#OnStreamAdd} event. <br>
     *        + This function and the StartScreenVideoCapture{@link #IRtcEngineLite#StartScreenVideoCapture} function are mutually exclusive calls, which can be called after the
     * UnpublishScreen{@link #UnpublishScreen} function is called. <br>
     *        + After receiving the OnFirstRemoteVideoFrameRendered{@link #IRTCRoomEventHandler#OnFirstRemoteVideoFrameRendered} event, set the remote screen sharing view by calling the SetRemoteVideoCanvas{@link #SetRemoteVideoCanvas} or SetRemoteVideoSink{@link #SetRemoteVideoSink} function. <br>
     *        + Locally, you can call the SetLocalVideoCanvas{@link #IRtcEngineLite#SetLocalVideoCanvas} or SetLocalVideoSink{@link #IRtcEngineLite#SetLocalVideoSink} function to set the local screen share view. <br>
     *        + You can also get the original data source by registering the RegisterVideoFrameObserver{@link #RegisterVideoFrameObserver} video callback object to listen for the OnLocalScreenFrame{@link #IVideoFrameObserver#OnLocalScreenFrame} local screen video callback event and the OnRemoteScreenFrame {@link #IVideoFrameObserver #OnRemoteScreenFrame} remote screen share video callback event. <br>
     *        + This method can only be called after JoinRoom{@link #IRtcRoom#JoinRoom}. <br>
     */
    virtual void PublishScreen(const ScreenParameters& parameters) = 0;

    /** 
     * @hidden
     * @deprecated since 336.1
     * @type api
     * @region screen share
     * @brief Publish local screen share streams to the room.
     * @notes   <br>
     *        + You must join the room before you can call this method. <br>
     *        + In live, game, cloud game room mode, only visible users can call this method. You can set the user's visibility in the room by calling the SetUserVisibility{@link #IRtcRoom#SetUserVisibility} method. <br>
     *        + This method only affects the publishing status of the screen shared video stream, and does not affect the collection of the screen video stream. You can capture in two ways: <br>
     *               - RTC SDK internal capture: see StartScreenVideoCapture{@link #IRtcEngineLite#StartScreenVideoCapture} and StartScreenAudioCapture{@link #IRtcEngineLite#StartScreenAudioCapture} <br>
     *               - RTC SDK external capture: you can capture the screen audio & video stream yourself and periodically call PushScreenFrame{@link #IRtcEngineLite#PushScreenFrame} and PushScreenAudioFrame{@link #IRtcEngineLite#PushScreenAudioFrame} Push the collected audio & video frames to the RTC SDK for encoding transmission. <br>
     *        + After successful publication, the remote will receive an OnStreamAdd{@link #IRTCRoomEventHandler#OnStreamAdd} callback; if you call RegisterVideoFrameObserver{@link #RegisterVideoFrameObserver} to register a video frame callback observer, then you will also receive an OnRemoteScreenFrame{@link #IVideoFrameObserver#OnRemoteScreenFrame} callback.
     *        + Call UnpublishScreen{@link #IRtcRoom#UnpublishScreen} to cancel publishing.
     */
    virtual void PublishScreen() = 0;

    /** 
     * @type api
     * @region Screen Sharing
     * @brief Publishes local screen sharing streams in the current room.
     * @param [in] type Media stream type, used for specifying whether to publish audio stream or video stream. See MediaStreamType{@link #MediaStreamType}.
     * @notes  <br>
     *         + An invisible user cannot publish media streams. Call SetUserVisibility{@link #IRtcRoom#SetUserVisibility} to change your visibility in the room. <br>
     *         + After you called this API, the other users in the room will receive OnUserPublishScreen{@link #IRTCRoomEventHandler#OnUserPublishScreen}. Those who successfully received your streams will receive OnFirstRemoteAudioFrame{@link #IRTCRoomEventHandler#OnFirstRemoteAudioFrame}/OnFirstRemoteVideoFrameDecoded{@link #IRTCRoomEventHandler#OnFirstRemoteVideoFrameDecoded} at the same time.  <br>
     *         + Call StartForwardStreamToRooms{@link #IRtcRoom#StartForwardStreamToRooms} to forward the published streams to the other rooms.  <br>
     *         + Call UnpublishScreen{@link #IRtcRoom#UnpublishScreen} to stop publishing screen sharing streams.
     */
    virtual void PublishScreen(MediaStreamType type) = 0;

    /** 
     * @hidden
     * @deprecated since 336.1
     * @type api
     * @region screen share
     * @brief Stop publishing local screen share streams to the room.
     * @notes The remote receives the OnStreamRemove{@link #IRTCRoomEventHandler#OnStreamRemove} event.
     * @notes   <br>
     *        + This method only affects the publishing status of the screen shared video stream, and does not affect the collection of the screen video stream. <br>
     *        + Call PublishScreen{@link #IRtcRoom#PublishScreen} to start publishing.
     */
    virtual void UnpublishScreen() = 0;

    /** 
     * @type api
     * @region Screen Sharing
     * @brief Stops publishing local screen sharing streams in the current room.
     * @param [in] type Media stream type, used for specifying whether to stop publishing audio stream or video stream. See MediaStreamType{@link #MediaStreamType}.
     * @notes  <br>
     *         + After calling PublishScreen{@link #IRtcRoom#PublishScreen}, call this API to stop publishing streams. <br>
     *         + After calling this API, the other users in the room will receive OnUserUnPublishScreen{@link #IRTCRoomEventHandler#OnUserUnPublishScreen}.
     */
    virtual void UnpublishScreen(MediaStreamType type) = 0;

    /** 
     * @hidden
     * @deprecated since 326.1, use SubscribeUserStream instead
     * @type api
     * @region multiroom
     * @brief To subscribe to the specified room remote audio & video stream.   <br>
     *         Turn off the automatic subscription function. When using the manual subscription mode, the user calls this method to subscribe to the audio & video stream in the room on demand. <br>
     *        This method only takes effect when the automatic subscription function is turned off. Users need to call the EnableAutoSubscribe{@link #EnableAutoSubscribe}
     *  Method to turn off the automatic subscription function before joining the room to use the manual subscription mode. <br> Users call this method when the automatic subscription function is turned on. The SDK does not subscribe to the audio & video stream, and notifies the user that the subscription failed through a
     * OnRoomWarning{@link #OnRoomWarning} callback with the callback warning code
     * kWarningCodeSubscribeStreamForbiden{@link #kWarningCodeSubscribeStreamForbiden}. <br> When the subscription stream fails, the SDK
     *  Will call back OnStreamStateChanged{@link #IRTCRoomEventHandler#OnStreamStateChanged} according to the cause of the failure to notify the user. The specific error can be described by the error code ErrorCode {@link
     *  #ErrorCode}. <br>
     * @param  [in] user_id The user ID of the remote user that specifies the subscription. <br>
     * @param  [in] info Select the subscription configuration. See the data structure SubscribeConfig{@link #SubscribeConfig}. <br>
     * @notes   <br>
     *        + This method only supports subscribing to remote audio & video streams in the current room. If the specified stream does not exist in the current room, the SDK does not subscribe. <br>
     *        + After the user calls the JoinRoom{@link #IRtcRoom#JoinRoom} method to join the room: 1. When a new audio & video stream is released in the room, you will receive a OnStreamAdd{@link #OnStreamAdd} callback notification locally; 2. OnStreamRemove {@link #OnStreamRemove} callback notification is received locally when audio & video streams in the room stop publishing. Through these two callbacks, the user can obtain the audio & video stream information in the current room and call this method to subscribe to the audio & video stream when needed. <br>
     */
    virtual void SubscribeStream(const char* user_id, const SubscribeConfig& info) = 0;

    /** 
     * @hidden
 * @deprecated since 336.1, use SubscribeStream, UnsubscribeStream, SubscribeScreen and UnsubscribeScreen instead.
     * @type api
     * @region Multi-room
     * @brief Subscribe to media streams in the room.   <br>
     *        Call this API to subscribe to media streams in the room regardless of subscription mode, either Auto or Manual. <br>
     *        Call this API to update the properties and configurations of the stream you have already subscribed to. <br>
     * @param  [in] user_id User ID of the publisher of the subscribed media stream <br>
     * @param  [in] stream_type Mainstream or screen-sharing stream. Refer to StreamIndex{@link #StreamIndex} for details.
     * @param  [in] media_type Media stream type to be subscribed to. Refer to  SubscribeMediaType{@link #SubscribeMediaType} for details. <br>
     * @param  [in] video_config Subscription configuration of video streams. Refer to SubscribeVideoConfig{@link #SubscribeVideoConfig} for details. <br>
     * @notes   <br>
     *         + Get the media stream information in the room from OnStreamAdd{@link #IRTCRoomEventHandler#OnStreamAdd} and OnStreamRemove{@link #IRTCRoomEventHandler#OnStreamRemove}. <br>
     *         + If the subscription fails, you will receive OnStreamStateChanged{@link #IRTCRoomEventHandler#OnStreamStateChanged}. Refer to ErrorCode{@link #ErrorCode} for the specific reason of failure.
     *         + You can update the subscription configuration by calling the API even after calling PauseAllSubscribedStream{@link #IRtcRoom#PauseAllSubscribedStream} to pause streaming. Call ResumeAllSubscribedStream{@link #IRtcRoom#ResumeAllSubscribedStream} to resume streaming and apply the changes. <br>
     */
    virtual void SubscribeUserStream(const char* user_id, StreamIndex stream_type, SubscribeMediaType media_type, const SubscribeVideoConfig& video_config) = 0;
    /** 
     * @type api
     * @region Video Management
     * @brief Sets your expected configuration of the remote video stream that you want to subscribe to or have subscribed to.
     * @param [in] user_id The ID of the target stream publisher
     * @param [in] remote_video_config Your expected configuration, see RemoteVideoConfig{@link #RemoteVideoConfig}.
     * @notes <br>
     *        + This API only works after the publisher calls EnableSimulcastMode{@link #IRtcEngineLite#EnableSimulcastMode} to enable publishing multiple video streams, in which case the subscriber will receive the stream from the publisher that is closest to the set configuration;  otherwise the subscriber will only receive one video stream with a resolution of 640px × 360px and a frame rate of 15fps.  <br>
     *        + If you don't call this API after the publisher enables the function of publishing multiple streams, you will receive by default the video stream with the largest resolution set by the publisher.  <br>
     *        + You should call this API in the room. If you want to call it before entering the room, you should set the `remote_video_config` in the `room_config` when calling JoinRoom{@link #IRtcRoom#JoinRoom}.  <br>
     *        + You can call this API anytime before or after subscribing to the target stream. If you have already subscribed to the target stream when calling this API, your settings will take effect when you resubscribe to this stream.  <br>
     *        + SDK will automatically select the stream to be published or subscribed based on the settings of both sides.
     */
    virtual void SetRemoteVideoConfig(const char *user_id,const RemoteVideoConfig &remote_video_config) = 0;

    /** 
     * @type api
     * @region Room Management
     * @brief Subscribes to specific remote media streams captured by camera/microphone.  <br>
     *        You can call this API in both automatic subscription mode and manual subscription mode.
     * @param [in] user_id The ID of the remote user who published the target audio/video stream.
     * @param [in] type Media stream type, used for specifying whether to subscribe to the audio stream or the video stream. See MediaStreamType{@link #MediaStreamType}.
     * @notes  <br>
     *        + You must first get the remote stream information through OnUserPublishStream{@link #IRTCRoomEventHandler#OnUserPublishStream} before calling this API to subscribe to streams accordingly.  <br>
     *        + After calling this API, you will be informed of the calling result with OnStreamSubscribed{@link #IRTCRoomEventHandler#OnStreamSubscribed}.  <br>
     *        + Any other exceptions will be included in OnStreamStateChanged{@link #IRTCRoomEventHandler#OnStreamStateChanged}, see ErrorCode{@link #ErrorCode} for the reasons.
     */
    virtual void SubscribeStream(const char *user_id,MediaStreamType type) = 0;

    /** 
     * @type api
     * @region Room Management
     * @brief Unsubscribes from specific remote media streams captured by camera/microphone.  <br>
     *        You can call this API in both automatic subscription mode and manual subscription mode.
     * @param [in] user_id The ID of the remote user who published the target audio/video stream.
     * @param [in] type Media stream type, used for specifying whether to unsubscribe from the audio stream or the video stream. See MediaStreamType{@link #MediaStreamType}.
     * @notes  <br>
     *        + After calling this API, you will be informed of the calling result with OnStreamSubscribed{@link #IRTCRoomEventHandler#OnStreamSubscribed}.  <br>
     *        + Any other exceptions will be included in OnStreamStateChanged{@link #IRTCRoomEventHandler#OnStreamStateChanged}, see ErrorCode{@link #ErrorCode} for the reasons.
     */
    virtual void UnsubscribeStream(const char *user_id,MediaStreamType type) = 0;

    /** 
     * @type api
     * @region Room Management
     * @brief Subscribes to specific screen sharing media stream.  <br>
     *        You can call this API in both automatic subscription mode and manual subscription mode.
     * @param [in] user_id The ID of the remote user who published the target screen audio/video stream.
     * @param [in] type Media stream type, used for specifying whether to subscribe to the audio stream or the video stream. See MediaStreamType{@link #MediaStreamType}.
     * @notes  <br>
     *        + You must first get the remote stream information through OnUserPublishScreen{@link #IRTCRoomEventHandler#OnUserPublishScreen} before calling this API to subscribe to streams accordingly.  <br>
     *        + After calling this API, you will be informed of the calling result with OnStreamSubscribed{@link #IRTCRoomEventHandler#OnStreamSubscribed}.  <br>
     *        + Any other exceptions will be included in OnStreamStateChanged{@link #IRTCRoomEventHandler#OnStreamStateChanged}, see ErrorCode{@link #ErrorCode} for the reasons.
     */
    virtual void SubscribeScreen(const char *user_id,MediaStreamType type) = 0;

    /** 
     * @type api
     * @region Room Management
     * @brief Unsubscribes from specific screen sharing media stream.  <br>
     *        You can call this API in both automatic subscription mode and manual subscription mode.
     * @param [in] user_id The ID of the remote user who published the target screen audio/video stream.
     * @param [in] type Media stream type, used for specifying whether to unsubscribe from the audio stream or the video stream. See MediaStreamType{@link #MediaStreamType}.
     * @notes  <br>
     *        + After calling this API, you will be informed of the calling result with OnStreamSubscribed{@link #IRTCRoomEventHandler#OnStreamSubscribed}.  <br>
     *        + Any other exceptions will be included in OnStreamStateChanged{@link #IRTCRoomEventHandler#OnStreamStateChanged}, see ErrorCode{@link #ErrorCode} for the reasons.
     */
    virtual void UnsubscribeScreen(const char *user_id,MediaStreamType type) = 0;
    /** 
     * @type api
     * @region Multi-room
     * @brief Stop subscribing to the remote audio & video stream in the specified room.   <br>
     *         Turn off the automatic subscription function. When using the manual subscription mode, the user calls this method to stop subscribing to the subscribed audio & video stream. <br>
     *        This method only takes effect when the automatic subscription function is turned off. Users need to set config to turn off the automatic subscription function when joining the room to use the manual subscription mode. <br>
     * @param  [in] user_id Specifies the ID of the user who stopped subscribing to the remote audio & video stream. <br>
     * @param  [in] is_screen Specifies whether the stopped stream is a screen stream. <br>
     * @notes   <br>
     *        + You can call this method to stop the subscription regardless of whether you subscribe to the specified remote audio & video stream. Stop subscription stream_id invalid or unsubscribed, SDK will not do any processing, no negative impact. <br>
     */
    virtual void Unsubscribe(const char* user_id, bool is_screen) = 0;

    /**
     * @hidden
     */
    virtual void EnableSubscribeLocalStream(bool enable) = 0;

    /** 
     * @hidden
     * @deprecated Since 323.1, use OnUserStartAudioCapture instead
     * @type api
     * @region multiroom
     * @brief To turn on/off volume prompts. Off by default. <br>
     *         When the volume prompt is turned on, you will receive a callback of the volume information of the remote user who collected the volume locally or subscribed at the set time interval.
     *         For specific information about the callback. See OnAudioVolumeIndication{@link #IRTCRoomEventHandler#OnAudioVolumeIndication}
     * @param  [in] interval The time interval for receiving the volume prompt callback: <br>
     *                 + ≤ 0: Disable the volume prompt function. <br>
     *                 + > 0: Enable the volume prompt function and set the time interval for receiving the volume prompt callback. In milliseconds. <br>
     *                 It is recommended to set to greater than or equal to 200 milliseconds; when less than 10 milliseconds, the behavior is undefined.
     */
    virtual void SetAudioVolumeIndicationInterval(int interval) = 0;


    /** 
     * @type api
     * @region  multi-room
     * @brief  Adjust the audio playback volume from the remote user
     * @param user_id  The remote user ID of the audio source
     * @param volume Ratio(%) of playback volume to original volume, in the range [0, 400], with overflow protection.  <br>
     *        To ensure the audio quality, we recommend setting the volume within `100`.  <br>
     *               + 0: mute <br>
     *               + 100: original volume <br>
     *               + 400: Up to 4 times the original volume (with overflow protection) <br>
     */
    virtual void SetRemoteAudioPlaybackVolume(const char* user_id, int volume) = 0;

    /** 
     * @hidden
     * @deprecated since 326.1, use PauseAllSubscribedStream/ResumeAllSubscribedStream instead
     * @type api
     * @region Multiroom
     * @brief  To set the reception status for all audio streams from the remote. The default is receive.
     * @param  [in] mute_state Reception status. See: MuteState{@link #MuteState}
     * @notes  This method only affects whether the remote audio stream is received locally, and does not affect the collection and transmission function of the remote audio device.
     */
    virtual void MuteAllRemoteAudio(MuteState mute_state) = 0;

    /** 
     * @type api
     * @region Multi-room
     * @brief Pause receiving remote media streams.
     * @param  [in] media_type Media stream type subscribed to. Refer to PauseResumeControlMediaType{@link #PauseResumeControlMediaType}.
     * @notes   <br>
     *         + Calling this API does not change the capture state and the transmission state of the remote clients. <br>
     *         + Calling this API does not cancel the subscription or change any subscription configuration. <br>
     *         + To resume, call ResumeAllSubscribedStream{@link #IRtcRoom#ResumeAllSubscribedStream}.
     */
    virtual void PauseAllSubscribedStream(PauseResumeControlMediaType media_type) = 0;

    /** 
     * @type api
     * @region  multi-room
     * @brief  Media stream type subscribed to. Refer to
     * @param  [in] media_type Media stream type subscribed to. Refer to PauseResumeControlMediaType{@link #PauseResumeControlMediaType}
     * @notes  <br>
     *         + Calling this API does not change the capture state and the transmission state of the remote clients.
     *         + Calling this API does not change any subscription configuration.
     */
    virtual void ResumeAllSubscribedStream(PauseResumeControlMediaType media_type) = 0;

    /** 
     * @hidden
     * @deprecated since 326.1, use SubscribeUserStream instead
     * @type api
     * @region multiroom
     * @brief  To set the reception status of the audio stream from the remote specified user. The default is receive.
     * @param  [in] uid Specifies the ID of the remote user
     * @param  [in] mute_state Receiving state. See: MuteState{@link #MuteState}
     * @notes  This method only affects whether the remote audio stream is received locally, and does not affect the collection and transmission function of the remote audio device.
     */
    virtual void MuteRemoteAudio(const char* uid, MuteState mute_state) = 0;


#ifndef ByteRTC_AUDIO_ONLY

    /** 
     * @type api
     * @region Multi-room
     * @brief Synchronizes published audio and video.  <br>
     *        When the same user simultaneously uses separate devices to capture and publish audio and video, there is a possiblity that the streams are out of sync due to the network disparity. In this case, you can call this API on the video publisher side and the SDK will automatically line the video stream up according to the timestamp of the audio stream, ensuring that the audio the receiver hears corresponds to the video the receiver watches.
     * @param [in] audio_user_id The ID of audio publisher. You can stop the current A/V synchronization by setting this parameter to null.
     * @notes  <br>
     *        + You can call this API anytime before or after entering the room.  <br>
     *        + The source user IDs of the audio and video stream to be synchronized must be in the same RTC room.  <br>
     *        + When the A/V synchronization state changes, you will receive OnAVSyncStateChange{@link #IRTCRoomEventHandler#OnAVSyncStateChange}.  <br>
     *        + More than one pair of audio and video can be synchronized simultaneously in the same RTC room, but you should note that one single audio source cannot be synchronized with multiple video sources at the same time.  <br>
     *        + If you want to change the audio source, call this API again with a new `audio_user_id`. If you want to change the video source, you need to stop the current synchronization first, then call this API on the new video publisher side.
     */
    virtual void SetMultiDeviceAVSync(const char* audio_user_id) = 0;

    /**
     * @hidden
    */
    virtual void UpdateCloudRending(const char* cloudrenderJsonString) = 0;

    /** 
     * @type api
     * @region Multi-room
     * @brief Create a new task of pushing media streams to CDN and sets the relevant configurations.  <br>
     *        When pushing more than one live streams in the same task, SDK will first mix those streams into one single stream and then push it to CDN.
     * @param [in] task_id Task ID. <br>
     *        You may want to push more than one mixed stream to CDN from the same room. When you do that, use different ID for corresponding tasks; if you will start only one task, use an empty string.
     * @param [in] param Configurations to be set when pushing streams to CDN. See ITranscoderParam{@link #ITranscoderParam}.
     * @param [in] observer Register this observer to receive callbacks from the SDK. See ITranscoderObserver{@link #ITranscoderObserver}.
     * @notes   <br>
     *       + After calling this API, you will be informed of the result and errors during the pushing process via the OnStreamMixingEvent{@link #ITranscoderObserver#OnStreamMixingEvent} callback.
     *       + Call StopLiveTranscoding{@link #IRtcRoom#StopLiveTranscoding} to stop pushing streams to CDN.
     */
    virtual void StartLiveTranscoding(const char* task_id, ITranscoderParam* param, ITranscoderObserver* observer) = 0;

    /** 
     * @type api
     * @region Multi-room
     * @brief Stops pushing media streams to CDN. <br>
     *        For starting pushing media streams to CDN, see StartLiveTranscoding{@link #IRtcRoom#StartLiveTranscoding}.
     * @param [in] task_id Task ID. Specifys which pushing task you want to stop.
     */
    virtual void StopLiveTranscoding(const char* task_id) = 0;

    /** 
     * @type api
     * @region Multi-room
     * @brief Update parameters needed when pushing media streams to CDN.  <br>
     *        After calling StartLiveTranscoding{@link #IRtcRoom#StartLiveTranscoding} to enable the function of pushing streams to CDN, you can call this API to update the relevant configurations.
     * @param [in] task_id Task ID. Specifys of which pushing task you want to update the parameters.
     * @param [in] param Configurations that you want to update. See ITranscoderParam{@link #ITranscoderParam}.
     */
    virtual void UpdateLiveTranscoding(const char* task_id, ITranscoderParam* param) = 0;
    /** 
     * @deprecated  since 336, use SetRemoteStreamVideoCanvas instead
     * @hidden
     * @type api
     * @region Multi-room
     * @brief Sets the view to be used for remote video rendering and the rendering mode. <br>
     *        If you need to unbind the remote video stream from the current view, you can set the canvas to `null`.
     * @notes   <br>
     * @param [in] user_id The ID of the remote user who published the video stream.
     * @param [in] index Video stream type. See StreamIndex{@link #StreamIndex}.
     * @param [in] canvas View information and rendering mode. See VideoCanvas{@link #VideoCanvas}.
     * @return  <br>
     *         + 0: Success <br>
     *         +!: Failure <br>
     * @notes  <br>
     *        + You should bind the stream to a view after entering the room. This setting will remain in effect after you leave the room. <br>
     *        + You can get the remote user ID in OnFirstRemoteVideoFrameRendered{@link #IRTCRoomEventHandler#OnFirstRemoteVideoFrameRendered}.
     */
    virtual void SetRemoteVideoCanvas(const char* user_id, StreamIndex index, const VideoCanvas& canvas) = 0;

    /** 
     * @deprecated Since 325.1
     * @hidden
     * @type api
     * @region Multi-room
     * @brief Remove all remote videos.
     */
    virtual void RemoveAllRemoteVideo() = 0;

    /** 
     * @deprecated Since 325.1
     * @hidden
     * @type api
     * @region Multi-room
     * @brief Remove all remote views shared by the screen.
     */
    virtual void RemoveAllRemoteScreen() = 0;

    /** 
     * @hidden
     * @deprecated since 326.1, use PauseAllSubscribedStream/ResumeAllSubscribedStream instead
     * @type api
     * @region Multi-room
     * @brief Set whether to play all remote video streams
     * @param   [in] muteState Reception status to identify whether to receive all remote audio/video streams. See MuteState{@link #MuteState} <br>
     *        + True: stop playing <br>
     *        + False: start playing <br>
     * @notes This method does not affect the remote video capture and transmission status
     */
    virtual void MuteAllRemoteVideo(MuteState muteState) = 0;


    /** 
     * @hidden
     * @deprecated since 326.1, use SubscribeUserStream instead
     * @type api
     * @region Multi-room
     * @brief Set whether to play the remote video stream
     * @param  [in] userid The remote user ID of the video source
     * @param   [in] muteState Reception status to identify whether to receive the remote audio/video stream. See MuteState{@link #MuteState} <br>
     *        + True: stop playing <br>
     *        + False: turn on playback <br>
     * @notes  This method does not affect the remote video capture and transmission status
     */
    virtual void MuteRemoteVideo(const char* userid, MuteState muteState) = 0;

#endif

    /**
     * @hidden
     */
    virtual void SetCustomUserRole(const char* role) = 0;

    /** 
     * @hidden
     * @deprecated since 326.1, use JoinRoom instead
     * @type api
     * @region multiroom
     * @brief To set the subscription mode.   <br>
     *        The user calls this method to set the subscription mode for audio & video streams in the room. The default is automatic subscription. <br>
     *        To enable the manual subscription function, it is recommended to complete the setting before calling the JoinRoom{@link #IRtcRoom#JoinRoom} method to join the room. <br>
     * @param  [in] audio_mode Subscription mode for audio streams. See SubscribeMode{@link #SubscribeMode}. <br>
     * @param  [in] video_mode Subscription mode for video streams. See SubscribeMode{@link #SubscribeMode}. <br>
     */
    virtual void EnableAutoSubscribe(SubscribeMode audio_mode, SubscribeMode video_mode) = 0;

    /** 
     * @type api
     * @region Multi-room
     * @brief Start relaying media stream across rooms. <br>
     *        After a user joins a room, you can call this method to publish the media stream to multiple rooms that applies to scenarios such as anOnline talent contest and so on.
     * @param [in] configuration Information of the rooms where you want to relay the media stream to. Refer to ForwardStreamConfiguration{@link #ForwardStreamConfiguration} for more inforamtion.
     * @return  <br>
     *        0: Success
     *        <0: Failure  <br>
     * @notes <br>
     *        + Call this method will trigger OnForwardStreamStateChanged{@link #IRTCRoomEventHandler#OnForwardStreamStateChanged}.
     *        + After calling this method, listen the events from each room during the relaying by registering OnForwardStreamEvent{@link #IRTCRoomEventHandler#OnForwardStreamEvent}.
     *        + Once the relaying begins, the other users in the room will receive callback of OnUserJoined{@link #IRTCRoomEventHandler#OnUserJoined} and OnUserPublishStream{@link #IRTCRoomEventHandler#OnUserPublishStream}/OnUserPublishScreen{@link #IRTCRoomEventHandler#OnUserPublishScreen}.
     *        + Call UpdateForwardStreamToRooms{@link #IRtcRoom#UpdateForwardStreamToRooms} to add or remove the target room(s) after calling this method.
     *        + Call StopForwardStreamToRooms{@link #IRtcRoom#StopForwardStreamToRooms} to stop relaying to all rooms after calling this method.
     *        + Call PauseForwardStreamToAllRooms{@link #IRtcRoom#PauseForwardStreamToAllRooms} to pause relaying to all rooms after calling this method.
     */
    virtual int StartForwardStreamToRooms(const ForwardStreamConfiguration& configuration) = 0;

    /** 
     * @type api
     * @region Multi-room
     * @brief Update information of the rooms where you want to relay the media stream to after calling startForwardStreamToRooms{@link #IRtcRoom#StartForwardStreamToRooms}. <br>
     *        Adding and removing rooms by calling this method will trigger OnForwardStreamStateChanged{@link #IRTCRoomEventHandler#OnForwardStreamStateChanged} on the local.
     * @param [in] configuration Information of the rooms where you want to relay the media stream to. Refer to ForwardStreamConfiguration{@link #ForwardStreamConfiguration} for more information.
     * @return  <br>
     *          0: Success
     *          <0: Failure  <br>
     * @notes <br>
     *        + Users in the room which is added by calling this method will receive OnUserJoined{@link #IRTCRoomEventHandler#OnUserJoined} and OnUserPublishStream{@link #IRTCRoomEventHandler#OnUserPublishStream}/OnUserPublishScreen{@link #IRTCRoomEventHandler#OnUserPublishScreen}.
     *        + Users in the room which is removed by calling this method will receive OnUserUnPublishStream{@link #IRTCRoomEventHandler#OnUserUnPublishStream}/OnUserUnPublishScreen{@link #IRTCRoomEventHandler#OnUserUnPublishScreen} and OnUserLeave{@link #IRTCRoomEventHandler#OnUserLeave}.
     */
    virtual int UpdateForwardStreamToRooms(const ForwardStreamConfiguration& configuration) = 0;

    /** 
     * @type api
     * @region Multi-room
     * @brief Call to this method to stop relaying media stream to all rooms after calling startForwardStreamToRooms{@link #IRtcRoom#StartForwardStreamToRooms}. <br>
     * @notes <br>
     *        + Call this method will trigger OnForwardStreamStateChanged{@link #IRTCRoomEventHandler#OnForwardStreamStateChanged}.
     *        + The other users in the room will receive callback of OnUserJoined{@link #IRTCRoomEventHandler#OnUserJoined} and OnUserPublishStream{@link #IRTCRoomEventHandler#OnUserPublishStream}/OnUserPublishScreen{@link #IRTCRoomEventHandler#OnUserPublishScreen} when you stop relaying.
     *        + To stop relaying media stream to specific rooms, call UpdateForwardStreamToRooms{@link #IRtcRoom#UpdateForwardStreamToRooms} instead.
     *        + To resume the relaying in a short time, call PauseForwardStreamToAllRooms{@link #IRtcRoom#PauseForwardStreamToAllRooms} instead and then call ResumeForwardStreamToAllRooms{@link #IRtcRoom#ResumeForwardStreamToAllRooms} to recsume after that.
     */
    virtual void StopForwardStreamToRooms() = 0;

    /** 
     * @type api
     * @region Multi-room
     * @brief Call this method to pause relaying media stream to all rooms after calling startForwardStreamToRooms{@link #IRtcRoom#StartForwardStreamToRooms}. <br>
     *        After that, call ResumeForwardStreamToAllRooms{@link #IRtcRoom#ResumeForwardStreamToAllRooms} to resume.
     * @notes The other users in the room will receive callback of OnUserUnPublishStream{@link #IRTCRoomEventHandler#OnUserUnPublishStream}/OnUserUnPublishScreen{@link #IRTCRoomEventHandler#OnUserUnPublishScreen} and OnUserLeave{@link #IRTCRoomEventHandler#OnUserLeave} when you pause relaying.
     */
    virtual void PauseForwardStreamToAllRooms() = 0;

    /** 
     * @type api
     * @region Multi-room
     * @brief Call this method to resume relaying to all rooms from the pause by calling PauseForwardStreamToAllRooms{@link #IRtcRoom#PauseForwardStreamToAllRooms}.
     * @notes The other users in the room will receive callback of OnUserJoined{@link #IRTCRoomEventHandler#OnUserJoined} and OnUserPublishStream{@link #IRTCRoomEventHandler#OnUserPublishStream}/OnUserPublishScreen{@link #IRTCRoomEventHandler#OnUserPublishScreen} when you resume relaying.
     */
    virtual void ResumeForwardStreamToAllRooms() = 0;

    /** 
     * @type api
     * @brief Start publishing a public media stream.<br>
     *        A public stream refers to a media stream that does not belong to any room or any user. Users within the same `appID` can call StartPlayPublicStream{@link #IRtcEngineLite#StartPlayPublicStream} to subscribe to the public stream regardless the user has joined which room or has not joined any room.
     * @param public_stream_id ID of the pulbic stream
     * @param param Properties of the public stream. Refer to IPublicStreamParam{@link #IPublicStreamParam} for more details.<br>
     *              A public stream can include a bundle of media streams and appears as the designated layout.
     * @return
     *        + 0: Success. And you will be informed by OnPushPublicStreamResult{@link #IRTCRoomEventHandler#OnPushPublicStreamResult}. <br>
     *        + !0: Failure because of invalid parameter or empty parameters.<br>
     * @notes
     *        + Call UpdatePublicStreamParam{@link #IRtcRoom#UpdatePublicStreamParam} to update the properties of the public stream which is published by the same user. Calling this API with the same stream ID repeatedly by the same user can not update the existing public stream.<br>
     *        + If Users with different userID call this API with the same stream ID, the public stream will be updated with the parameters passed in the latest call.<br>
     *        + To publish multiple public streams, call this API with different stream ID respectively.<br>
     *        + To stop publishing the public stream, call StopPushPublicStream{@link #IRtcRoom#StopPushPublicStream}.
     */
    virtual int StartPushPublicStream(const char* public_stream_id, IPublicStreamParam* param) = 0;
    /** 
     * @type api
     * @brief Stop the public stream published by the current user.<br>
     *        Refer to StartPushPublicStream{@link #IRtcRoom#StartPushPublicStream} for details about starting publishing a public stream.
     * @param public_stream_id ID of the public stream<br>
     *                  The public stream must be published by the current user.
     * @return
     *        + 0: Success<br>
     *        + !0: Failure<br>
     */
    virtual int StopPushPublicStream(const char* public_stream_id) = 0;
    /** 
     * @type api
     * @brief Update the properties of the public stream published by the current user.<br>
     *        Refer to StartPushPublicStream{@link #IRtcRoom#StartPushPublicStream} for details about starting publishing a public stream.
     * @param public_stream_id ID of the public stream<br>
     * @param param Properties of the public stream. Refer to IPublicStreamParam{@link #IPublicStreamParam} for more details.<br>
     *              The stream to be updated must be published by the current user.
     * @return
     *        + 0: Success<br>
     *        + !0: Failure<br>
     */
    virtual int UpdatePublicStreamParam(const char* public_stream_id, IPublicStreamParam* param) = 0;

    /** 
     * @type api
     * @region Range Audio
     * @brief Gets range audio instance.
     * @return API call result:  <br>
     *        + IRangeAudio: Success. You will get an IRangeAudio{@link #IRangeAudio} returned from the SDK.  <br>
     *        + nullptr: Failure. The current SDK does not offer range audio function.
     * @notes The first time this API is called must be between you create a room and you actually enter the room.
     */
    virtual IRangeAudio* GetRangeAudio() = 0;

    /** 
     * @type api
     * @region Spatial Audio
     * @brief Gets spatial audio instance.   <br>
     * @return API call result:  <br>
     *        + ISpatialAudio: Success. You will get an ISpatialAudio{@link #ISpatialAudio} returned from the SDK.  <br>
     *        + nullptr: Failure. The current SDK does not offer spatial audio function.
     * @notes   <br>
     *        + The first time this API is called must be between you create a room and you actually enter the room.  <br>
     *        + Only using real dual-channel playback device can you actually enjoy spatial audio effect.  <br>
     *        + Low-end device may have audio stalling issues due to its poor performance, so it is not recommended to enable spatial audio function on such kind of devices. <br>
     *        + SDK currently supports up to 30 users publishing audio with spatial audio enabled at the same time in the same room.
     */
    virtual ISpatialAudio* GetSpatialAudio() = 0;
};

} // namespace bytertc

#endif // BYTE_RTC_ROOM_INTERFACE_H__
